@@ -40,5 +40,7 @@ extern int set_cloexec (int fd);
 extern int set_nonblock (int fd);
 extern char **copy_environ (char **env, ...) __attribute__((__sentinel__));
 extern char *make_temporary_directory (void);
+extern ssize_t full_pread (int fd, void *buf, size_t count, off_t offset);
+extern ssize_t full_pwrite (int fd, const void *buf, size_t count, off_t offset);
 
 #endif /* NBDKIT_UTILS_H */
