@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright (C) 2018-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,31 +40,43 @@
 #include <inttypes.h>
 #include <limits.h>
 #include <errno.h>
+#include <pthread.h>
 
 #include <nbdkit-filter.h>
 
 #include "cleanup.h"
+#include "ispowerof2.h"
 #include "minmax.h"
 #include "rounding.h"
 
 #define BLOCKSIZE_MIN_LIMIT (64U * 1024)
 
-/* As long as we don't have parallel requests, we can reuse a common
- * buffer for alignment purposes; size it to the maximum we allow for
- * minblock. */
-static char bounce[BLOCKSIZE_MIN_LIMIT];
-static unsigned int minblock;
-static unsigned int maxdata;
-static unsigned int maxlen;
-
-/* We are using a common bounce buffer (see above) so we must
- * serialize all requests.
+/* Lock in order to handle overlapping requests safely.
+ *
+ * Grabbed for exclusive access (wrlock) when using the bounce buffer.
+ *
+ * Grabbed for shared access (rdlock) when doing aligned writes.
+ * These can happen in parallel with one another, but must not land in
+ * between the read and write of an unaligned RMW operation.
  */
-static int
-blocksize_thread_model (void)
-{
-  return NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS;
-}
+static pthread_rwlock_t lock = PTHREAD_RWLOCK_INITIALIZER;
+
+/* A single bounce buffer for alignment purposes, guarded by the lock.
+ * Size it to the maximum we allow for minblock.
+ */
+static char bounce[BLOCKSIZE_MIN_LIMIT];
+
+/* Globals set by .config */
+static unsigned int config_minblock;
+static unsigned int config_maxdata;
+static unsigned int config_maxlen;
+
+/* Per-handle values set during .prepare */
+struct blocksize_handle {
+  uint32_t minblock;
+  uint32_t maxdata;
+  uint32_t maxlen;
+};
 
 static int
 blocksize_parse (const char *name, const char *s, unsigned int *v)
@@ -87,56 +99,51 @@ blocksize_parse (const char *name, const char *s, unsigned int *v)
 
 /* Called for each key=value passed on the command line. */
 static int
-blocksize_config (nbdkit_next_config *next, void *nxdata,
+blocksize_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
                   const char *key, const char *value)
 {
 
   if (strcmp (key, "minblock") == 0)
-    return blocksize_parse (key, value, &minblock);
+    return blocksize_parse (key, value, &config_minblock);
   if (strcmp (key, "maxdata") == 0)
-    return blocksize_parse (key, value, &maxdata);
+    return blocksize_parse (key, value, &config_maxdata);
   if (strcmp (key, "maxlen") == 0)
-    return blocksize_parse (key, value, &maxlen);
+    return blocksize_parse (key, value, &config_maxlen);
   return next (nxdata, key, value);
 }
 
 /* Check that limits are sane. */
 static int
-blocksize_config_complete (nbdkit_next_config_complete *next, void *nxdata)
+blocksize_config_complete (nbdkit_next_config_complete *next,
+                           nbdkit_backend *nxdata)
 {
-  if (minblock) {
-    if (minblock & (minblock - 1)) {
+  if (config_minblock) {
+    if (!is_power_of_2 (config_minblock)) {
       nbdkit_error ("minblock must be a power of 2");
       return -1;
     }
-    if (minblock > BLOCKSIZE_MIN_LIMIT) {
+    if (config_minblock > BLOCKSIZE_MIN_LIMIT) {
       nbdkit_error ("minblock must not exceed %u", BLOCKSIZE_MIN_LIMIT);
       return -1;
     }
   }
-  else
-    minblock = 1;
 
-  if (maxdata) {
-    if (maxdata & (minblock - 1)) {
-      nbdkit_error ("maxdata must be a multiple of %u", minblock);
+  if (config_maxdata && config_minblock) {
+    if (config_maxdata & (config_minblock - 1)) {
+      nbdkit_error ("maxdata must be a multiple of %u", config_minblock);
       return -1;
     }
   }
-  else if (maxlen)
-    maxdata = MIN (maxlen, 64 * 1024 * 1024);
-  else
-    maxdata = 64 * 1024 * 1024;
 
-  if (maxlen) {
-    if (maxlen & (minblock - 1)) {
-      nbdkit_error ("maxlen must be a multiple of %u", minblock);
+  if (config_maxlen && config_minblock) {
+    if (config_maxlen & (config_minblock - 1)) {
+      nbdkit_error ("maxlen must be a multiple of %u", config_minblock);
       return -1;
     }
   }
-  else
-    maxlen = -minblock;
 
+  nbdkit_debug ("configured values minblock=%u maxdata=%u maxlen=%u",
+                config_minblock, config_maxdata, config_maxlen);
   return next (nxdata);
 }
 
@@ -145,44 +152,118 @@ blocksize_config_complete (nbdkit_next_config_complete *next, void *nxdata)
   "maxdata=<SIZE>       Maximum size for read/write (default 64M).\n" \
   "maxlen=<SIZE>        Maximum size for trim/zero (default 4G-minblock)."
 
-/* Round size down to avoid issues at end of file. */
-static int64_t
-blocksize_get_size (struct nbdkit_next_ops *next_ops, void *nxdata,
-                    void *handle)
+static void *
+blocksize_open (nbdkit_next_open *next, nbdkit_context *nxdata,
+                int readonly, const char *exportname, int is_tls)
 {
-  int64_t size = next_ops->get_size (nxdata);
+  struct blocksize_handle *h;
 
-  if (size == -1)
-    return -1;
-  return ROUND_DOWN (size, minblock);
+  if (next (nxdata, readonly, exportname) == -1)
+    return NULL;
+
+  h = malloc (sizeof *h);
+  if (h == NULL) {
+    nbdkit_error ("malloc: %m");
+    return NULL;
+  }
+
+  h->minblock = config_minblock;
+  h->maxdata = config_maxdata;
+  h->maxlen = config_maxlen;
+  return h;
 }
 
 static int
-blocksize_can_multi_conn (struct nbdkit_next_ops *next_ops, void *nxdata,
-                          void *handle)
+blocksize_prepare (nbdkit_next *next, void *handle,
+                   int readonly)
 {
-  /* Although we are serializing all requests anyway so this likely
-   * doesn't make a difference, return false because the bounce buffer
-   * is not consistent for flush.
+  struct blocksize_handle *h = handle;
+  uint32_t minimum, preferred, maximum;
+
+  /* Here, minimum and maximum will clamp per-handle defaults not set
+   * by globals in .config; preferred has no impact until .block_size.
    */
+  if (next->block_size (next, &minimum, &preferred, &maximum) == -1)
+    return -1;
+
+  h->minblock = MAX (MAX (h->minblock, 1), minimum);
+
+  if (h->maxdata == 0) {
+    if (h->maxlen)
+      h->maxdata = MIN (h->maxlen, 64 * 1024 * 1024);
+    else
+      h->maxdata = 64 * 1024 * 1024;
+  }
+  if (maximum)
+    h->maxdata = MIN (h->maxdata, maximum);
+  h->maxdata = ROUND_DOWN (h->maxdata, h->minblock);
+
+  if (h->maxlen == 0)
+    h->maxlen = -h->minblock;
+  else
+    h->maxlen = ROUND_DOWN (h->maxlen, h->minblock);
+
+  nbdkit_debug ("handle values minblock=%u maxdata=%u maxlen=%u",
+                h->minblock, h->maxdata, h->maxlen);
+  return 0;
+}
+
+/* Round size down to avoid issues at end of file. */
+static int64_t
+blocksize_get_size (nbdkit_next *next,
+                    void *handle)
+{
+  struct blocksize_handle *h = handle;
+  int64_t size = next->get_size (next);
+
+  if (size == -1)
+    return -1;
+  return ROUND_DOWN (size, h->minblock);
+}
+
+/* Block size constraints.
+ *
+ * This filter is a little unusual because it allows clients to send a
+ * wider range of request sizes than the underlying plugin allows.
+ * Therefore we advertise the widest possible minimum and maximum
+ * block size to clients.
+ */
+static int
+blocksize_block_size (nbdkit_next *next, void *handle,
+                      uint32_t *minimum, uint32_t *preferred, uint32_t *maximum)
+{
+  struct blocksize_handle *h = handle;
+
+  /* Here we only need preferred; see also blocksize_prepare. */
+  if (next->block_size (next, minimum, preferred, maximum) == -1)
+    return -1;
+
+  *preferred = MAX (MAX (*preferred, 4096), h->minblock);
+
+  *minimum = 1;
+  *maximum = 0xffffffff;
+
+  nbdkit_debug ("advertising min=%" PRIu32 " pref=%" PRIu32 " max=%" PRIu32,
+                *minimum, *preferred, *maximum);
   return 0;
 }
 
 static int
-blocksize_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
+blocksize_pread (nbdkit_next *next,
                  void *handle, void *b, uint32_t count, uint64_t offs,
                  uint32_t flags, int *err)
 {
+  struct blocksize_handle *h = handle;
   char *buf = b;
   uint32_t keep;
   uint32_t drop;
 
   /* Unaligned head */
-  if (offs & (minblock - 1)) {
-    drop = offs & (minblock - 1);
-    keep = MIN (minblock - drop, count);
-    if (next_ops->pread (nxdata, bounce, minblock, offs - drop, flags,
-                         err) == -1)
+  if (offs & (h->minblock - 1)) {
+    ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&lock);
+    drop = offs & (h->minblock - 1);
+    keep = MIN (h->minblock - drop, count);
+    if (next->pread (next, bounce, h->minblock, offs - drop, flags, err) == -1)
       return -1;
     memcpy (buf, bounce + drop, keep);
     buf += keep;
@@ -191,9 +272,9 @@ blocksize_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
   }
 
   /* Aligned body */
-  while (count >= minblock) {
-    keep = MIN (maxdata, ROUND_DOWN (count, minblock));
-    if (next_ops->pread (nxdata, buf, keep, offs, flags, err) == -1)
+  while (count >= h->minblock) {
+    keep = MIN (h->maxdata, ROUND_DOWN (count, h->minblock));
+    if (next->pread (next, buf, keep, offs, flags, err) == -1)
       return -1;
     buf += keep;
     offs += keep;
@@ -202,7 +283,8 @@ blocksize_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
 
   /* Unaligned tail */
   if (count) {
-    if (next_ops->pread (nxdata, bounce, minblock, offs, flags, err) == -1)
+    ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&lock);
+    if (next->pread (next, bounce, h->minblock, offs, flags, err) == -1)
       return -1;
     memcpy (buf, bounce, count);
   }
@@ -211,30 +293,31 @@ blocksize_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
 }
 
 static int
-blocksize_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
+blocksize_pwrite (nbdkit_next *next,
                   void *handle, const void *b, uint32_t count, uint64_t offs,
                   uint32_t flags, int *err)
 {
+  struct blocksize_handle *h = handle;
   const char *buf = b;
   uint32_t keep;
   uint32_t drop;
   bool need_flush = false;
 
   if ((flags & NBDKIT_FLAG_FUA) &&
-      next_ops->can_fua (nxdata) == NBDKIT_FUA_EMULATE) {
+      next->can_fua (next) == NBDKIT_FUA_EMULATE) {
     flags &= ~NBDKIT_FLAG_FUA;
     need_flush = true;
   }
 
   /* Unaligned head */
-  if (offs & (minblock - 1)) {
-    drop = offs & (minblock - 1);
-    keep = MIN (minblock - drop, count);
-    if (next_ops->pread (nxdata, bounce, minblock, offs - drop, 0, err) == -1)
+  if (offs & (h->minblock - 1)) {
+    ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&lock);
+    drop = offs & (h->minblock - 1);
+    keep = MIN (h->minblock - drop, count);
+    if (next->pread (next, bounce, h->minblock, offs - drop, 0, err) == -1)
       return -1;
     memcpy (bounce + drop, buf, keep);
-    if (next_ops->pwrite (nxdata, bounce, minblock, offs - drop, flags,
-                          err) == -1)
+    if (next->pwrite (next, bounce, h->minblock, offs - drop, flags, err) == -1)
       return -1;
     buf += keep;
     offs += keep;
@@ -242,9 +325,10 @@ blocksize_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
   }
 
   /* Aligned body */
-  while (count >= minblock) {
-    keep = MIN (maxdata, ROUND_DOWN (count, minblock));
-    if (next_ops->pwrite (nxdata, buf, keep, offs, flags, err) == -1)
+  while (count >= h->minblock) {
+    ACQUIRE_RDLOCK_FOR_CURRENT_SCOPE (&lock);
+    keep = MIN (h->maxdata, ROUND_DOWN (count, h->minblock));
+    if (next->pwrite (next, buf, keep, offs, flags, err) == -1)
       return -1;
     buf += keep;
     offs += keep;
@@ -253,61 +337,65 @@ blocksize_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
 
   /* Unaligned tail */
   if (count) {
-    if (next_ops->pread (nxdata, bounce, minblock, offs, 0, err) == -1)
+    ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&lock);
+    if (next->pread (next, bounce, h->minblock, offs, 0, err) == -1)
       return -1;
     memcpy (bounce, buf, count);
-    if (next_ops->pwrite (nxdata, bounce, minblock, offs, flags, err) == -1)
+    if (next->pwrite (next, bounce, h->minblock, offs, flags, err) == -1)
       return -1;
   }
 
   if (need_flush)
-    return next_ops->flush (nxdata, 0, err);
+    return next->flush (next, 0, err);
   return 0;
 }
 
 static int
-blocksize_trim (struct nbdkit_next_ops *next_ops, void *nxdata,
+blocksize_trim (nbdkit_next *next,
                 void *handle, uint32_t count, uint64_t offs, uint32_t flags,
                 int *err)
 {
+  struct blocksize_handle *h = handle;
   uint32_t keep;
   bool need_flush = false;
 
   if ((flags & NBDKIT_FLAG_FUA) &&
-      next_ops->can_fua (nxdata) == NBDKIT_FUA_EMULATE) {
+      next->can_fua (next) == NBDKIT_FUA_EMULATE) {
     flags &= ~NBDKIT_FLAG_FUA;
     need_flush = true;
   }
 
   /* Ignore unaligned head */
-  if (offs & (minblock - 1)) {
-    keep = MIN (minblock - (offs & (minblock - 1)), count);
+  if (offs & (h->minblock - 1)) {
+    keep = MIN (h->minblock - (offs & (h->minblock - 1)), count);
     offs += keep;
     count -= keep;
   }
 
   /* Ignore unaligned tail */
-  count = ROUND_DOWN (count, minblock);
+  count = ROUND_DOWN (count, h->minblock);
 
   /* Aligned body */
   while (count) {
-    keep = MIN (maxlen, count);
-    if (next_ops->trim (nxdata, keep, offs, flags, err) == -1)
+    ACQUIRE_RDLOCK_FOR_CURRENT_SCOPE (&lock);
+    keep = MIN (h->maxlen, count);
+    if (next->trim (next, keep, offs, flags, err) == -1)
       return -1;
     offs += keep;
     count -= keep;
   }
 
   if (need_flush)
-    return next_ops->flush (nxdata, 0, err);
+    return next->flush (next, 0, err);
   return 0;
 }
 
 static int
-blocksize_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+blocksize_zero (nbdkit_next *next,
                 void *handle, uint32_t count, uint64_t offs, uint32_t flags,
                 int *err)
 {
+  struct blocksize_handle *h = handle;
   uint32_t keep;
   uint32_t drop;
   bool need_flush = false;
@@ -318,36 +406,38 @@ blocksize_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
      * calls; it's easier to just declare that anything that can't be
      * done in one call to the plugin is not fast.
      */
-    if ((offs | count) & (minblock - 1) || count > maxlen) {
+    if ((offs | count) & (h->minblock - 1) || count > h->maxlen) {
       *err = ENOTSUP;
       return -1;
     }
   }
 
   if ((flags & NBDKIT_FLAG_FUA) &&
-      next_ops->can_fua (nxdata) == NBDKIT_FUA_EMULATE) {
+      next->can_fua (next) == NBDKIT_FUA_EMULATE) {
     flags &= ~NBDKIT_FLAG_FUA;
     need_flush = true;
   }
 
   /* Unaligned head */
-  if (offs & (minblock - 1)) {
-    drop = offs & (minblock - 1);
-    keep = MIN (minblock - drop, count);
-    if (next_ops->pread (nxdata, bounce, minblock, offs - drop, 0, err) == -1)
+  if (offs & (h->minblock - 1)) {
+    ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&lock);
+    drop = offs & (h->minblock - 1);
+    keep = MIN (h->minblock - drop, count);
+    if (next->pread (next, bounce, h->minblock, offs - drop, 0, err) == -1)
       return -1;
     memset (bounce + drop, 0, keep);
-    if (next_ops->pwrite (nxdata, bounce, minblock, offs - drop,
-                          flags & ~NBDKIT_FLAG_MAY_TRIM, err) == -1)
+    if (next->pwrite (next, bounce, h->minblock, offs - drop,
+                      flags & ~NBDKIT_FLAG_MAY_TRIM, err) == -1)
       return -1;
     offs += keep;
     count -= keep;
   }
 
   /* Aligned body */
-  while (count >= minblock) {
-    keep = MIN (maxlen, ROUND_DOWN (count, minblock));
-    if (next_ops->zero (nxdata, keep, offs, flags, err) == -1)
+  while (count >= h->minblock) {
+    ACQUIRE_RDLOCK_FOR_CURRENT_SCOPE (&lock);
+    keep = MIN (h->maxlen, ROUND_DOWN (count, h->minblock));
+    if (next->zero (next, keep, offs, flags, err) == -1)
       return -1;
     offs += keep;
     count -= keep;
@@ -355,21 +445,22 @@ blocksize_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
 
   /* Unaligned tail */
   if (count) {
-    if (next_ops->pread (nxdata, bounce, minblock, offs, 0, err) == -1)
+    ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&lock);
+    if (next->pread (next, bounce, h->minblock, offs, 0, err) == -1)
       return -1;
     memset (bounce, 0, count);
-    if (next_ops->pwrite (nxdata, bounce, minblock, offs,
-                          flags & ~NBDKIT_FLAG_MAY_TRIM, err) == -1)
+    if (next->pwrite (next, bounce, h->minblock, offs,
+                      flags & ~NBDKIT_FLAG_MAY_TRIM, err) == -1)
       return -1;
   }
 
   if (need_flush)
-    return next_ops->flush (nxdata, 0, err);
+    return next->flush (next, 0, err);
   return 0;
 }
 
 static int
-blocksize_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
+blocksize_extents (nbdkit_next *next,
                    void *handle, uint32_t count, uint64_t offset,
                    uint32_t flags, struct nbdkit_extents *extents, int *err)
 {
@@ -379,21 +470,22 @@ blocksize_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
    * fine to return less than the full count as long as we're making
    * progress.
    */
+  struct blocksize_handle *h = handle;
   CLEANUP_EXTENTS_FREE struct nbdkit_extents *extents2 = NULL;
   size_t i;
   struct nbdkit_extent e;
 
-  extents2 = nbdkit_extents_new (ROUND_DOWN (offset, minblock),
-                                 ROUND_UP (offset + count, minblock));
+  extents2 = nbdkit_extents_new (ROUND_DOWN (offset, h->minblock),
+                                 ROUND_UP (offset + count, h->minblock));
   if (extents2 == NULL) {
     *err = errno;
     return -1;
   }
 
-  if (nbdkit_extents_aligned (next_ops, nxdata,
-                              MIN (ROUND_UP (count, minblock), maxlen),
-                              ROUND_DOWN (offset, minblock),
-                              flags, minblock, extents2, err) == -1)
+  if (nbdkit_extents_aligned (next, MIN (ROUND_UP (count, h->minblock),
+                                         h->maxlen),
+                              ROUND_DOWN (offset, h->minblock), flags,
+                              h->minblock, extents2, err) == -1)
     return -1;
 
   for (i = 0; i < nbdkit_extents_count (extents2); ++i) {
@@ -407,25 +499,26 @@ blocksize_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
 }
 
 static int
-blocksize_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
+blocksize_cache (nbdkit_next *next,
                  void *handle, uint32_t count, uint64_t offs, uint32_t flags,
                  int *err)
 {
+  struct blocksize_handle *h = handle;
   uint32_t limit;
   uint64_t remaining = count; /* Rounding out could exceed 32 bits */
 
   /* Unaligned head */
-  limit = offs & (minblock - 1);
+  limit = offs & (h->minblock - 1);
   remaining += limit;
   offs -= limit;
 
   /* Unaligned tail */
-  remaining = ROUND_UP (remaining, minblock);
+  remaining = ROUND_UP (remaining, h->minblock);
 
   /* Aligned body */
   while (remaining) {
-    limit = MIN (maxdata, remaining);
-    if (next_ops->cache (nxdata, limit, offs, flags, err) == -1)
+    limit = MIN (h->maxdata, remaining);
+    if (next->cache (next, limit, offs, flags, err) == -1)
       return -1;
     offs += limit;
     remaining -= limit;
@@ -437,12 +530,14 @@ blocksize_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
 static struct nbdkit_filter filter = {
   .name              = "blocksize",
   .longname          = "nbdkit blocksize filter",
-  .thread_model      = blocksize_thread_model,
   .config            = blocksize_config,
   .config_complete   = blocksize_config_complete,
   .config_help       = blocksize_config_help,
+  .open              = blocksize_open,
+  .prepare           = blocksize_prepare,
+  .close             = free,
   .get_size          = blocksize_get_size,
-  .can_multi_conn    = blocksize_can_multi_conn,
+  .block_size        = blocksize_block_size,
   .pread             = blocksize_pread,
   .pwrite            = blocksize_pwrite,
   .trim              = blocksize_trim,
