@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright (C) 2018-2021 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,6 +40,7 @@
 #include <inttypes.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <errno.h>
 #include <assert.h>
 #include <sys/types.h>
@@ -62,6 +63,7 @@
 #include "blk.h"
 #include "reclaim.h"
 #include "isaligned.h"
+#include "ispowerof2.h"
 #include "minmax.h"
 #include "rounding.h"
 
@@ -70,20 +72,16 @@
  */
 static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
 
-unsigned blksize;
+unsigned blksize;            /* actual block size (picked by blk.c) */
+unsigned min_block_size = 65536;
 enum cache_mode cache_mode = CACHE_MODE_WRITEBACK;
 int64_t max_size = -1;
 unsigned hi_thresh = 95, lo_thresh = 80;
-bool cache_on_read = false;
+enum cor_mode cor_mode = COR_OFF;
+const char *cor_path;
 
-static int cache_flush (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle, uint32_t flags, int *err);
-
-static void
-cache_load (void)
-{
-  if (blk_init () == -1)
-    exit (EXIT_FAILURE);
-}
+static int cache_flush (nbdkit_next *next, void *handle, uint32_t flags,
+                        int *err);
 
 static void
 cache_unload (void)
@@ -92,7 +90,7 @@ cache_unload (void)
 }
 
 static int
-cache_config (nbdkit_next_config *next, void *nxdata,
+cache_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
               const char *key, const char *value)
 {
   if (strcmp (key, "cache") == 0) {
@@ -114,6 +112,19 @@ cache_config (nbdkit_next_config *next, void *nxdata,
       return -1;
     }
   }
+  else if (strcmp (key, "cache-min-block-size") == 0) {
+    int64_t r;
+
+    r = nbdkit_parse_size (value);
+    if (r == -1)
+      return -1;
+    if (r < 4096 || !is_power_of_2 (r) || r > UINT_MAX) {
+      nbdkit_error ("cache-min-block-size is not a power of 2, or is too small or too large");
+      return -1;
+    }
+    min_block_size = r;
+    return 0;
+  }
 #ifdef HAVE_CACHE_RECLAIM
   else if (strcmp (key, "cache-max-size") == 0) {
     int64_t r;
@@ -160,12 +171,16 @@ cache_config (nbdkit_next_config *next, void *nxdata,
   }
 #endif /* !HAVE_CACHE_RECLAIM */
   else if (strcmp (key, "cache-on-read") == 0) {
-    int r;
-
-    r = nbdkit_parse_bool (value);
-    if (r == -1)
-      return -1;
-    cache_on_read = r;
+    if (value[0] == '/') {
+      cor_path = value;
+      cor_mode = COR_PATH;
+    }
+    else {
+      int r = nbdkit_parse_bool (value);
+      if (r == -1)
+        return -1;
+      cor_mode = r ? COR_ON : COR_OFF;
+    }
     return 0;
   }
   else {
@@ -176,7 +191,7 @@ cache_config (nbdkit_next_config *next, void *nxdata,
 #define cache_config_help_common \
   "cache=MODE                Set cache MODE, one of writeback (default),\n" \
   "                          writethrough, or unsafe.\n" \
-  "cache-on-read=BOOL        Set to true to cache on reads (default false).\n"
+  "cache-on-read=BOOL|/PATH  Set to true to cache on reads (default false).\n"
 #ifndef HAVE_CACHE_RECLAIM
 #define cache_config_help cache_config_help_common
 #else
@@ -186,8 +201,21 @@ cache_config (nbdkit_next_config *next, void *nxdata,
   "cache-low-threshold=PCT   Percentage of max size where reclaim ends.\n"
 #endif
 
+/* Decide if cache-on-read is currently on or off. */
+bool
+cache_on_read (void)
+{
+  switch (cor_mode) {
+  case COR_ON: return true;
+  case COR_OFF: return false;
+  case COR_PATH: return access (cor_path, F_OK) == 0;
+  default: abort ();
+  }
+}
+
 static int
-cache_config_complete (nbdkit_next_config_complete *next, void *nxdata)
+cache_config_complete (nbdkit_next_config_complete *next,
+                       nbdkit_backend *nxdata)
 {
   /* If cache-max-size was set then check the thresholds. */
   if (max_size != -1) {
@@ -201,22 +229,28 @@ cache_config_complete (nbdkit_next_config_complete *next, void *nxdata)
   return next (nxdata);
 }
 
-/* Get the file size; round it down to cache granularity before
- * setting cache size.
- */
+static int
+cache_get_ready (int thread_model)
+{
+  if (blk_init () == -1)
+    return -1;
+
+  return 0;
+}
+
+/* Get the file size, set the cache size. */
 static int64_t
-cache_get_size (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_get_size (nbdkit_next *next,
                 void *handle)
 {
   int64_t size;
   int r;
 
-  size = next_ops->get_size (nxdata);
+  size = next->get_size (next);
   if (size == -1)
     return -1;
 
   nbdkit_debug ("cache: underlying file size: %" PRIi64, size);
-  size = ROUND_DOWN (size, blksize);
 
   ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
   r = blk_set_size (size);
@@ -226,16 +260,38 @@ cache_get_size (struct nbdkit_next_ops *next_ops, void *nxdata,
   return size;
 }
 
-/* Force an early call to cache_get_size, consequently truncating the
- * cache to the correct size.
+/* Block size constraints. */
+static int
+cache_block_size (nbdkit_next *next, void *handle,
+                  uint32_t *minimum, uint32_t *preferred, uint32_t *maximum)
+{
+  if (next->block_size (next, minimum, preferred, maximum) == -1)
+    return -1;
+
+  if (*minimum == 0) {         /* No constraints set by the plugin. */
+    *minimum = 1;
+    *preferred = blksize;
+    *maximum = 0xffffffff;
+  }
+  else {
+    if (*maximum >= blksize)
+      *preferred = MAX (*preferred, blksize);
+  }
+
+  return 0;
+}
+
+/* Force an early call to cache_get_size because we have to set the
+ * backing file size and bitmap size before any other read or write
+ * calls.
  */
 static int
-cache_prepare (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_prepare (nbdkit_next *next,
                void *handle, int readonly)
 {
   int64_t r;
 
-  r = cache_get_size (next_ops, nxdata, handle);
+  r = cache_get_size (next, handle);
   if (r < 0)
     return -1;
   return 0;
@@ -243,14 +299,14 @@ cache_prepare (struct nbdkit_next_ops *next_ops, void *nxdata,
 
 /* Override the plugin's .can_cache, because we are caching here instead */
 static int
-cache_can_cache (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle)
+cache_can_cache (nbdkit_next *next, void *handle)
 {
   return NBDKIT_CACHE_NATIVE;
 }
 
 /* Override the plugin's .can_fast_zero, because our .zero is not fast */
 static int
-cache_can_fast_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_can_fast_zero (nbdkit_next *next,
                      void *handle)
 {
   /* It is better to advertise support even when we always reject fast
@@ -259,14 +315,61 @@ cache_can_fast_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
   return 1;
 }
 
+/* Override the plugin's .can_flush, if we are cache=unsafe */
+static int
+cache_can_flush (nbdkit_next *next,
+                 void *handle)
+{
+  if (cache_mode == CACHE_MODE_UNSAFE)
+    return 1;
+  return next->can_flush (next);
+}
+
+
+/* Override the plugin's .can_fua, if we are cache=unsafe */
+static int
+cache_can_fua (nbdkit_next *next,
+               void *handle)
+{
+  if (cache_mode == CACHE_MODE_UNSAFE)
+    return NBDKIT_FUA_NATIVE;
+  return next->can_fua (next);
+}
+
+/* Override the plugin's .can_multi_conn, if we are not cache=writethrough */
+static int
+cache_can_multi_conn (nbdkit_next *next,
+                      void *handle)
+{
+  /* For CACHE_MODE_UNSAFE, we always advertise a no-op flush because
+   * our local cache access is consistent between connections, and we
+   * don't care about persisting the data to the underlying plugin.
+   *
+   * For CACHE_MODE_WRITEBACK, things are more subtle: we only write
+   * to the plugin during NBD_CMD_FLUSH, at which point that one
+   * connection writes back ALL cached blocks regardless of which
+   * connection originally wrote them, so a client can be assured that
+   * blocks from all connections have reached the plugin's permanent
+   * storage with only one connection having to send a flush.
+   *
+   * But for CACHE_MODE_WRITETHROUGH, we are at the mercy of the
+   * plugin; data written by connection A is not guaranteed to be made
+   * persistent by a flush from connection B unless the plugin itself
+   * supports multi-conn.
+   */
+  if (cache_mode != CACHE_MODE_WRITETHROUGH)
+    return 1;
+  return next->can_multi_conn (next);
+}
+
 /* Read data. */
 static int
-cache_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_pread (nbdkit_next *next,
              void *handle, void *buf, uint32_t count, uint64_t offset,
              uint32_t flags, int *err)
 {
   CLEANUP_FREE uint8_t *block = NULL;
-  uint64_t blknum, blkoffs;
+  uint64_t blknum, blkoffs, nrblocks;
   int r;
 
   assert (!flags);
@@ -288,7 +391,7 @@ cache_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
 
     assert (block);
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, block, err);
+    r = blk_read (next, blknum, block, err);
     if (r == -1)
       return -1;
 
@@ -301,29 +404,24 @@ cache_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
   }
 
   /* Aligned body */
-  /* XXX This breaks up large read requests into smaller ones, which
-   * is a problem for plugins which have a large, fixed per-request
-   * overhead (hello, curl).  We should try to keep large requests
-   * together as much as possible, but that requires us to be much
-   * smarter here.
-   */
-  while (count >= blksize) {
+  nrblocks = count / blksize;
+  if (nrblocks > 0) {
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, buf, err);
+    r = blk_read_multiple (next, blknum, nrblocks, buf, err);
     if (r == -1)
       return -1;
 
-    buf += blksize;
-    count -= blksize;
-    offset += blksize;
-    blknum++;
+    buf += nrblocks * blksize;
+    count -= nrblocks * blksize;
+    offset += nrblocks * blksize;
+    blknum += nrblocks;
   }
 
   /* Unaligned tail */
   if (count) {
     assert (block);
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, block, err);
+    r = blk_read (next, blknum, block, err);
     if (r == -1)
       return -1;
 
@@ -335,7 +433,7 @@ cache_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
 
 /* Write data. */
 static int
-cache_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_pwrite (nbdkit_next *next,
               void *handle, const void *buf, uint32_t count, uint64_t offset,
               uint32_t flags, int *err)
 {
@@ -354,7 +452,8 @@ cache_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
   }
 
   if ((flags & NBDKIT_FLAG_FUA) &&
-      next_ops->can_fua (nxdata) == NBDKIT_FUA_EMULATE) {
+      (cache_mode == CACHE_MODE_UNSAFE ||
+       next->can_fua (next) == NBDKIT_FUA_EMULATE)) {
     flags &= ~NBDKIT_FLAG_FUA;
     need_flush = true;
   }
@@ -371,10 +470,10 @@ cache_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
      */
     assert (block);
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, block, err);
+    r = blk_read (next, blknum, block, err);
     if (r != -1) {
       memcpy (&block[blkoffs], buf, n);
-      r = blk_write (next_ops, nxdata, blknum, block, flags, err);
+      r = blk_write (next, blknum, block, flags, err);
     }
     if (r == -1)
       return -1;
@@ -388,7 +487,7 @@ cache_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
   /* Aligned body */
   while (count >= blksize) {
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_write (next_ops, nxdata, blknum, buf, flags, err);
+    r = blk_write (next, blknum, buf, flags, err);
     if (r == -1)
       return -1;
 
@@ -402,23 +501,23 @@ cache_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
   if (count) {
     assert (block);
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, block, err);
+    r = blk_read (next, blknum, block, err);
     if (r != -1) {
       memcpy (block, buf, count);
-      r = blk_write (next_ops, nxdata, blknum, block, flags, err);
+      r = blk_write (next, blknum, block, flags, err);
     }
     if (r == -1)
       return -1;
   }
 
   if (need_flush)
-    return cache_flush (next_ops, nxdata, handle, 0, err);
+    return cache_flush (next, handle, 0, err);
   return 0;
 }
 
 /* Zero data. */
 static int
-cache_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_zero (nbdkit_next *next,
             void *handle, uint32_t count, uint64_t offset, uint32_t flags,
             int *err)
 {
@@ -427,7 +526,7 @@ cache_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
   int r;
   bool need_flush = false;
 
-  /* We are purposefully avoiding next_ops->zero, so a zero request is
+  /* We are purposefully avoiding next->zero, so a zero request is
    * never faster than plain writes.
    */
   if (flags & NBDKIT_FLAG_FAST_ZERO) {
@@ -444,7 +543,8 @@ cache_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
 
   flags &= ~NBDKIT_FLAG_MAY_TRIM;
   if ((flags & NBDKIT_FLAG_FUA) &&
-      next_ops->can_fua (nxdata) == NBDKIT_FUA_EMULATE) {
+      (cache_mode == CACHE_MODE_UNSAFE ||
+       next->can_fua (next) == NBDKIT_FUA_EMULATE)) {
     flags &= ~NBDKIT_FLAG_FUA;
     need_flush = true;
   }
@@ -460,10 +560,10 @@ cache_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
      * Hold the lock over the whole operation.
      */
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, block, err);
+    r = blk_read (next, blknum, block, err);
     if (r != -1) {
       memset (&block[blkoffs], 0, n);
-      r = blk_write (next_ops, nxdata, blknum, block, flags, err);
+      r = blk_write (next, blknum, block, flags, err);
     }
     if (r == -1)
       return -1;
@@ -477,9 +577,9 @@ cache_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
   if (count >= blksize)
     memset (block, 0, blksize);
   while (count >=blksize) {
-    /* Intentional that we do not use next_ops->zero */
+    /* Intentional that we do not use next->zero */
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_write (next_ops, nxdata, blknum, block, flags, err);
+    r = blk_write (next, blknum, block, flags, err);
     if (r == -1)
       return -1;
 
@@ -491,17 +591,17 @@ cache_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
   /* Unaligned tail */
   if (count) {
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_read (next_ops, nxdata, blknum, block, err);
+    r = blk_read (next, blknum, block, err);
     if (r != -1) {
-      memset (&block[count], 0, blksize - count);
-      r = blk_write (next_ops, nxdata, blknum, block, flags, err);
+      memset (block, 0, count);
+      r = blk_write (next, blknum, block, flags, err);
     }
     if (r == -1)
       return -1;
   }
 
   if (need_flush)
-    return cache_flush (next_ops, nxdata, handle, 0, err);
+    return cache_flush (next, handle, 0, err);
   return 0;
 }
 
@@ -510,19 +610,18 @@ struct flush_data {
   uint8_t *block;               /* bounce buffer */
   unsigned errors;              /* count of errors seen */
   int first_errno;              /* first errno seen */
-  struct nbdkit_next_ops *next_ops;
-  void *nxdata;
+  nbdkit_next *next;
 };
 
 static int flush_dirty_block (uint64_t blknum, void *);
 
 static int
-cache_flush (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
+cache_flush (nbdkit_next *next, void *handle,
              uint32_t flags, int *err)
 {
   CLEANUP_FREE uint8_t *block = NULL;
   struct flush_data data =
-    { .errors = 0, .first_errno = 0, .next_ops = next_ops, .nxdata = nxdata };
+    { .errors = 0, .first_errno = 0, .next = next };
   int tmp;
 
   if (cache_mode == CACHE_MODE_UNSAFE)
@@ -550,8 +649,7 @@ cache_flush (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
   }
 
   /* Now issue a flush request to the underlying storage. */
-  if (next_ops->flush (nxdata, 0,
-                       data.errors ? &tmp : &data.first_errno) == -1)
+  if (next->flush (next, 0, data.errors ? &tmp : &data.first_errno) == -1)
     data.errors++;
 
   if (data.errors > 0) {
@@ -570,10 +668,10 @@ flush_dirty_block (uint64_t blknum, void *datav)
   /* Perform a read + writethrough which will read from the
    * cache and write it through to the underlying storage.
    */
-  if (blk_read (data->next_ops, data->nxdata, blknum, data->block,
+  if (blk_read (data->next, blknum, data->block,
                 data->errors ? &tmp : &data->first_errno) == -1)
     goto err;
-  if (blk_writethrough (data->next_ops, data->nxdata, blknum, data->block, 0,
+  if (blk_writethrough (data->next, blknum, data->block, 0,
                         data->errors ? &tmp : &data->first_errno) == -1)
     goto err;
 
@@ -587,7 +685,7 @@ flush_dirty_block (uint64_t blknum, void *datav)
 
 /* Cache data. */
 static int
-cache_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
+cache_cache (nbdkit_next *next,
              void *handle, uint32_t count, uint64_t offset,
              uint32_t flags, int *err)
 {
@@ -617,7 +715,7 @@ cache_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
   /* Aligned body */
   while (remaining) {
     ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-    r = blk_cache (next_ops, nxdata, blknum, block, err);
+    r = blk_cache (next, blknum, block, err);
     if (r == -1)
       return -1;
 
@@ -632,15 +730,19 @@ cache_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
 static struct nbdkit_filter filter = {
   .name              = "cache",
   .longname          = "nbdkit caching filter",
-  .load              = cache_load,
   .unload            = cache_unload,
   .config            = cache_config,
   .config_complete   = cache_config_complete,
   .config_help       = cache_config_help,
+  .get_ready         = cache_get_ready,
   .prepare           = cache_prepare,
   .get_size          = cache_get_size,
+  .block_size        = cache_block_size,
   .can_cache         = cache_can_cache,
   .can_fast_zero     = cache_can_fast_zero,
+  .can_flush         = cache_can_flush,
+  .can_fua           = cache_can_fua,
+  .can_multi_conn    = cache_can_multi_conn,
   .pread             = cache_pread,
   .pwrite            = cache_pwrite,
   .zero              = cache_zero,
