@@ -39,26 +39,32 @@
 #include <stdbool.h>
 #include <errno.h>
 #include <limits.h>
+#include <time.h>
 
 #include <nbdkit-filter.h>
 
-static int delay_read_ms = 0;   /* read delay (milliseconds) */
-static int delay_write_ms = 0;  /* write delay (milliseconds) */
-static int delay_zero_ms = 0;   /* zero delay (milliseconds) */
-static int delay_trim_ms = 0;   /* trim delay (milliseconds) */
-static int delay_extents_ms = 0;/* extents delay (milliseconds) */
-static int delay_cache_ms = 0;  /* cache delay (milliseconds) */
+static unsigned delay_read_ms = 0;   /* read delay (milliseconds) */
+static unsigned delay_write_ms = 0;  /* write delay (milliseconds) */
+static unsigned delay_zero_ms = 0;   /* zero delay (milliseconds) */
+static unsigned delay_trim_ms = 0;   /* trim delay (milliseconds) */
+static unsigned delay_extents_ms = 0;/* extents delay (milliseconds) */
+static unsigned delay_cache_ms = 0;  /* cache delay (milliseconds) */
+static unsigned delay_open_ms = 0;   /* open delay (milliseconds) */
+static unsigned delay_close_ms = 0;  /* close delay (milliseconds) */
+
 static int delay_fast_zero = 1; /* whether delaying zero includes fast zero */
 
 static int
-parse_delay (const char *key, const char *value)
+parse_delay (const char *key, const char *value, unsigned *r)
 {
   size_t len = strlen (value);
-  int r;
 
   if (len > 2 && strcmp (&value[len-2], "ms") == 0) {
-    if (sscanf (value, "%d", &r) == 1 && r >= 0)
-      return r;
+    /* We have to use sscanf here instead of nbdkit_parse_unsigned
+     * because that function will reject the "ms" suffix.
+     */
+    if (sscanf (value, "%u", r) == 1)
+      return 0;
     else {
       nbdkit_error ("cannot parse %s in milliseconds parameter: %s",
                     key, value);
@@ -66,24 +72,19 @@ parse_delay (const char *key, const char *value)
     }
   }
   else {
-    if (sscanf (value, "%d", &r) == 1 && r >= 0) {
-      if (r * 1000LL > INT_MAX) {
-        nbdkit_error ("seconds parameter %s is too large: %s",
-                      key, value);
-        return -1;
-      }
-      return r * 1000;
-    }
-    else {
-      nbdkit_error ("cannot parse %s in seconds parameter: %s",
-                    key, value);
+    if (nbdkit_parse_unsigned (key, value, r) == -1)
+      return -1;
+    if (*r * UINT64_C(1000) > UINT_MAX) {
+      nbdkit_error ("seconds parameter %s is too large: %s", key, value);
       return -1;
     }
+    *r *= 1000;
+    return 0;
   }
 }
 
 static int
-delay (int ms, int *err)
+delay (unsigned ms, int *err)
 {
   if (ms > 0 && nbdkit_nanosleep (ms / 1000, (ms % 1000) * 1000000) == -1) {
     *err = errno;
@@ -128,22 +129,26 @@ cache_delay (int *err)
   return delay (delay_cache_ms, err);
 }
 
+static int
+open_delay (int *err)
+{
+  return delay (delay_open_ms, err);
+}
+
 /* Called for each key=value passed on the command line. */
 static int
-delay_config (nbdkit_next_config *next, void *nxdata,
+delay_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
               const char *key, const char *value)
 {
   if (strcmp (key, "rdelay") == 0 ||
       strcmp (key, "delay-read") == 0 ||
       strcmp (key, "delay-reads") == 0) {
-    delay_read_ms = parse_delay (key, value);
-    if (delay_read_ms == -1)
+    if (parse_delay (key, value, &delay_read_ms) == -1)
       return -1;
     return 0;
   }
   else if (strcmp (key, "wdelay") == 0) {
-    delay_write_ms = parse_delay (key, value);
-    if (delay_write_ms == -1)
+    if (parse_delay (key, value, &delay_write_ms) == -1)
       return -1;
     /* Historically wdelay set all write-related delays. */
     delay_zero_ms = delay_trim_ms = delay_write_ms;
@@ -151,15 +156,13 @@ delay_config (nbdkit_next_config *next, void *nxdata,
   }
   else if (strcmp (key, "delay-write") == 0 ||
            strcmp (key, "delay-writes") == 0) {
-    delay_write_ms = parse_delay (key, value);
-    if (delay_write_ms == -1)
+    if (parse_delay (key, value, &delay_write_ms) == -1)
       return -1;
     return 0;
   }
   else if (strcmp (key, "delay-zero") == 0 ||
            strcmp (key, "delay-zeroes") == 0) {
-    delay_zero_ms = parse_delay (key, value);
-    if (delay_zero_ms == -1)
+    if (parse_delay (key, value, &delay_zero_ms) == -1)
       return -1;
     return 0;
   }
@@ -167,21 +170,18 @@ delay_config (nbdkit_next_config *next, void *nxdata,
            strcmp (key, "delay-trims") == 0 ||
            strcmp (key, "delay-discard") == 0 ||
            strcmp (key, "delay-discards") == 0) {
-    delay_trim_ms = parse_delay (key, value);
-    if (delay_trim_ms == -1)
+    if (parse_delay (key, value, &delay_trim_ms) == -1)
       return -1;
     return 0;
   }
   else if (strcmp (key, "delay-extent") == 0 ||
            strcmp (key, "delay-extents") == 0) {
-    delay_extents_ms = parse_delay (key, value);
-    if (delay_extents_ms == -1)
+    if (parse_delay (key, value, &delay_extents_ms) == -1)
       return -1;
     return 0;
   }
   else if (strcmp (key, "delay-cache") == 0) {
-    delay_cache_ms = parse_delay (key, value);
-    if (delay_cache_ms == -1)
+    if (parse_delay (key, value, &delay_cache_ms) == -1)
       return -1;
     return 0;
   }
@@ -191,6 +191,16 @@ delay_config (nbdkit_next_config *next, void *nxdata,
       return -1;
     return 0;
   }
+  else if (strcmp (key, "delay-open") == 0) {
+    if (parse_delay (key, value, &delay_open_ms) == -1)
+      return -1;
+    return 0;
+  }
+  else if (strcmp (key, "delay-close") == 0) {
+    if (parse_delay (key, value, &delay_close_ms) == -1)
+      return -1;
+    return 0;
+  }
   else
     return next (nxdata, key, value);
 }
@@ -204,45 +214,98 @@ delay_config (nbdkit_next_config *next, void *nxdata,
   "delay-extents=<NN>[ms]         Extents delay in seconds/milliseconds.\n" \
   "delay-cache=<NN>[ms]           Cache delay in seconds/milliseconds.\n" \
   "wdelay=<NN>[ms]                Write, zero and trim delay in secs/msecs.\n" \
-  "delay-fast-zero=<BOOL>         Delay fast zero requests (default true).\n"
+  "delay-fast-zero=<BOOL>         Delay fast zero requests (default true).\n" \
+  "delay-open=<NN>[ms]            Open delay in seconds/milliseconds.\n" \
+  "delay-close=<NN>[ms]           Close delay in seconds/milliseconds."
 
 /* Override the plugin's .can_fast_zero if needed */
 static int
-delay_can_fast_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_can_fast_zero (nbdkit_next *next,
                      void *handle)
 {
   /* Advertise if we are handling fast zero requests locally */
   if (delay_zero_ms && !delay_fast_zero)
     return 1;
-  return next_ops->can_fast_zero (nxdata);
+  return next->can_fast_zero (next);
+}
+
+/* Open connection. */
+static void *
+delay_open (nbdkit_next_open *next, nbdkit_context *nxdata,
+            int readonly, const char *exportname, int is_tls)
+{
+  int err;
+
+  if (open_delay (&err) == -1) {
+    errno = err;
+    nbdkit_error ("delay: %m");
+    return NULL;
+  }
+
+  if (next (nxdata, readonly, exportname) == -1)
+    return NULL;
+
+  return NBDKIT_HANDLE_NOT_NEEDED;
+}
+
+/* Close connection.
+ *
+ * We cannot call nbdkit_nanosleep here because the socket may have
+ * been closed and that function will abort and return immediately.
+ * However we want to force a sleep (even if the server is shutting
+ * down) so use regular nanosleep instead.
+ *
+ * We cannot use the .close callback because that happens after the
+ * socket has closed, thus not delaying the client.  By using
+ * .finalize we can delay well-behaved clients (those that use
+ * NBD_CMD_DISC).  We cannot delay clients that drop the connection.
+ */
+static int
+delay_finalize (nbdkit_next *next, void *handle)
+{
+  const unsigned ms = delay_close_ms;
+
+  if (ms > 0) {
+    struct timespec ts;
+
+    ts.tv_sec = ms / 1000;
+    ts.tv_nsec = (ms % 1000) * 1000000;
+    /* If nanosleep fails we don't really want to interrupt the chain
+     * of finalize calls through the other filters, so ignore any
+     * error here.
+     */
+    nanosleep (&ts, NULL);
+  }
+
+  return next->finalize (next);
 }
 
 /* Read data. */
 static int
-delay_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_pread (nbdkit_next *next,
              void *handle, void *buf, uint32_t count, uint64_t offset,
              uint32_t flags, int *err)
 {
   if (read_delay (err) == -1)
     return -1;
-  return next_ops->pread (nxdata, buf, count, offset, flags, err);
+  return next->pread (next, buf, count, offset, flags, err);
 }
 
 /* Write data. */
 static int
-delay_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_pwrite (nbdkit_next *next,
               void *handle,
               const void *buf, uint32_t count, uint64_t offset, uint32_t flags,
               int *err)
 {
   if (write_delay (err) == -1)
     return -1;
-  return next_ops->pwrite (nxdata, buf, count, offset, flags, err);
+  return next->pwrite (next, buf, count, offset, flags, err);
 }
 
 /* Zero data. */
 static int
-delay_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_zero (nbdkit_next *next,
             void *handle, uint32_t count, uint64_t offset, uint32_t flags,
             int *err)
 {
@@ -252,40 +315,40 @@ delay_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
   }
   if (zero_delay (err) == -1)
     return -1;
-  return next_ops->zero (nxdata, count, offset, flags, err);
+  return next->zero (next, count, offset, flags, err);
 }
 
 /* Trim data. */
 static int
-delay_trim (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_trim (nbdkit_next *next,
             void *handle, uint32_t count, uint64_t offset,
             uint32_t flags, int *err)
 {
   if (trim_delay (err) == -1)
     return -1;
-  return next_ops->trim (nxdata, count, offset, flags, err);
+  return next->trim (next, count, offset, flags, err);
 }
 
 /* Extents. */
 static int
-delay_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_extents (nbdkit_next *next,
                void *handle, uint32_t count, uint64_t offset, uint32_t flags,
                struct nbdkit_extents *extents, int *err)
 {
   if (extents_delay (err) == -1)
     return -1;
-  return next_ops->extents (nxdata, count, offset, flags, extents, err);
+  return next->extents (next, count, offset, flags, extents, err);
 }
 
 /* Cache. */
 static int
-delay_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
+delay_cache (nbdkit_next *next,
              void *handle, uint32_t count, uint64_t offset, uint32_t flags,
              int *err)
 {
   if (cache_delay (err) == -1)
     return -1;
-  return next_ops->cache (nxdata, count, offset, flags, err);
+  return next->cache (next, count, offset, flags, err);
 }
 
 static struct nbdkit_filter filter = {
@@ -294,6 +357,8 @@ static struct nbdkit_filter filter = {
   .config            = delay_config,
   .config_help       = delay_config_help,
   .can_fast_zero     = delay_can_fast_zero,
+  .open              = delay_open,
+  .finalize          = delay_finalize,
   .pread             = delay_pread,
   .pwrite            = delay_pwrite,
   .zero              = delay_zero,
