@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2017-2020 Red Hat Inc.
+ * Copyright (C) 2017-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -62,7 +62,7 @@ ext2_load (void)
 }
 
 static int
-ext2_config (nbdkit_next_config *next, void *nxdata,
+ext2_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
              const char *key, const char *value)
 {
   if (strcmp (key, "ext2file") == 0) {
@@ -78,7 +78,7 @@ ext2_config (nbdkit_next_config *next, void *nxdata,
 }
 
 static int
-ext2_config_complete (nbdkit_next_config_complete *next, void *nxdata)
+ext2_config_complete (nbdkit_next_config_complete *next, nbdkit_backend *nxdata)
 {
   if (file == NULL) {
     nbdkit_error ("you must supply ext2file=<FILE> parameter "
@@ -106,12 +106,12 @@ struct handle {
   ext2_filsys fs;               /* Filesystem handle. */
   ext2_ino_t ino;               /* Inode of open file. */
   ext2_file_t file;             /* File handle. */
-  struct nbdkit_next next;      /* "name" parameter to ext2fs_open. */
+  nbdkit_next *next;            /* "name" parameter to ext2fs_open. */
 };
 
 /* Export list. */
 static int
-ext2_list_exports (nbdkit_next_list_exports *next, void *nxdata,
+ext2_list_exports (nbdkit_next_list_exports *next, nbdkit_backend *nxdata,
                    int readonly, int is_tls, struct nbdkit_exports *exports)
 {
   /* If we are honoring export names, the default export "" won't
@@ -133,7 +133,7 @@ ext2_list_exports (nbdkit_next_list_exports *next, void *nxdata,
 
 /* Default export. */
 static const char *
-ext2_default_export (nbdkit_next_default_export *next, void *nxdata,
+ext2_default_export (nbdkit_next_default_export *next, nbdkit_backend *nxdata,
                      int readonly, int is_tls)
 {
   /* If we are honoring exports, "" will fail (even if we resolve to
@@ -152,7 +152,7 @@ ext2_default_export (nbdkit_next_default_export *next, void *nxdata,
 
 /* Create the per-connection handle. */
 static void *
-ext2_open (nbdkit_next_open *next, void *nxdata,
+ext2_open (nbdkit_next_open *next, nbdkit_context *nxdata,
            int readonly, const char *exportname, int is_tls)
 {
   struct handle *h;
@@ -185,8 +185,7 @@ ext2_open (nbdkit_next_open *next, void *nxdata,
 }
 
 static int
-ext2_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
-              int readonly)
+ext2_prepare (nbdkit_next *next, void *handle, int readonly)
 {
   struct handle *h = handle;
   errcode_t err;
@@ -202,10 +201,10 @@ ext2_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
 #ifdef EXT2_FLAG_64BITS
   fs_flags |= EXT2_FLAG_64BITS;
 #endif
-  r = next_ops->get_size (nxdata);
+  r = next->get_size (next);
   if (r == -1)
     return -1;
-  r = next_ops->can_write (nxdata);
+  r = next->can_write (next);
   if (r == -1)
     return -1;
   if (r == 0)
@@ -214,9 +213,8 @@ ext2_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
   if (!readonly)
     fs_flags |= EXT2_FLAG_RW;
 
-  h->next.next_ops = next_ops;
-  h->next.nxdata = nxdata;
-  name = nbdkit_io_encode (&h->next);
+  h->next = next;
+  name = nbdkit_io_encode (next);
   if (!name) {
     nbdkit_error ("nbdkit_io_encode: %m");
     return -1;
@@ -293,18 +291,73 @@ ext2_close (void *handle)
 }
 
 static int
-ext2_can_fua (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle)
+ext2_can_fua (nbdkit_next *next, void *handle)
 {
   return NBDKIT_FUA_NATIVE;
 }
 
 static int
-ext2_can_cache (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle)
+ext2_can_cache (nbdkit_next *next, void *handle)
 {
   /* Let nbdkit call pread to populate the file system cache. */
   return NBDKIT_CACHE_EMULATE;
 }
 
+static int
+ext2_can_multi_conn (nbdkit_next *next, void *handle)
+{
+  /* Since we do not permit parallel connections, it does not matter
+   * what we advertise here, and we could just as easily inherit the
+   * plugin's .can_multi_conn.  But realistically, if we adjust
+   * .thread_model, we cannot advertise support unless .flush is
+   * consistent, and that would require inspecting the ext2 source
+   * code, so for now, we hard-code a safe answer.
+   */
+  return 0;
+}
+
+static int
+ext2_can_flush (nbdkit_next *next, void *handle)
+{
+  /* Regardless of the underlying plugin, we handle flush at the level
+   * of the filesystem.  However, we also need to cache the underlying
+   * plugin ability, since ext2 wants to flush the filesystem into
+   * permanent storage when possible.
+   */
+  if (next->can_flush (next) == -1)
+    return -1;
+  return 1;
+}
+
+/* XXX It seems as if we should be able to support trim and zero, if
+ * the ext2fs API were to ever add something like ext2fs_file_fallocate.
+ */
+static int
+ext2_can_zero (nbdkit_next *next, void *handle)
+{
+  /* For now, tell nbdkit to call .pwrite instead of any optimization.
+   * However, we also want to cache the underlying plugin support - even
+   * though we don't implement .zero, the file system wants to know if
+   * it can use next->zero() during io_zeroout.
+   */
+  if (next->can_zero (next) == -1)
+    return -1;
+  return NBDKIT_ZERO_EMULATE;
+}
+
+static int
+ext2_can_trim (nbdkit_next *next, void *handle)
+{
+  /* For now, tell nbdkit to never call .trim.  However, we also want
+   * to cache the underlying plugin support - even though we don't
+   * implement .trim, the file system wants to know if it can use
+   * next->trim() during io_discard.
+   */
+  if (next->can_trim (next) == -1)
+    return -1;
+  return 0;
+}
+
 /* It might be possible to relax this, but it's complicated.
  *
  * It's desirable for ‘nbdkit -r’ to behave the same way as
@@ -324,13 +377,12 @@ static int ext2_thread_model (void)
 
 /* Description. */
 static const char *
-ext2_export_description (struct nbdkit_next_ops *next_ops, void *nxdata,
-                         void *handle)
+ext2_export_description (nbdkit_next *next, void *handle)
 {
   struct handle *h = handle;
   const char *fname = file ?: h->exportname;
   const char *slash = fname[0] == '/' ? "" : "/";
-  const char *base = next_ops->export_description (nxdata);
+  const char *base = next->export_description (next);
 
   if (!base)
     return NULL;
@@ -340,7 +392,7 @@ ext2_export_description (struct nbdkit_next_ops *next_ops, void *nxdata,
 
 /* Get the disk size. */
 static int64_t
-ext2_get_size (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle)
+ext2_get_size (nbdkit_next *next, void *handle)
 {
   struct handle *h = handle;
   errcode_t err;
@@ -356,7 +408,7 @@ ext2_get_size (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle)
 
 /* Read data. */
 static int
-ext2_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
+ext2_pread (nbdkit_next *next,
             void *handle, void *buf, uint32_t count, uint64_t offset,
             uint32_t flags, int *errp)
 {
@@ -393,7 +445,7 @@ ext2_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
 
 /* Write data to the file. */
 static int
-ext2_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
+ext2_pwrite (nbdkit_next *next,
              void *handle, const void *buf, uint32_t count, uint64_t offset,
              uint32_t flags, int *errp)
 {
@@ -434,7 +486,7 @@ ext2_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
 }
 
 static int
-ext2_flush (struct nbdkit_next_ops *next_ops, void *nxdata,
+ext2_flush (nbdkit_next *next,
             void *handle, uint32_t flags, int *errp)
 {
   struct handle *h = handle;
@@ -450,11 +502,6 @@ ext2_flush (struct nbdkit_next_ops *next_ops, void *nxdata,
   return 0;
 }
 
-/* XXX It seems as if we should be able to support trim and zero, if
- * we could work out how those are implemented in the ext2fs API which
- * is very obscure.
- */
-
 static struct nbdkit_filter filter = {
   .name               = "ext2",
   .longname           = "nbdkit ext2 filter",
@@ -470,6 +517,10 @@ static struct nbdkit_filter filter = {
   .close              = ext2_close,
   .can_fua            = ext2_can_fua,
   .can_cache          = ext2_can_cache,
+  .can_multi_conn     = ext2_can_multi_conn,
+  .can_zero           = ext2_can_zero,
+  .can_trim           = ext2_can_trim,
+  .can_flush          = ext2_can_flush,
   .export_description = ext2_export_description,
   .get_size           = ext2_get_size,
   .pread              = ext2_pread,
