@@ -54,6 +54,8 @@
 
 #ifdef HAVE_LINUX_VM_SOCKETS_H
 #include <linux/vm_sockets.h>
+#elif HAVE_SYS_VSOCK_H
+#include <sys/vsock.h>
 #endif
 
 #include <nbdkit-filter.h>
@@ -62,7 +64,7 @@
 #include "cleanup.h"
 
 /* -D ip.rules=1 to enable debugging of rules and rule matching. */
-int ip_debug_rules;
+NBDKIT_DLL_PUBLIC int ip_debug_rules;
 
 struct rule {
   struct rule *next;
@@ -84,10 +86,12 @@ static struct rule *deny_rules, *deny_rules_last;
 static void
 print_rule (const char *name, const struct rule *rule, const char *suffix)
 {
+#if defined(INET_NTOP)
   union {
     char addr4[INET_ADDRSTRLEN];
     char addr6[INET6_ADDRSTRLEN];
   } u;
+#endif
 
   switch (rule->type) {
   case ANY:
@@ -99,6 +103,7 @@ print_rule (const char *name, const struct rule *rule, const char *suffix)
   case ANYV6:
     nbdkit_debug ("%s=anyipv6%s", name, suffix);
     break;
+#if defined(INET_NTOP)
   case IPV4:
     inet_ntop (AF_INET, &rule->u.ipv4, u.addr4, sizeof u.addr4);
     nbdkit_debug ("%s=ipv4:%s/%u%s", name, u.addr4, rule->prefixlen, suffix);
@@ -107,6 +112,7 @@ print_rule (const char *name, const struct rule *rule, const char *suffix)
     inet_ntop (AF_INET6, &rule->u.ipv6, u.addr6, sizeof u.addr6);
     nbdkit_debug ("%s=ipv6:[%s]/%u%s", name, u.addr6, rule->prefixlen, suffix);
     break;
+#endif
 
   case ANYUNIX:
     nbdkit_debug ("%s=anyunix%s", name, suffix);
@@ -186,11 +192,13 @@ parse_ip_address (const char *value, size_t n, struct rule *ret)
   strncpy (addr, value, n);
   addr[n] = '\0';
 
+#ifdef INET_PTON
   if (inet_pton (AF_INET, addr, &ret->u.ipv4) == 1)
     return IPV4;
 
   if (inet_pton (AF_INET6, addr, &ret->u.ipv6) == 1)
     return IPV6;
+#endif
 
   return 0;
 }
@@ -399,7 +407,7 @@ parse_rules (const char *paramname,
 }
 
 static int
-ip_config (nbdkit_next_config *next, void *nxdata,
+ip_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
            const char *key, const char *value)
 {
   /* For convenience we permit multiple allow and deny parameters,
@@ -420,7 +428,7 @@ ip_config (nbdkit_next_config *next, void *nxdata,
 }
 
 static int
-ip_config_complete (nbdkit_next_config_complete *next, void *nxdata)
+ip_config_complete (nbdkit_next_config_complete *next, nbdkit_backend *nxdata)
 {
   if (ip_debug_rules) {
     print_rules ("ip: parsed allow", allow_rules);
@@ -471,7 +479,7 @@ matches_rule (const struct rule *rule,
   const struct sockaddr_in *sin;
   uint32_t cin, rin, mask;
   const struct sockaddr_in6 *sin6;
-#ifdef AF_VSOCK
+#if defined(AF_VSOCK) && defined(VMADDR_CID_ANY)
   const struct sockaddr_vm *svm;
 #endif
 
@@ -514,7 +522,7 @@ matches_rule (const struct rule *rule,
     if (family != AF_UNIX) return false;
     return nbdkit_peer_gid () == rule->u.id;
 
-#ifdef AF_VSOCK
+#if defined(AF_VSOCK) && defined(VMADDR_CID_ANY)
   case ANYVSOCK:
     return family == AF_VSOCK;
 
@@ -568,7 +576,7 @@ check_if_allowed (const struct sockaddr *addr)
    * sockets, see the manual.
    */
   if (family != AF_INET && family != AF_INET6 && family != AF_UNIX
-#ifdef AF_VSOCK
+#if defined(AF_VSOCK) && defined(VMADDR_CID_ANY)
       && family != AF_VSOCK
 #endif
       )
@@ -586,7 +594,8 @@ check_if_allowed (const struct sockaddr *addr)
 }
 
 static int
-ip_preconnect (nbdkit_next_preconnect *next, void *nxdata, int readonly)
+ip_preconnect (nbdkit_next_preconnect *next, nbdkit_backend *nxdata,
+               int readonly)
 {
   struct sockaddr_storage addr;
   socklen_t addrlen = sizeof addr;
