@@ -68,10 +68,9 @@ static char *rate_file = NULL;
 
 /* Bucket capacity controls the burst rate.  It is expressed as the
  * length of time in "rate-equivalent seconds" that the client can
- * burst for after a period of inactivity.  This could be adjustable
- * in future.
+ * burst for after a period of inactivity.
  */
-#define BUCKET_CAPACITY 2.0
+static double bucket_capacity = 2.0 /* seconds */;
 
 /* Global read and write buckets. */
 static struct bucket read_bucket;
@@ -97,7 +96,7 @@ rate_unload (void)
 
 /* Called for each key=value passed on the command line. */
 static int
-rate_config (nbdkit_next_config *next, void *nxdata,
+rate_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
              const char *key, const char *value)
 {
   if (strcmp (key, "rate") == 0) {
@@ -142,18 +141,25 @@ rate_config (nbdkit_next_config *next, void *nxdata,
       return -1;
     return 0;
   }
+  else if (strcmp (key, "burstiness") == 0) {
+    if (sscanf (value, "%lg", &bucket_capacity) != 1) {
+      nbdkit_error ("burstiness must be a floating point number (seconds)");
+      return -1;
+    }
+    return 0;
+  }
   else
     return next (nxdata, key, value);
 }
 
 static int
-rate_get_ready (nbdkit_next_get_ready *next, void *nxdata, int thread_model)
+rate_get_ready (int thread_model)
 {
   /* Initialize the global buckets. */
-  bucket_init (&read_bucket, rate, BUCKET_CAPACITY);
-  bucket_init (&write_bucket, rate, BUCKET_CAPACITY);
+  bucket_init (&read_bucket, rate, bucket_capacity);
+  bucket_init (&write_bucket, rate, bucket_capacity);
 
-  return next (nxdata);
+  return 0;
 }
 
 #define rate_config_help \
@@ -164,7 +170,7 @@ rate_get_ready (nbdkit_next_get_ready *next, void *nxdata, int thread_model)
 
 /* Create the per-connection handle. */
 static void *
-rate_open (nbdkit_next_open *next, void *nxdata,
+rate_open (nbdkit_next_open *next, nbdkit_context *nxdata,
            int readonly, const char *exportname, int is_tls)
 {
   struct rate_handle *h;
@@ -178,8 +184,8 @@ rate_open (nbdkit_next_open *next, void *nxdata,
     return NULL;
   }
 
-  bucket_init (&h->read_bucket, connection_rate, BUCKET_CAPACITY);
-  bucket_init (&h->write_bucket, connection_rate, BUCKET_CAPACITY);
+  bucket_init (&h->read_bucket, connection_rate, bucket_capacity);
+  bucket_init (&h->write_bucket, connection_rate, bucket_capacity);
   pthread_mutex_init (&h->read_bucket_lock, NULL);
   pthread_mutex_init (&h->write_bucket_lock, NULL);
 
@@ -197,6 +203,7 @@ rate_close (void *handle)
   free (h);
 }
 
+/* Check rate files and adjust rates if necessary. */
 static void
 maybe_adjust (const char *file, struct bucket *bucket, pthread_mutex_t *lock)
 {
@@ -245,9 +252,11 @@ maybe_adjust (const char *file, struct bucket *bucket, pthread_mutex_t *lock)
                   old_rate, new_rate);
 }
 
+/* Wait for sufficient tokens to be available in the bucket. */
 static inline int
-maybe_sleep (struct bucket *bucket, pthread_mutex_t *lock, uint32_t count,
-             int *err)
+maybe_sleep (struct bucket *bucket, pthread_mutex_t *lock,
+             const char *bucket_name,
+             uint32_t count, int *err)
 {
   struct timespec ts;
   uint64_t bits;
@@ -262,7 +271,7 @@ maybe_sleep (struct bucket *bucket, pthread_mutex_t *lock, uint32_t count,
     /* Run the token bucket algorithm. */
     {
       ACQUIRE_LOCK_FOR_CURRENT_SCOPE (lock);
-      bits = bucket_run (bucket, bits, &ts);
+      bits = bucket_run (bucket, bucket_name, bits, &ts);
     }
 
     if (bits > 0 && nbdkit_nanosleep (ts.tv_sec, ts.tv_nsec) == -1) {
@@ -275,25 +284,27 @@ maybe_sleep (struct bucket *bucket, pthread_mutex_t *lock, uint32_t count,
 
 /* Read data. */
 static int
-rate_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
+rate_pread (nbdkit_next *next,
             void *handle, void *buf, uint32_t count, uint64_t offset,
             uint32_t flags, int *err)
 {
   struct rate_handle *h = handle;
 
   maybe_adjust (rate_file, &read_bucket, &read_bucket_lock);
-  if (maybe_sleep (&read_bucket, &read_bucket_lock, count, err))
+  if (maybe_sleep (&read_bucket, &read_bucket_lock,
+                   "read (global limit)", count, err))
     return -1;
   maybe_adjust (connection_rate_file, &h->read_bucket, &h->read_bucket_lock);
-  if (maybe_sleep (&h->read_bucket, &h->read_bucket_lock, count, err))
+  if (maybe_sleep (&h->read_bucket, &h->read_bucket_lock,
+                   "read (connection limit)", count, err))
     return -1;
 
-  return next_ops->pread (nxdata, buf, count, offset, flags, err);
+  return next->pread (next, buf, count, offset, flags, err);
 }
 
 /* Write data. */
 static int
-rate_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
+rate_pwrite (nbdkit_next *next,
              void *handle,
              const void *buf, uint32_t count, uint64_t offset, uint32_t flags,
              int *err)
@@ -301,13 +312,15 @@ rate_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
   struct rate_handle *h = handle;
 
   maybe_adjust (rate_file, &write_bucket, &write_bucket_lock);
-  if (maybe_sleep (&write_bucket, &write_bucket_lock, count, err))
+  if (maybe_sleep (&write_bucket, &write_bucket_lock,
+                   "write (global limit)", count, err))
     return -1;
   maybe_adjust (connection_rate_file, &h->write_bucket, &h->write_bucket_lock);
-  if (maybe_sleep (&h->write_bucket, &h->write_bucket_lock, count, err))
+  if (maybe_sleep (&h->write_bucket, &h->write_bucket_lock,
+                   "write (connection limit)", count, err))
     return -1;
 
-  return next_ops->pwrite (nxdata, buf, count, offset, flags, err);
+  return next->pwrite (next, buf, count, offset, flags, err);
 }
 
 static struct nbdkit_filter filter = {
