@@ -53,13 +53,19 @@ static bool unlink_on_exit = false;
 
 /* C compiler and flags. */
 static const char *cc = CC;
-static const char *cflags = CFLAGS;
+static const char *cflags =
+  CFLAGS
+  " -fPIC -shared"
+#ifdef __APPLE__
+  " -Wl,-undefined,dynamic_lookup"
+#endif
+  ;
 static const char *extra_cflags;
 
 /* List of parameters for the subplugin. */
 struct key_value { const char *key, *value; };
-DEFINE_VECTOR_TYPE(string_vector, struct key_value);
-static string_vector params = empty_vector;
+DEFINE_VECTOR_TYPE(params_vector, struct key_value);
+static params_vector params = empty_vector;
 
 /* The subplugin. */
 static void *dl;
@@ -168,7 +174,7 @@ cc_config (const char *key, const char *value)
       /* Anything else is saved for the subplugin. */
       struct key_value kv = { .key = key, .value = value };
 
-      if (string_vector_append (&params, kv) == -1) {
+      if (params_vector_append (&params, kv) == -1) {
         nbdkit_error ("realloc: %m");
         return -1;
       }
@@ -243,7 +249,7 @@ cc_config_complete (void)
   dlerror ();
   *(void **) (&plugin_init) = dlsym (dl, "plugin_init");
   if ((error = dlerror ()) != NULL) {
-    nbdkit_error ("no plugin_init symbol found: %s", dlerror ());
+    nbdkit_error ("no plugin_init symbol found: %s", error);
     return -1;
   }
   if (!plugin_init) {
@@ -292,12 +298,12 @@ cc_config_complete (void)
   if (subplugin.load)
     subplugin.load ();
   if (subplugin.config) {
-    for (i = 0; i < params.size; ++i) {
+    for (i = 0; i < params.len; ++i) {
       if (subplugin.config (params.ptr[i].key, params.ptr[i].value) == -1)
         return -1;
     }
   }
-  else if (params.size > 0) {
+  else if (params.len > 0) {
     /* Just print the first one in the error message. */
     nbdkit_error ("unknown parameter: %s", params.ptr[0].key);
     return -1;
@@ -391,6 +397,18 @@ cc_get_size (void *handle)
   return subplugin.get_size (handle);
 }
 
+static int
+cc_block_size (void *handle,
+               uint32_t *minimum, uint32_t *preferred, uint32_t *maximum)
+{
+  if (subplugin.block_size)
+    return subplugin.block_size (handle, minimum, preferred, maximum);
+  else {
+    *minimum = *preferred = *maximum = 0;
+    return 0;
+  }
+}
+
 static int
 cc_can_write (void *handle)
 {
@@ -567,6 +585,13 @@ cc_cache (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
     return 0;
 }
 
+static void
+cc_cleanup (void)
+{
+  if (subplugin.cleanup)
+    subplugin.cleanup ();
+}
+
 static struct nbdkit_plugin plugin = {
   .name              = "cc",
   .longname          = "nbdkit C compiler plugin",
@@ -585,6 +610,7 @@ static struct nbdkit_plugin plugin = {
    */
   .get_ready          = cc_get_ready,
   .after_fork         = cc_after_fork,
+  .cleanup            = cc_cleanup,
 
   .preconnect         = cc_preconnect,
   .list_exports       = cc_list_exports,
@@ -594,6 +620,7 @@ static struct nbdkit_plugin plugin = {
 
   .export_description = cc_export_description,
   .get_size           = cc_get_size,
+  .block_size         = cc_block_size,
   .can_write          = cc_can_write,
   .can_flush          = cc_can_flush,
   .is_rotational      = cc_is_rotational,
