@@ -69,8 +69,11 @@ const char *url = NULL;         /* required */
 const char *cainfo = NULL;
 const char *capath = NULL;
 char *cookie = NULL;
+const char *cookiefile = NULL;
+const char *cookiejar = NULL;
 const char *cookie_script = NULL;
 unsigned cookie_script_renew = 0;
+bool followlocation = true;
 struct curl_slist *headers = NULL;
 const char *header_script = NULL;
 unsigned header_script_renew = 0;
@@ -80,6 +83,9 @@ const char *proxy = NULL;
 char *proxy_password = NULL;
 const char *proxy_user = NULL;
 bool sslverify = true;
+const char *ssl_version = NULL;
+const char *ssl_cipher_list = NULL;
+const char *tls13_ciphers = NULL;
 bool tcp_keepalive = false;
 bool tcp_nodelay = true;
 uint32_t timeout = 0;
@@ -88,7 +94,7 @@ const char *user = NULL;
 const char *user_agent = NULL;
 
 /* Use '-D curl.verbose=1' to set. */
-int curl_debug_verbose = 0;
+NBDKIT_DLL_PUBLIC int curl_debug_verbose = 0;
 
 static void
 curl_load (void)
@@ -210,6 +216,28 @@ curl_config (const char *key, const char *value)
       return -1;
   }
 
+  else if (strcmp (key, "cookiefile") == 0) {
+    /* Reject cookiefile=- because it will cause libcurl to try to
+     * read from stdin when we connect.
+     */
+    if (strcmp (value, "-") == 0) {
+      nbdkit_error ("cookiefile parameter cannot be \"-\"");
+      return -1;
+    }
+    cookiefile = value;
+  }
+
+  else if (strcmp (key, "cookiejar") == 0) {
+    /* Reject cookiejar=- because it will cause libcurl to try to
+     * write to stdout.
+     */
+    if (strcmp (value, "-") == 0) {
+      nbdkit_error ("cookiejar parameter cannot be \"-\"");
+      return -1;
+    }
+    cookiejar = value;
+  }
+
   else if (strcmp (key, "cookie-script") == 0) {
     cookie_script = value;
   }
@@ -220,6 +248,13 @@ curl_config (const char *key, const char *value)
       return -1;
   }
 
+  else if (strcmp (key, "followlocation") == 0) {
+    r = nbdkit_parse_bool (value);
+    if (r == -1)
+      return -1;
+    followlocation = r;
+  }
+
   else if (strcmp (key, "header") == 0) {
     headers = curl_slist_append (headers, value);
     if (headers == NULL) {
@@ -269,6 +304,15 @@ curl_config (const char *key, const char *value)
     sslverify = r;
   }
 
+  else if (strcmp (key, "ssl-version") == 0)
+    ssl_version = value;
+
+  else if (strcmp (key, "ssl-cipher-list") == 0)
+    ssl_cipher_list = value;
+
+  else if (strcmp (key, "tls13-ciphers") == 0)
+    tls13_ciphers = value;
+
   else if (strcmp (key, "tcp-keepalive") == 0) {
     r = nbdkit_parse_bool (value);
     if (r == -1)
@@ -355,8 +399,12 @@ curl_config_complete (void)
   "cainfo=<CAINFO>            Path to Certificate Authority file.\n" \
   "capath=<CAPATH>            Path to directory with CA certificates.\n" \
   "cookie=<COOKIE>            Set HTTP/HTTPS cookies.\n" \
+  "cookiefile=                Enable cookie processing.\n" \
+  "cookiefile=<FILENAME>      Read cookies from file.\n" \
+  "cookiejar=<FILENAME>       Read and write cookies to jar.\n" \
   "cookie-script=<SCRIPT>     Script to set HTTP/HTTPS cookies.\n" \
   "cookie-script-renew=<SECS> Time to renew HTTP/HTTPS cookies.\n" \
+  "followlocation=false       Do not follow redirects.\n" \
   "header=<HEADER>            Set HTTP/HTTPS header.\n" \
   "header-script=<SCRIPT>     Script to set HTTP/HTTPS headers.\n" \
   "header-script-renew=<SECS> Time to renew HTTP/HTTPS headers.\n" \
@@ -367,6 +415,9 @@ curl_config_complete (void)
   "proxy-user=<USER>          The proxy user.\n" \
   "timeout=<TIMEOUT>          Set the timeout for requests (seconds).\n" \
   "sslverify=false            Do not verify SSL certificate of remote host.\n" \
+  "ssl-version=<VERSION>      Specify preferred TLS/SSL version.\n " \
+  "ssl-cipher-list=C1:C2:..   Specify TLS/SSL cipher suites to be used.\n" \
+  "tls13-ciphers=C1:C2:..     Specify TLS 1.3 cipher suites to be used.\n" \
   "tcp-keepalive=true         Enable TCP keepalives.\n" \
   "tcp-nodelay=false          Disable Nagle’s algorithm.\n" \
   "unix-socket-path=<PATH>    Open Unix domain socket instead of TCP/IP.\n" \
@@ -441,18 +492,31 @@ curl_open (int readonly)
     goto err;
   }
 
-  /* Various options we always set. */
-  curl_easy_setopt (h->c, CURLOPT_AUTOREFERER, 1);
-  curl_easy_setopt (h->c, CURLOPT_FOLLOWLOCATION, 1);
-  curl_easy_setopt (h->c, CURLOPT_FAILONERROR, 1);
+  /* Various options we always set.
+   *
+   * NB: Both here and below constants must be explicitly long because
+   * the parameter is varargs.
+   */
+  curl_easy_setopt (h->c, CURLOPT_AUTOREFERER, 1L);
+  if (followlocation)
+    curl_easy_setopt (h->c, CURLOPT_FOLLOWLOCATION, 1L);
+  curl_easy_setopt (h->c, CURLOPT_FAILONERROR, 1L);
 
   /* Options. */
-  if (cainfo)
-    curl_easy_setopt (h->c, CURLOPT_CAINFO, cainfo);
+  if (cainfo) {
+    if (strlen (cainfo) == 0)
+      curl_easy_setopt (h->c, CURLOPT_CAINFO, NULL);
+    else
+      curl_easy_setopt (h->c, CURLOPT_CAINFO, cainfo);
+  }
   if (capath)
     curl_easy_setopt (h->c, CURLOPT_CAPATH, capath);
   if (cookie)
     curl_easy_setopt (h->c, CURLOPT_COOKIE, cookie);
+  if (cookiefile)
+    curl_easy_setopt (h->c, CURLOPT_COOKIEFILE, cookiefile);
+  if (cookiejar)
+    curl_easy_setopt (h->c, CURLOPT_COOKIEJAR, cookiejar);
   if (headers)
     curl_easy_setopt (h->c, CURLOPT_HTTPHEADER, headers);
   if (password)
@@ -468,12 +532,44 @@ curl_open (int readonly)
   if (proxy_user)
     curl_easy_setopt (h->c, CURLOPT_PROXYUSERNAME, proxy_user);
   if (!sslverify) {
-    /* NB: Constants must be explicitly long because the parameter is
-     * varargs.
-     */
     curl_easy_setopt (h->c, CURLOPT_SSL_VERIFYPEER, 0L);
     curl_easy_setopt (h->c, CURLOPT_SSL_VERIFYHOST, 0L);
   }
+  if (ssl_version) {
+    if (strcmp (ssl_version, "tlsv1") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
+    else if (strcmp (ssl_version, "sslv2") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv2);
+    else if (strcmp (ssl_version, "sslv3") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
+    else if (strcmp (ssl_version, "tlsv1.0") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_0);
+    else if (strcmp (ssl_version, "tlsv1.1") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_1);
+    else if (strcmp (ssl_version, "tlsv1.2") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2);
+    else if (strcmp (ssl_version, "tlsv1.3") == 0)
+      curl_easy_setopt (h->c, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_3);
+    else {
+      display_curl_error (h, r, "curl_easy_setopt: CURLOPT_SSLVERSION [%s]",
+			  ssl_version);
+      goto err;
+    }
+
+  }
+  if (ssl_cipher_list)
+    curl_easy_setopt (h->c, CURLOPT_SSL_CIPHER_LIST, ssl_cipher_list);
+  if (tls13_ciphers) {
+#if (LIBCURL_VERSION_MAJOR > 7) || \
+    (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR >= 61)
+    curl_easy_setopt (h->c, CURLOPT_TLS13_CIPHERS, tls13_ciphers);
+#else
+    /* This is not available before curl-7.61 */
+    nbdkit_error ("tls13-ciphers is not supported in this build of "
+                  "nbdkit-curl-plugin");
+    goto err;
+#endif
+  }
   if (tcp_keepalive)
     curl_easy_setopt (h->c, CURLOPT_TCP_KEEPALIVE, 1L);
   if (!tcp_nodelay)
@@ -496,7 +592,7 @@ curl_open (int readonly)
    */
   if (do_scripts (h) == -1) goto err;
   h->accept_range = false;
-  curl_easy_setopt (h->c, CURLOPT_NOBODY, 1); /* No Body, not nobody! */
+  curl_easy_setopt (h->c, CURLOPT_NOBODY, 1L); /* No Body, not nobody! */
   curl_easy_setopt (h->c, CURLOPT_HEADERFUNCTION, header_cb);
   curl_easy_setopt (h->c, CURLOPT_HEADERDATA, h);
   r = curl_easy_perform (h->c);
@@ -693,7 +789,7 @@ curl_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
   h->write_buf = buf;
   h->write_count = count;
 
-  curl_easy_setopt (h->c, CURLOPT_HTTPGET, 1);
+  curl_easy_setopt (h->c, CURLOPT_HTTPGET, 1L);
 
   /* Make an HTTP range request. */
   snprintf (range, sizeof range, "%" PRIu64 "-%" PRIu64,
@@ -757,7 +853,7 @@ curl_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset)
   h->read_buf = buf;
   h->read_count = count;
 
-  curl_easy_setopt (h->c, CURLOPT_UPLOAD, 1);
+  curl_easy_setopt (h->c, CURLOPT_UPLOAD, 1L);
 
   /* Make an HTTP range request. */
   snprintf (range, sizeof range, "%" PRIu64 "-%" PRIu64,
