@@ -37,7 +37,7 @@ C<nbdkit-example4-plugin> first appeared in nbdkit 1.2.
 
 =head1 SEE ALSO
 
-L<https://github.com/libguestfs/nbdkit/blob/master/plugins/example4/example4.pl>,
+L<https://gitlab.com/nbdkit/nbdkit/blob/master/plugins/example4/example4.pl>,
 L<nbdkit(1)>,
 L<nbdkit-plugin(3)>,
 L<nbdkit-perl-plugin(3)>.
@@ -72,10 +72,19 @@ sub config
     }
 }
 
-# When all config parameters have been seen, allocate the disk.
+# After all the parameters have been parsed, this can be used to check
+# for invalid or missing parameters.
 sub config_complete
 {
     die "size was not set" unless defined $size;
+}
+
+# This is called just before forking into the background and is the
+# last opportunity for the plugin to print an error message that the
+# user can see (without digging through log files).  Here we allocate
+# the disk.
+sub get_ready
+{
     $disk = "\0" x $size;
 }
 
