@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright (C) 2013-2021 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -30,6 +30,10 @@
  * SUCH DAMAGE.
  */
 
+#ifdef WIN32
+#error "build error: winfile.c should be used on Windows"
+#endif
+
 #include <config.h>
 
 #include <stdio.h>
@@ -39,10 +43,15 @@
 #include <inttypes.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/ioctl.h>
 #include <errno.h>
+#include <dirent.h>
+
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
 
 #include <pthread.h>
 
@@ -50,7 +59,7 @@
 #include <linux/falloc.h>   /* For FALLOC_FL_*, glibc < 2.18 */
 #endif
 
-#if defined (__linux__)
+#if defined (__linux__) && HAVE_LINUX_FS_H
 #include <linux/fs.h>       /* For BLKZEROOUT */
 #endif
 
@@ -59,10 +68,7 @@
 
 #include "cleanup.h"
 #include "isaligned.h"
-
-#ifndef HAVE_FDATASYNC
-#define fdatasync fsync
-#endif
+#include "fdatasync.h"
 
 static char *filename = NULL;
 static char *directory = NULL;
@@ -79,13 +85,76 @@ static int fadvise_mode =
 /* cache mode */
 static enum { cache_default, cache_none } cache_mode = cache_default;
 
+/* Define EVICT_WRITES if we are going to evict the page cache
+ * (cache=none) after writing.  This is only known to work on Linux.
+ */
+#ifdef __linux__
+#define EVICT_WRITES 1
+#endif
+
+#ifdef EVICT_WRITES
+/* Queue writes so they will be evicted from the cache.  See
+ * libnbd.git copy/file-ops.c for the rationale behind this.
+ */
+#define NR_WINDOWS 8
+
+struct write_window {
+  int fd;
+  uint64_t offset;
+  size_t len;
+};
+
+static pthread_mutex_t window_lock = PTHREAD_MUTEX_INITIALIZER;
+static struct write_window window[NR_WINDOWS];
+
+static void
+evict_writes (int fd, uint64_t offset, size_t len)
+{
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&window_lock);
+
+  /* Evict the oldest window from the page cache. */
+  if (window[0].len > 0) {
+    sync_file_range (window[0].fd, window[0].offset, window[0].len,
+                     SYNC_FILE_RANGE_WAIT_BEFORE|SYNC_FILE_RANGE_WRITE|
+                     SYNC_FILE_RANGE_WAIT_AFTER);
+    posix_fadvise (window[0].fd, window[0].offset, window[0].len,
+                   POSIX_FADV_DONTNEED);
+  }
+
+  /* Move the Nth window to N-1. */
+  memmove (&window[0], &window[1], sizeof window[0] * (NR_WINDOWS-1));
+
+  /* Set up the current window and tell Linux to start writing it out
+   * to disk (asynchronously).
+   */
+  sync_file_range (fd, offset, len, SYNC_FILE_RANGE_WRITE);
+  window[NR_WINDOWS-1].fd = fd;
+  window[NR_WINDOWS-1].offset = offset;
+  window[NR_WINDOWS-1].len = len;
+}
+
+/* When we close the handle we must remove any windows which are still
+ * associated.  They missed the boat, oh well :-(
+ */
+static void
+remove_fd_from_window (int fd)
+{
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&window_lock);
+  size_t i;
+
+  for (i = 0; i < NR_WINDOWS; ++i)
+    if (window[i].len > 0 && window[i].fd == fd)
+      window[i].len = 0;
+}
+#endif /* EVICT_WRITES */
+
 /* Any callbacks using lseek must be protected by this lock. */
 static pthread_mutex_t lseek_lock = PTHREAD_MUTEX_INITIALIZER;
 
 /* to enable: -D file.zero=1 */
-int file_debug_zero;
+NBDKIT_DLL_PUBLIC int file_debug_zero;
 
-static bool
+static bool __attribute__((unused))
 is_enotsup (int err)
 {
   return err == ENOTSUP || err == EOPNOTSUPP;
@@ -176,7 +245,7 @@ file_config (const char *key, const char *value)
   return 0;
 }
 
-/* Check the user passed exactly one parameter. */
+/* Check that the user passed exactly one parameter. */
 static int
 file_config_complete (void)
 {
@@ -241,11 +310,64 @@ file_dump_plugin (void)
 #endif
 }
 
+static int
+file_list_exports (int readonly, int default_only,
+                   struct nbdkit_exports *exports)
+{
+  /* We don't fork, so no need to worry about FD_CLOEXEC on the directory */
+  DIR *dir;
+  struct dirent *entry;
+  struct stat sb;
+  int fd;
+
+  if (!directory)
+    return nbdkit_add_export (exports, "", NULL);
+
+  dir = opendir (directory);
+  if (dir == NULL) {
+    nbdkit_error ("opendir: %m");
+    return -1;
+  }
+  fd = dirfd (dir);
+  if (fd == -1) {
+    nbdkit_error ("dirfd: %m");
+    closedir (dir);
+    return -1;
+  }
+  errno = 0;
+  while ((entry = readdir (dir)) != NULL) {
+    int r = -1;
+#if HAVE_STRUCT_DIRENT_D_TYPE
+    if (entry->d_type == DT_BLK || entry->d_type == DT_REG)
+      r = 1;
+    else if (entry->d_type != DT_LNK && entry->d_type != DT_UNKNOWN)
+      r = 0;
+#endif
+    /* TODO: when chasing symlinks, is statx any nicer than fstatat? */
+    if (r < 0 && fstatat (fd, entry->d_name, &sb, 0) == 0 &&
+        (S_ISREG (sb.st_mode) || S_ISBLK (sb.st_mode)))
+      r = 1;
+    if (r == 1 && nbdkit_add_export (exports, entry->d_name, NULL) == -1) {
+      closedir (dir);
+      return -1;
+    }
+    errno = 0;
+  }
+  if (errno) {
+    nbdkit_error ("readdir: %m");
+    closedir (dir);
+    return -1;
+  }
+  closedir (dir);
+  return 0;
+}
+
 /* The per-connection handle. */
 struct handle {
   int fd;
   bool is_block_device;
   int sector_size;
+  bool can_write;
   bool can_punch_hole;
   bool can_zero_range;
   bool can_fallocate;
@@ -281,18 +403,31 @@ file_open (int readonly)
   h = malloc (sizeof *h);
   if (h == NULL) {
     nbdkit_error ("malloc: %m");
+    if (dfd != -1)
+      close (dfd);
     return NULL;
   }
 
   flags = O_CLOEXEC|O_NOCTTY;
-  if (readonly)
+  if (readonly) {
     flags |= O_RDONLY;
-  else
+    h->can_write = false;
+  }
+  else {
     flags |= O_RDWR;
+    h->can_write = true;
+  }
 
   h->fd = openat (dfd, file, flags);
+  if (h->fd == -1 && !readonly) {
+    nbdkit_debug ("open O_RDWR failed, falling back to read-only: %s: %m",
+                  file);
+    flags = (flags & ~O_ACCMODE) | O_RDONLY;
+    h->fd = openat (dfd, file, flags);
+    h->can_write = false;
+  }
   if (h->fd == -1) {
-    nbdkit_error ("openat: %s: %m", file);
+    nbdkit_error ("open: %s: %m", file);
     if (dfd != -1)
       close (dfd);
     free (h);
@@ -303,6 +438,7 @@ file_open (int readonly)
 
   if (fstat (h->fd, &statbuf) == -1) {
     nbdkit_error ("fstat: %s: %m", file);
+    close (h->fd);
     free (h);
     return NULL;
   }
@@ -361,6 +497,9 @@ file_close (void *handle)
 {
   struct handle *h = handle;
 
+#ifdef EVICT_WRITES
+  remove_fd_from_window (h->fd);
+#endif
   close (h->fd);
   free (h);
 }
@@ -368,7 +507,7 @@ file_close (void *handle)
 #define THREAD_MODEL NBDKIT_THREAD_MODEL_PARALLEL
 
 /* For block devices, stat->st_size is not the true size.  The caller
- * grabs the lseek_lock.
+ * grabs the lock.
  */
 static int64_t
 block_device_size (int fd)
@@ -406,6 +545,15 @@ file_get_size (void *handle)
   }
 }
 
+/* Check if file is read-only. */
+static int
+file_can_write (void *handle)
+{
+  struct handle *h = handle;
+
+  return h->can_write;
+}
+
 /* Allow multiple parallel connections from a single client. */
 static int
 file_can_multi_conn (void *handle)
@@ -504,15 +652,9 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
 {
   struct handle *h = handle;
 
-#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
+#if EVICT_WRITES
   uint32_t orig_count = count;
   uint64_t orig_offset = offset;
-
-  /* If cache=none we want to force pages we have just written to the
-   * file to be flushed to disk so we can immediately evict them from
-   * the page cache.
-   */
-  if (cache_mode == cache_none) flags |= NBDKIT_FLAG_FUA;
 #endif
 
   while (count > 0) {
@@ -529,10 +671,9 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
   if ((flags & NBDKIT_FLAG_FUA) && file_flush (handle, 0) == -1)
     return -1;
 
-#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
-  /* On Linux this will evict the pages we just wrote from the page cache. */
+#if EVICT_WRITES
   if (cache_mode == cache_none)
-    posix_fadvise (h->fd, orig_offset, orig_count, POSIX_FADV_DONTNEED);
+    evict_writes (h->fd, orig_offset, orig_count);
 #endif
 
   return 0;
@@ -556,11 +697,12 @@ do_fallocate (int fd, int mode, off_t offset, off_t len)
 static int
 file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
-  struct handle *h = handle;
-  int r;
+  struct handle *h __attribute__((unused)) = handle;
 
 #ifdef FALLOC_FL_PUNCH_HOLE
   if (h->can_punch_hole && (flags & NBDKIT_FLAG_MAY_TRIM)) {
+    int r;
+
     r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
                       offset, count);
     if (r == 0) {
@@ -581,6 +723,8 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 
 #ifdef FALLOC_FL_ZERO_RANGE
   if (h->can_zero_range) {
+    int r;
+
     r = do_fallocate (h->fd, FALLOC_FL_ZERO_RANGE, offset, count);
     if (r == 0) {
       if (file_debug_zero)
@@ -603,6 +747,8 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
    * fallocate to zero a range. This is expected to be more efficient than
    * writing zeroes manually. */
   if (h->can_punch_hole && h->can_fallocate) {
+    int r;
+
     r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
                       offset, count);
     if (r == 0) {
@@ -634,6 +780,7 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 #ifdef BLKZEROOUT
   /* For aligned range and block device, we can use BLKZEROOUT. */
   if (h->can_zeroout && IS_ALIGNED (offset | count, h->sector_size)) {
+    int r;
     uint64_t range[2] = {offset, count};
 
     r = ioctl (h->fd, BLKZEROOUT, &range);
@@ -659,7 +806,10 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   errno = EOPNOTSUPP;
   return -1;
 
- out:
+#ifdef __clang__
+  __attribute__((unused))
+#endif
+    out:
   if ((flags & NBDKIT_FLAG_FUA) && file_flush (handle, 0) == -1)
     return -1;
   return 0;
@@ -818,9 +968,11 @@ static struct nbdkit_plugin plugin = {
   .config_help       = file_config_help,
   .magic_config_key  = "file",
   .dump_plugin       = file_dump_plugin,
+  .list_exports      = file_list_exports,
   .open              = file_open,
   .close             = file_close,
   .get_size          = file_get_size,
+  .can_write         = file_can_write,
   .can_multi_conn    = file_can_multi_conn,
   .can_trim          = file_can_trim,
   .can_fua           = file_can_fua,
