@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2019 Red Hat Inc.
+ * Copyright (C) 2018-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -50,6 +50,9 @@ static char *dir = NULL;
 /* Volume label. */
 static const char *label = "NBDKITFLOPY";
 
+/* Filesystem size. */
+static uint64_t size = 0;
+
 /* Virtual floppy. */
 static struct virtual_floppy floppy;
 
@@ -69,6 +72,8 @@ floppy_unload (void)
 static int
 floppy_config (const char *key, const char *value)
 {
+  int64_t r;
+
   if (strcmp (key, "dir") == 0) {
     if (dir != NULL) {
       /* TODO: Support merging of multiple directories, like iso plugin. */
@@ -82,6 +87,12 @@ floppy_config (const char *key, const char *value)
   else if (strcmp (key, "label") == 0) {
     label = value;
   }
+  else if (strcmp (key, "size") == 0) {
+    r = nbdkit_parse_size (value);
+    if (r == -1)
+      return -1;
+    size = r;
+  }
   else {
     nbdkit_error ("unknown parameter '%s'", key);
     return -1;
@@ -99,13 +110,19 @@ floppy_config_complete (void)
     return -1;
   }
 
-  return create_virtual_floppy (dir, label, &floppy);
+  return 0;
 }
 
 #define floppy_config_help \
   "dir=<DIRECTORY>     (required) The directory to serve.\n" \
   "label=<LABEL>                  The volume label." \
 
+static int
+floppy_get_ready (void)
+{
+  return create_virtual_floppy (dir, label, size, &floppy);
+}
+
 static void *
 floppy_open (int readonly)
 {
@@ -155,8 +172,8 @@ floppy_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
     switch (region->type) {
     case region_file:
       i = region->u.i;
-      assert (i < floppy.nr_files);
-      host_path = floppy.files[i].host_path;
+      assert (i < floppy.files.len);
+      host_path = floppy.files.ptr[i].host_path;
       fd = open (host_path, O_RDONLY|O_CLOEXEC);
       if (fd == -1) {
         nbdkit_error ("open: %s: %m", host_path);
@@ -204,6 +221,7 @@ static struct nbdkit_plugin plugin = {
   .config_complete   = floppy_config_complete,
   .config_help       = floppy_config_help,
   .magic_config_key  = "dir",
+  .get_ready         = floppy_get_ready,
   .open              = floppy_open,
   .get_size          = floppy_get_size,
   .can_multi_conn    = floppy_can_multi_conn,
