@@ -85,21 +85,22 @@ init_virtual_floppy (struct virtual_floppy *floppy)
 }
 
 int
-create_virtual_floppy (const char *dir, const char *label,
+create_virtual_floppy (const char *dir, const char *label, uint64_t size,
                        struct virtual_floppy *floppy)
 {
   size_t i;
   uint64_t nr_bytes, nr_clusters;
+  uint64_t data_used_size;
   uint32_t cluster;
 
   if (visit (dir, floppy) == -1)
     return -1;
 
   nbdkit_debug ("floppy: %zu directories and %zu files",
-                floppy->dirs.size, floppy->files.size);
+                floppy->dirs.len, floppy->files.len);
 
   /* Create the on disk directory tables. */
-  for (i = 0; i < floppy->dirs.size; ++i) {
+  for (i = 0; i < floppy->dirs.len; ++i) {
     if (create_directory (i, label, floppy) == -1)
       return -1;
   }
@@ -112,24 +113,24 @@ create_virtual_floppy (const char *dir, const char *label,
    * The first cluster number is always 2 (0 and 1 are reserved), and
    * (in this implementation) always contains the root directory.
    */
-  floppy->data_size = 0;
+  data_used_size = 0;
   cluster = 2;
-  for (i = 0; i < floppy->dirs.size; ++i) {
+  for (i = 0; i < floppy->dirs.len; ++i) {
     floppy->dirs.ptr[i].first_cluster = cluster;
     nr_bytes =
-      ROUND_UP (floppy->dirs.ptr[i].table.size * sizeof (struct dir_entry),
+      ROUND_UP (floppy->dirs.ptr[i].table.len * sizeof (struct dir_entry),
                 CLUSTER_SIZE);
-    floppy->data_size += nr_bytes;
+    data_used_size += nr_bytes;
     nr_clusters = nr_bytes / CLUSTER_SIZE;
     if (cluster + nr_clusters > UINT32_MAX)
       goto too_big;
     floppy->dirs.ptr[i].nr_clusters = nr_clusters;
     cluster += nr_clusters;
   }
-  for (i = 0; i < floppy->files.size; ++i) {
+  for (i = 0; i < floppy->files.len; ++i) {
     floppy->files.ptr[i].first_cluster = cluster;
     nr_bytes = ROUND_UP (floppy->files.ptr[i].statbuf.st_size, CLUSTER_SIZE);
-    floppy->data_size += nr_bytes;
+    data_used_size += nr_bytes;
     nr_clusters = nr_bytes / CLUSTER_SIZE;
     if (cluster + nr_clusters > UINT32_MAX)
       goto too_big;
@@ -137,7 +138,21 @@ create_virtual_floppy (const char *dir, const char *label,
     cluster += nr_clusters;
   }
 
+  if (size > 0) {
+    uint64_t data_size = size - (2080 * SECTOR_SIZE);
+    data_size = data_size - 2 * DIV_ROUND_UP((data_size / CLUSTER_SIZE + 2) * 4,
+                                             CLUSTER_SIZE) * CLUSTER_SIZE;
+    if (data_used_size > data_size) {
+      nbdkit_error ("filesystem is larger than \"size\" bytes");
+      return -1;
+    }
+    floppy->data_size = data_size;
+  } else {
+    floppy->data_size = data_used_size;
+  }
+
   floppy->data_clusters = floppy->data_size / CLUSTER_SIZE;
+  floppy->data_used_clusters = data_used_size / CLUSTER_SIZE;
 
   /* Despite its name, FAT32 only allows 28 bit cluster numbers, so
    * give an error if we go beyond this.
@@ -172,7 +187,7 @@ create_virtual_floppy (const char *dir, const char *label,
    * directory entries (which we didn't have available during
    * create_directory above).
    */
-  for (i = 0; i < floppy->dirs.size; ++i) {
+  for (i = 0; i < floppy->dirs.len; ++i) {
     if (update_directory_first_cluster (i, floppy) == -1)
       return -1;
   }
@@ -199,6 +214,10 @@ create_virtual_floppy (const char *dir, const char *label,
   if (create_regions (floppy) == -1)
     return -1;
 
+  /* Check that if a size was specified, we ended up with it. */
+  if (size > 0)
+    assert (virtual_size (&floppy->regions) == size);
+
   return 0;
 }
 
@@ -211,13 +230,13 @@ free_virtual_floppy (struct virtual_floppy *floppy)
 
   free (floppy->fat);
 
-  for (i = 0; i < floppy->files.size; ++i) {
+  for (i = 0; i < floppy->files.len; ++i) {
     free (floppy->files.ptr[i].name);
     free (floppy->files.ptr[i].host_path);
   }
   free (floppy->files.ptr);
 
-  for (i = 0; i < floppy->dirs.size; ++i) {
+  for (i = 0; i < floppy->dirs.len; ++i) {
     free (floppy->dirs.ptr[i].name);
     free (floppy->dirs.ptr[i].subdirs.ptr);
     free (floppy->dirs.ptr[i].fileidxs.ptr);
@@ -247,7 +266,7 @@ visit (const char *dir, struct virtual_floppy *floppy)
    * directory will always be at dirs[0].
    */
   memset (&null_dir, 0, sizeof null_dir);
-  di = floppy->dirs.size;
+  di = floppy->dirs.len;
   if (dirs_append (&floppy->dirs, null_dir) == -1) {
     nbdkit_error ("realloc: %m");
     goto error0;
@@ -318,10 +337,14 @@ visit (const char *dir, struct virtual_floppy *floppy)
   closedir (DIR);
  error1:
   err = errno;
+#if defined(__GNUC__) && __GNUC__ >= 5
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-result"
+#endif
   chdir (origdir);
+#if defined(__GNUC__) && __GNUC__ >= 5
 #pragma GCC diagnostic pop
+#endif
   errno = err;
  error0:
   return -1;
@@ -404,9 +427,10 @@ visit_file (const char *dir, const char *name,
   }
   new_file.host_path = host_path;
   new_file.statbuf = *statbuf;
-  fi = floppy->files.size;
+  fi = floppy->files.len;
   if (files_append (&floppy->files, new_file) == -1) {
     nbdkit_error ("realloc: %m");
+    free (new_file.name);
     free (host_path);
     return -1;
   }
@@ -488,7 +512,7 @@ create_partition_boot_sector (const char *label, struct virtual_floppy *floppy)
   floppy->bootsect.sectors_per_track = htole16 (0);
   floppy->bootsect.nr_heads = htole16 (0);
   floppy->bootsect.nr_hidden_sectors = htole32 (0);
-  floppy->bootsect.nr_sectors = htole32 (floppy->data_last_sector + 1);
+  floppy->bootsect.nr_sectors = htole32 (floppy->data_last_sector - 2048 + 1);
 
   floppy->bootsect.sectors_per_fat =
     htole32 (floppy->fat_clusters * SECTORS_PER_CLUSTER);
@@ -524,8 +548,9 @@ create_fsinfo (struct virtual_floppy *floppy)
   floppy->fsinfo.signature2[1] = 0x72;
   floppy->fsinfo.signature2[2] = 0x41;
   floppy->fsinfo.signature2[3] = 0x61;
-  floppy->fsinfo.free_data_clusters = htole32 (0);
-  floppy->fsinfo.last_free_cluster = htole32 (2 + floppy->data_clusters);
+  floppy->fsinfo.free_data_clusters = htole32 (floppy->data_clusters -
+                                               floppy->data_used_clusters);
+  floppy->fsinfo.last_free_cluster = htole32 (2 + floppy->data_used_clusters);
   floppy->fsinfo.signature3[0] = 0x00;
   floppy->fsinfo.signature3[1] = 0x00;
   floppy->fsinfo.signature3[2] = 0x55;
@@ -554,11 +579,11 @@ create_fat (struct virtual_floppy *floppy)
   floppy->fat[0] = htole32 (0x0ffffff8);
   floppy->fat[1] = htole32 (0x0fffffff);
 
-  for (i = 0; i < floppy->dirs.size; ++i) {
+  for (i = 0; i < floppy->dirs.len; ++i) {
     write_fat_file (floppy->dirs.ptr[i].first_cluster,
                     floppy->dirs.ptr[i].nr_clusters, floppy);
   }
-  for (i = 0; i < floppy->files.size; ++i) {
+  for (i = 0; i < floppy->files.len; ++i) {
     write_fat_file (floppy->files.ptr[i].first_cluster,
                     floppy->files.ptr[i].nr_clusters, floppy);
   }
@@ -656,15 +681,15 @@ create_regions (struct virtual_floppy *floppy)
   /* Now we're into the data region.  We add all directory tables
    * first.
    */
-  for (i = 0; i < floppy->dirs.size; ++i) {
+  for (i = 0; i < floppy->dirs.len; ++i) {
     /* Directories can never be completely empty because of the volume
      * label (root) or "." and ".." entries (non-root).
      */
-    assert (floppy->dirs.ptr[i].table.size > 0);
+    assert (floppy->dirs.ptr[i].table.len > 0);
 
     if (append_region_len (&floppy->regions,
                            i == 0 ? "root directory" : floppy->dirs.ptr[i].name,
-                           floppy->dirs.ptr[i].table.size *
+                           floppy->dirs.ptr[i].table.len *
                            sizeof (struct dir_entry),
                            0, CLUSTER_SIZE,
                            region_data,
@@ -673,7 +698,7 @@ create_regions (struct virtual_floppy *floppy)
   }
 
   /* Add all files. */
-  for (i = 0; i < floppy->files.size; ++i) {
+  for (i = 0; i < floppy->files.len; ++i) {
     /* It's possible for a file to have zero size, in which case it
      * doesn't occupy a region or cluster.
      */
@@ -687,6 +712,15 @@ create_regions (struct virtual_floppy *floppy)
       return -1;
   }
 
+  /* Append region for empty data region if we have extra space. */
+  if (floppy->data_used_clusters != floppy->data_clusters) {
+    uint64_t clusters = floppy->data_clusters - floppy->data_used_clusters;
+    if (append_region_len (&floppy->regions, "data padding",
+                           clusters * CLUSTER_SIZE, 0, CLUSTER_SIZE,
+                           region_zero) == -1)
+      return -1;
+  }
+
   nbdkit_debug ("floppy: %zu regions, "
                 "total disk size %" PRIi64,
                 nr_regions (&floppy->regions),
