@@ -42,15 +42,14 @@
 #include <nbdkit-plugin.h>
 
 #include "cleanup.h"
+#include "string-vector.h"
 #include "utils.h"
-#include "vector.h"
 
 /* List of directories parsed from the command line. */
-DEFINE_VECTOR_TYPE(string_vector, char *);
 static string_vector dirs = empty_vector;
 
-/* genisoimage or mkisofs program, picked at compile time, but can be
- * overridden at run time.
+/* xorriso or genisoimage or mkisofs program, picked at compile time,
+ * but can be overridden at run time.
  */
 static const char *isoprog = ISOPROG;
 
@@ -96,10 +95,16 @@ make_iso (void)
   }
 
   shell_quote (isoprog, fp);
+#if ISOPROG_IS_XORRISO
+  /* This is necessary to make xorriso command line parsing behave
+   * like mkisofs.
+   */
+  fprintf (fp, " -as mkisofs");
+#endif
   fprintf (fp, " -quiet");
   if (params)
     fprintf (fp, " %s", params);
-  for (i = 0; i < dirs.size; ++i) {
+  for (i = 0; i < dirs.len; ++i) {
     fputc (' ', fp);
     shell_quote (dirs.ptr[i], fp);
   }
@@ -163,7 +168,7 @@ iso_config (const char *key, const char *value)
 static int
 iso_config_complete (void)
 {
-  if (dirs.size == 0) {
+  if (dirs.len == 0) {
     nbdkit_error ("you must supply the dir=<DIRECTORY> parameter "
                   "after the plugin name on the command line");
     return -1;
