@@ -43,7 +43,7 @@ type fua_flag = FuaNone | FuaEmulate | FuaNative
 
 type cache_flag = CacheNone | CacheEmulate | CacheNop
 
-(** The type of the extent list returned by {!plugin.extents} *)
+(** The type of the extent list returned by [extents] *)
 type extent = {
   offset : int64;
   length : int64;
@@ -51,77 +51,80 @@ type extent = {
   is_zero : bool;
 }
 
-(** The type of the export list returned by {!plugin.list_exports} *)
+(** The type of the export list returned by [list_exports] *)
 type export = {
   name : string;
   description : string option;
 }
 
-(** The type of the thread model returned by {!plugin.thread_model} *)
+(** The type of the thread model returned by [thread_model] *)
 type thread_model =
 | THREAD_MODEL_SERIALIZE_CONNECTIONS
 | THREAD_MODEL_SERIALIZE_ALL_REQUESTS
 | THREAD_MODEL_SERIALIZE_REQUESTS
 | THREAD_MODEL_PARALLEL
 
-(** The plugin fields and callbacks.
+(** Register the plugin with nbdkit.
 
     The ['a] parameter is the handle type returned by your
     [open_connection] method and passed back to all connected calls. *)
-type 'a plugin = {
-  name : string;                                  (* required *)
-  longname : string;
-  version : string;
-  description : string;
+val register_plugin :
+  (* Plugin description. *)
+  name: string ->
+  ?longname: string ->
+  ?version: string ->
+  ?description: string ->
 
-  load : (unit -> unit) option;
-  unload : (unit -> unit) option;
+  (* Plugin lifecycle. *)
+  ?load: (unit -> unit) ->
+  ?get_ready: (unit -> unit) ->
+  ?after_fork: (unit -> unit) ->
+  ?cleanup: (unit -> unit) ->
+  ?unload: (unit -> unit) ->
 
-  dump_plugin : (unit -> unit) option;
+  (* Plugin configuration. *)
+  ?config: (string -> string -> unit) ->
+  ?config_complete: (unit -> unit) ->
+  ?config_help: string ->
+  ?thread_model: (unit -> thread_model) ->
+  ?magic_config_key: string ->
 
-  config : (string -> string -> unit) option;
-  config_complete : (unit -> unit) option;
-  config_help : string;
-  thread_model : (unit -> thread_model) option;
+  (* Connection lifecycle. *)
+  ?preconnect: (bool -> unit) ->
+  open_connection: (bool -> 'a) ->
+  ?close: ('a -> unit) ->
 
-  get_ready : (unit -> unit) option;
-  after_fork : (unit -> unit) option;
+  (* NBD negotiation. *)
+  get_size: ('a -> int64) ->
+  ?block_size: ('a -> int * int * int64) ->
+  ?can_cache: ('a -> cache_flag) ->
+  ?can_extents: ('a -> bool) ->
+  ?can_fast_zero: ('a -> bool) ->
+  ?can_flush: ('a -> bool) ->
+  ?can_fua: ('a -> fua_flag) ->
+  ?can_multi_conn: ('a -> bool) ->
+  ?can_trim: ('a -> bool) ->
+  ?can_write: ('a -> bool) ->
+  ?can_zero: ('a -> bool) ->
+  ?is_rotational: ('a -> bool) ->
 
-  preconnect : (bool -> unit) option;
-  list_exports : (bool -> bool -> export list) option;
-  default_export : (bool -> bool -> string) option;
-  open_connection : (bool -> 'a) option;          (* required *)
-  close : ('a -> unit) option;
+  (* Serving data. *)
+  pread: ('a -> int -> int64 -> flags -> string) ->
+  ?pwrite: ('a -> string -> int64 -> flags -> unit) ->
+  ?flush: ('a -> flags -> unit) ->
+  ?trim: ('a -> int64 -> int64 -> flags -> unit) ->
+  ?zero: ('a -> int64 -> int64 -> flags -> unit) ->
+  ?extents: ('a -> int64 -> int64 -> flags -> extent list) ->
+  ?cache: ('a -> int64 -> int64 -> flags -> unit) ->
 
-  get_size : ('a -> int64) option;                (* required *)
-  export_description : ('a -> string) option;
+  (* Miscellaneous. *)
+  ?dump_plugin: (unit -> unit) ->
+  ?list_exports: (bool -> bool -> export list) ->
+  ?default_export: (bool -> bool -> string) ->
+  ?export_description: ('a -> string) ->
 
-  can_cache : ('a -> cache_flag) option;
-  can_extents : ('a -> bool) option;
-  can_fast_zero : ('a -> bool) option;
-  can_flush : ('a -> bool) option;
-  can_fua : ('a -> fua_flag) option;
-  can_multi_conn : ('a -> bool) option;
-  can_trim : ('a -> bool) option;
-  can_write : ('a -> bool) option;
-  can_zero : ('a -> bool) option;
-  is_rotational : ('a -> bool) option;
-
-  pread : ('a -> int32 -> int64 -> flags -> string) option;  (* required *)
-  pwrite : ('a -> string -> int64 -> flags -> unit) option;
-  flush : ('a -> flags -> unit) option;
-  trim : ('a -> int32 -> int64 -> flags -> unit) option;
-  zero : ('a -> int32 -> int64 -> flags -> unit) option;
-  extents : ('a -> int32 -> int64 -> flags -> extent list) option;
-  cache : ('a -> int32 -> int64 -> flags -> unit) option;
-}
-
-(** The plugin with all fields set to [None], so you can write
-    [{ default_callbacks with field1 = Some foo1; field2 = Some foo2 }] *)
-val default_callbacks : 'a plugin
-
-(** Register the plugin with nbdkit. *)
-val register_plugin : 'a plugin -> unit
+  unit ->
+  unit
 
 (** Set the errno returned over the NBD protocol to the client.
 
