@@ -4,12 +4,11 @@
 
 # Run it from the build directory like this:
 #
-#   ./src/nbdkit -f -v ./plugins/ruby/.libs/nbdkit-ruby-plugin.so  \
-#       script=./plugins/ruby/example.rb test1=foo test2=bar
+#   ./nbdkit -f -v ruby ./plugins/ruby/example.rb test1=foo test2=bar
 #
 # Or run it after installing nbdkit like this:
 #
-#   nbdkit -f -v ruby script=./plugins/ruby/example.rb test1=foo test2=bar
+#   nbdkit -f -v ruby ./plugins/ruby/example.rb test1=foo test2=bar
 #
 # The -f -v arguments are optional.  They cause the server to stay in
 # the foreground and print debugging, which is useful when testing.
@@ -24,6 +23,8 @@
 #   ><fs> mount /dev/sda1 /
 #   ><fs> [etc]
 
+include Nbdkit
+
 $disk = "\0" * (1024 * 1024)
 
 def config(key, value)
@@ -50,3 +51,12 @@ def pwrite(h, buf, offset)
   # Hmm, is this using bytes or chars? XXX
   $disk[offset, buf.length] = buf
 end
+
+def zero(h, count, offset, may_trim)
+  if may_trim
+    $disk[offset, count] = "\0" * count
+  else
+    set_error(Errno::EOPNOTSUPP)
+    raise "falling back to pwrite"
+  end
+end
