@@ -29,7 +29,7 @@
 //!
 //! NBDKit is a toolkit for building Network Block Device servers.
 //!
-//! [https://github.com/libguestfs/nbdkit](https://github.com/libguestfs/nbdkit)
+//! <https://gitlab.com/nbdkit/nbdkit>
 //!
 //! To use nbdkit, build your project as a cdylib.  Define your plugin by
 //! implementing [`Server`], and then register it with [`plugin!`].  See 
@@ -64,13 +64,12 @@
 //! plugin!(MyPlugin {});
 //! # fn main() {}
 //! ```
-#![cfg_attr(feature = "nightly-docs", feature(doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![deny(missing_docs)]
 
 use bitflags::bitflags;
 #[cfg(feature = "nix")]
-#[cfg_attr(feature = "nightly-docs", doc(cfg(feature = "nix")))]
-pub use nix::sys::socket::{SockAddr, sockaddr_storage_to_addr};
+pub use nix::sys::socket::SockaddrLike;
 use std::{
     ffi::{CStr, CString},
     error,
@@ -217,9 +216,9 @@ pub enum CacheFlags {
     Native = 2,
 }
 
-impl Into<i32> for CacheFlags {
-    fn into(self) -> i32 {
-        self as i32
+impl From<CacheFlags> for i32 {
+    fn from(cf: CacheFlags) -> Self {
+        cf as Self
     }
 }
 
@@ -238,9 +237,9 @@ pub enum FuaFlags {
     Native = 2,
 }
 
-impl Into<i32> for FuaFlags {
-    fn into(self) -> i32 {
-        self as i32
+impl From<FuaFlags> for i32 {
+    fn from(cf: FuaFlags) -> Self {
+        cf as Self
     }
 }
 
@@ -369,7 +368,7 @@ mod ffi {
 
     pub(super) extern fn close(selfp: *mut c_void) {
         unsafe {
-            Box::from_raw(selfp as *mut Box<dyn Server>);
+            drop(Box::from_raw(selfp as *mut Box<dyn Server>));
         }
     }
 
@@ -812,9 +811,11 @@ pub trait Server {
     /// For security reasons (to avoid denial of service attacks) this callback
     /// should be written to be as fast and take as few resources as possible.
     /// If you use this callback, only use it to do basic access control, such
-    /// as checking [`peername`] against a whitelist.  It may be better to do
-    /// access control outside the server, for example using TCP wrappers or a
-    /// firewall.
+    /// as checking
+    #[cfg_attr(feature = "nix", doc = "[`peername`]")]
+    #[cfg_attr(not(feature = "nix"), doc = "the peer name")]
+    /// against a whitelist.  It may be better to do access control outside the
+    /// server, for example using TCP wrappers or a firewall.
     ///
     /// The `readonly` flag informs the plugin that the server was started with
     /// the `-r` flag on the command line.
@@ -963,25 +964,25 @@ impl Builder {
         });
 
         let config_help = S::config_help()
-            .map(|_| unsafe {CONFIG_HELP.as_ptr()} as *const i8)
+            .map(|_| unsafe {CONFIG_HELP.as_ptr()} as *const c_char)
             .unwrap_or(ptr::null());
         let description = S::description()
-            .map(|_| unsafe {DESCRIPTION.as_ptr()} as *const i8)
+            .map(|_| unsafe {DESCRIPTION.as_ptr()} as *const c_char)
             .unwrap_or(ptr::null());
         let longname = S::longname()
-            .map(|_| unsafe {LONGNAME.as_ptr()} as *const i8)
+            .map(|_| unsafe {LONGNAME.as_ptr()} as *const c_char)
             .unwrap_or(ptr::null());
         let magic_config_key = S::magic_config_key()
-            .map(|_| unsafe {MAGIC_CONFIG_KEY.as_ptr()} as *const i8)
+            .map(|_| unsafe {MAGIC_CONFIG_KEY.as_ptr()} as *const c_char)
             .unwrap_or(ptr::null());
         let version = S::version()
-            .map(|_| unsafe {VERSION.as_ptr()} as *const i8)
+            .map(|_| unsafe {VERSION.as_ptr()} as *const c_char)
             .unwrap_or(ptr::null());
         let plugin = Plugin {
             _struct_size: mem::size_of::<Plugin>() as u64,
             _api_version: 2,
             _thread_model: ThreadModel::Parallel as c_int,
-            name: unsafe{ NAME.as_ptr() } as *const i8,
+            name: unsafe{ NAME.as_ptr() } as *const c_char,
             longname,
             version,
             description,
@@ -1035,7 +1036,6 @@ impl Builder {
 }
 
 // C functions provided by the nbdkit binary
-// TODO: nbdkit_peer_name
 extern "C" {
     fn nbdkit_add_extent(extents: *mut c_void, offset: u64, length: u64,
                          ty: u32) -> c_int;
@@ -1081,20 +1081,23 @@ pub fn is_stdio_safe() -> bool {
 /// Note that this function must be called from one of nbdkit's own threads.
 /// That is, it can only be called in the same thread as one of the `Server`
 /// callbacks.
-#[cfg(any(feature = "nix", all(feature = "nightly-docs", rustdoc)))]
-#[cfg_attr(feature = "nightly-docs", doc(cfg(feature = "nix")))]
-pub fn peername() -> std::result::Result<SockAddr, Box<dyn error::Error>> {
-    let mut ss = mem::MaybeUninit::<libc::sockaddr_storage>::uninit();
-    let mut len = mem::size_of_val(&ss) as libc::socklen_t;
+#[cfg(feature = "nix")]
+pub fn peername<T: SockaddrLike>() -> std::result::Result<T, Box<dyn error::Error>> {
+    let mut s = mem::MaybeUninit::<T>::uninit();
+    let mut len = T::size();
     unsafe {
-        let sa = ss.as_mut_ptr() as *mut libc::sockaddr;
+        let sa = s.as_mut_ptr() as *mut libc::sockaddr;
         let r = nbdkit_peer_name(sa, &mut len as *mut _);
         if r == -1 {
             // Note that nbdkit_peer_name does _not_ set errno
             return Err("No peer name available".into());
         }
-        sockaddr_storage_to_addr(&ss.assume_init(), len as usize)
-            .map_err(|e| Box::new(e) as Box<dyn error::Error>)
+        T::from_raw(s.assume_init().as_ptr(), Some(len)).ok_or_else(|| {
+            Box::new(Error::new(
+                libc::EINVAL,
+                "Invalid sockaddr returned by nbdkit_peer_name"
+            )) as Box<dyn error::Error>
+        })
     }
 }
 
@@ -1240,7 +1243,9 @@ mod t {
     mod peername {
         use super::super::*;
         use lazy_static::lazy_static;
+        use memoffset::offset_of;
         use mockall::{mock, predicate::*};
+        use nix::sys::socket::{SockaddrIn, SockaddrIn6, UnixAddr};
         use std::sync::Mutex;
 
         lazy_static! {
@@ -1272,7 +1277,7 @@ mod t {
                 // Since nbdkit_peer_name does not set errno, all types of
                 // errors are indistinguishable to a plugin
                 .return_const(-1);
-            let e = peername().unwrap_err();
+            let e = peername::<SockaddrIn>().unwrap_err();
             ctx.checkpoint();
             assert_eq!("No peer name available", e.to_string());
         }
@@ -1284,7 +1289,7 @@ mod t {
             ctx.expect()
                 .withf(|_, len| {
                     let l = unsafe {**len as usize};
-                    l == mem::size_of::<libc::sockaddr_storage>()
+                    l == mem::size_of::<libc::sockaddr_in>()
                 }).returning(|sa, sl| {
                     let sin = sa as *mut libc::sockaddr_in;
                     unsafe {
@@ -1301,7 +1306,7 @@ mod t {
                     }
                     0
                 });
-            assert_eq!("127.0.0.1:1234", peername().unwrap().to_str());
+            assert_eq!("127.0.0.1:1234", peername::<SockaddrIn>().unwrap().to_string());
             ctx.checkpoint();
         }
 
@@ -1312,7 +1317,7 @@ mod t {
             ctx.expect()
                 .withf(|_, len| {
                        let l = unsafe {**len as usize};
-                       l == mem::size_of::<libc::sockaddr_storage>()
+                       l == mem::size_of::<libc::sockaddr_in6>()
                 }).returning(|sa, sl| {
                     let sin6 = sa as *mut libc::sockaddr_in6;
                     unsafe {
@@ -1330,7 +1335,7 @@ mod t {
                     }
                     0
                 });
-            assert_eq!("[::1]:1234", peername().unwrap().to_str());
+            assert_eq!("[::1]:1234", peername::<SockaddrIn6>().unwrap().to_string());
             ctx.checkpoint();
         }
 
@@ -1341,7 +1346,7 @@ mod t {
             ctx.expect()
                 .withf(|_, len| {
                        let l = unsafe {**len as usize};
-                       l == mem::size_of::<libc::sockaddr_storage>()
+                       l == mem::size_of::<libc::sockaddr_un>()
                 }).returning(|sa, sl| {
                     let sun = sa as *mut libc::sockaddr_un;
 
@@ -1349,14 +1354,26 @@ mod t {
                         *sun = mem::zeroed();
                         (*sun).sun_family = libc::AF_UNIX as libc::sa_family_t;
                         ptr::copy_nonoverlapping(
-                            b"/tmp/foo.sock\0".as_ptr() as *const i8,
+                            b"rust-test.sock\0".as_ptr() as *const c_char,
                             (*sun).sun_path.as_mut_ptr(), 14);
-                        *sl = mem::size_of::<libc::sockaddr_un>()
-                            as libc::socklen_t;
+                        let len = offset_of!(libc::sockaddr_un, sun_path) as u8
+                            + 14;
+                        #[cfg(any(target_os = "freebsd",
+                                  target_os = "netbsd",
+                                  target_os = "openbsd",
+                                  target_os = "macos",
+                                  target_os = "ios",
+                                  target_os = "dragonfly"
+                        ))]
+                        {
+                            (*sun).sun_len = len;
+                        }
+                        *sl = len as libc::socklen_t;
                     }
                     0
                 });
-            assert_eq!("/tmp/foo.sock", peername().unwrap().to_str());
+            assert_eq!("rust-test.sock",
+                       peername::<UnixAddr>().unwrap().to_string());
             ctx.checkpoint();
         }
     }
