@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright (C) 2018-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -56,14 +56,17 @@ sh_dump_plugin (void)
   const char *method = "dump_plugin";
   const char *script = get_script (method);
   const char *args[] = { script, method, NULL };
-  CLEANUP_FREE char *o = NULL;
-  size_t olen;
+  CLEANUP_FREE_STRING string o = empty_vector;
 
+  /* Dump information about the sh/eval features */
+  printf ("max_known_status=%d\n", RET_FALSE);
+
+  /* Dump any additional information from the script */
   if (script) {
     /* Call dump_plugin method. */
-    switch (call_read (&o, &olen, args)) {
+    switch (call_read (&o, args)) {
     case OK:
-      printf ("%s", o);
+      printf ("%s", o.ptr);
       break;
 
     case MISSING:
@@ -90,8 +93,7 @@ sh_thread_model (void)
   const char *method = "thread_model";
   const char *script = get_script (method);
   const char *args[] = { script, method, NULL };
-  CLEANUP_FREE char *s = NULL;
-  size_t slen;
+  CLEANUP_FREE_STRING string s = empty_vector;
   int r;
 
   /* For historical compatibility: the lack of a script is assumed to
@@ -102,24 +104,24 @@ sh_thread_model (void)
   if (!script)
     return NBDKIT_THREAD_MODEL_PARALLEL;
 
-  switch (call_read (&s, &slen, args)) {
+  switch (call_read (&s, args)) {
   case OK:
-    if (slen > 0 && s[slen-1] == '\n')
-      s[slen-1] = '\0';
-    if (ascii_strcasecmp (s, "parallel") == 0)
+    if (s.len > 0 && s.ptr[s.len-1] == '\n')
+      s.ptr[s.len-1] = '\0';
+    if (ascii_strcasecmp (s.ptr, "parallel") == 0)
       r = NBDKIT_THREAD_MODEL_PARALLEL;
-    else if (ascii_strcasecmp (s, "serialize_requests") == 0 ||
-             ascii_strcasecmp (s, "serialize-requests") == 0)
+    else if (ascii_strcasecmp (s.ptr, "serialize_requests") == 0 ||
+             ascii_strcasecmp (s.ptr, "serialize-requests") == 0)
       r = NBDKIT_THREAD_MODEL_SERIALIZE_REQUESTS;
-    else if (ascii_strcasecmp (s, "serialize_all_requests") == 0 ||
-             ascii_strcasecmp (s, "serialize-all-requests") == 0)
+    else if (ascii_strcasecmp (s.ptr, "serialize_all_requests") == 0 ||
+             ascii_strcasecmp (s.ptr, "serialize-all-requests") == 0)
       r = NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS;
-    else if (ascii_strcasecmp (s, "serialize_connections") == 0 ||
-             ascii_strcasecmp (s, "serialize-connections") == 0)
+    else if (ascii_strcasecmp (s.ptr, "serialize_connections") == 0 ||
+             ascii_strcasecmp (s.ptr, "serialize-connections") == 0)
       r = NBDKIT_THREAD_MODEL_SERIALIZE_CONNECTIONS;
     else {
       nbdkit_debug ("%s: ignoring unrecognized thread model: %s",
-                    script, s);
+                    script, s.ptr);
       r = NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS;
     }
     return r;
@@ -219,24 +221,169 @@ sh_preconnect (int readonly)
 }
 
 struct sh_handle {
-  char *h;
+  string h;
   int can_flush;
   int can_zero;
 };
 
+/* If @s begins with @prefix, return the next offset, else NULL */
+static const char *
+skip_prefix (const char *s, const char *prefix)
+{
+  size_t len = strlen (prefix);
+  if (strncmp (s, prefix, len) == 0)
+    return s + len;
+  return NULL;
+}
+
+static int
+parse_exports (const char *script,
+               const char *s, size_t slen, struct nbdkit_exports *exports)
+{
+  const char *n, *d, *p, *q;
+
+  /* The first line determines how to parse the rest of s.  Keep
+   * sh_default_export in sync with this.
+   */
+  if ((p = skip_prefix (s, "INTERLEAVED\n")) != NULL) {
+    n = p;
+    while ((d = strchr (n, '\n')) != NULL) {
+      p = strchr (d + 1, '\n') ?: d + 1;
+      CLEANUP_FREE char *name = strndup (n, d - n);
+      CLEANUP_FREE char *desc = strndup (d + 1, p - d - 1);
+      if (!name || !desc) {
+        nbdkit_error ("%s: strndup: %m", script);
+        return -1;
+      }
+      if (nbdkit_add_export (exports, name, desc) == -1)
+        return -1;
+      n = p + !!*p;
+    }
+  }
+  else if ((p = skip_prefix (s, "NAMES+DESCRIPTIONS\n")) != NULL) {
+    n = d = p;
+    /* Searching from both ends, using memrchr, would be less work, but
+     * memrchr is not widely portable. Multiple passes isn't too bad.
+     */
+    while (p && (p = strchr (p, '\n')) != NULL) {
+      p = strchr (p + 1, '\n');
+      if (p)
+        p++;
+      d = strchr (d, '\n') + 1;
+    }
+    s = d;
+    while (n < s) {
+      p = strchr (n, '\n');
+      q = strchr (d, '\n') ?: d;
+      CLEANUP_FREE char *name = strndup (n, p - n);
+      CLEANUP_FREE char *desc = strndup (d, q - d);
+      if (!name || !desc) {
+        nbdkit_error ("%s: strndup: %m", script);
+        return -1;
+      }
+      if (nbdkit_add_export (exports, name, desc) == -1)
+        return -1;
+      n = p + 1;
+      d = q + 1;
+    }
+  }
+  else {
+    n = skip_prefix (s, "NAMES\n") ?: s;
+    while ((p = strchr (n, '\n')) != NULL) {
+      CLEANUP_FREE char *name = strndup (n, p - n);
+      if (!name) {
+        nbdkit_error ("%s: strndup: %m", script);
+        return -1;
+      }
+      if (nbdkit_add_export (exports, name, NULL) == -1)
+        return -1;
+      n = p + 1;
+    }
+  }
+  return 0;
+}
+
+int
+sh_list_exports (int readonly, int is_tls, struct nbdkit_exports *exports)
+{
+  const char *method = "list_exports";
+  const char *script = get_script (method);
+  const char *args[] = { script, method, readonly ? "true" : "false",
+                         is_tls ? "true" : "false", NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
+
+  switch (call_read (&s, args)) {
+  case OK:
+    return parse_exports (script, s.ptr, s.len, exports);
+
+  case MISSING:
+    return nbdkit_use_default_export (exports);
+
+  case ERROR:
+    return -1;
+
+  case RET_FALSE:
+    nbdkit_error ("%s: %s method returned unexpected code (3/false)",
+                  script, method);
+    errno = EIO;
+    return -1;
+
+  default: abort ();
+  }
+}
+
+const char *
+sh_default_export (int readonly, int is_tls)
+{
+  const char *method = "default_export";
+  const char *script = get_script (method);
+  const char *args[] = { script, method, readonly ? "true" : "false",
+                         is_tls ? "true" : "false", NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
+  const char *p, *n;
+
+  switch (call_read (&s, args)) {
+  case OK:
+    /* The first line determines how to parse the rest of s.  For now,
+     * all export modes treat the next line as the first export.
+     */
+    if ((p = skip_prefix (s.ptr, "INTERLEAVED\n")) != NULL ||
+        (p = skip_prefix (s.ptr, "NAMES+DESCRIPTIONS\n")) != NULL ||
+        (p = skip_prefix (s.ptr, "NAMES\n")) != NULL)
+      ;
+    else
+      p = s.ptr;
+    n = strchr (p, '\n') ?: s.ptr + s.len;
+    return nbdkit_strndup_intern (p, n - p);
+
+  case MISSING:
+    return "";
+
+  case ERROR:
+    return NULL;
+
+  case RET_FALSE:
+    nbdkit_error ("%s: %s method returned unexpected code (3/false)",
+                  script, method);
+    errno = EIO;
+    return NULL;
+
+  default: abort ();
+  }
+}
+
 void *
 sh_open (int readonly)
 {
   const char *method = "open";
   const char *script = get_script (method);
-  size_t hlen;
   const char *args[] =
     { script, method,
       readonly ? "true" : "false",
       nbdkit_export_name () ? : "",
-      nbdkit_is_tls () ? "true" : "false",
+      nbdkit_is_tls () > 0 ? "true" : "false",
       NULL };
-  struct sh_handle *h = malloc (sizeof *h);
+  struct sh_handle *h = calloc (1, sizeof *h);
 
   if (!h) {
     nbdkit_error ("malloc: %m");
@@ -246,15 +393,13 @@ sh_open (int readonly)
   h->can_zero = -1;
 
   /* We store the string returned by open in the handle. */
-  switch (call_read (&h->h, &hlen, args)) {
+  switch (call_read (&h->h, args)) {
   case OK:
     /* Remove final newline if present. */
-    if (hlen > 0 && h->h[hlen-1] == '\n') {
-      h->h[hlen-1] = '\0';
-      hlen--;
-    }
-    if (hlen > 0)
-      nbdkit_debug ("sh: handle: %s", h->h);
+    if (h->h.len > 0 && h->h.ptr[h->h.len-1] == '\n')
+      h->h.ptr[--h->h.len] = '\0';
+    if (h->h.len > 0)
+      nbdkit_debug ("sh: handle: %s", h->h.ptr);
     return h;
 
   case MISSING:
@@ -262,22 +407,22 @@ sh_open (int readonly)
      * missing then we return "" as the handle.  Allocate a new string
      * for it because we don't know what call_read returned here.
      */
-    free (h->h);
-    h->h = strdup ("");
-    if (h->h == NULL) {
-      nbdkit_error ("strdup: %m");
+    string_reset (&h->h);
+    if (string_reserve (&h->h, 1) == -1) {
+      nbdkit_error ("realloc: %m");
       free (h);
       return NULL;
     }
+    h->h.ptr[0] = '\0';
     return h;
 
   case ERROR:
-    free (h->h);
+    string_reset (&h->h);
     free (h);
     return NULL;
 
   case RET_FALSE:
-    free (h->h);
+    string_reset (&h->h);
     free (h);
     nbdkit_error ("%s: %s method returned unexpected code (3/false)",
                   script, method);
@@ -294,39 +439,69 @@ sh_close (void *handle)
   const char *method = "close";
   const char *script = get_script (method);
   struct sh_handle *h = handle;
-  const char *args[] = { script, method, h->h, NULL };
+  const char *args[] = { script, method, h->h.ptr, NULL };
 
   switch (call (args)) {
   case OK:
   case MISSING:
   case ERROR:
   case RET_FALSE:
-    free (h->h);
+    string_reset (&h->h);
     free (h);
     return;
   default: abort ();
   }
 }
 
+const char *
+sh_export_description (void *handle)
+{
+  const char *method = "export_description";
+  const char *script = get_script (method);
+  struct sh_handle *h = handle;
+  const char *args[] = { script, method, h->h.ptr, NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
+
+  switch (call_read (&s, args)) {
+  case OK:
+    if (s.len > 0 && s.ptr[s.len-1] == '\n')
+      s.ptr[s.len-1] = '\0';
+    return nbdkit_strdup_intern (s.ptr);
+
+  case MISSING:
+    return NULL;
+
+  case ERROR:
+    return NULL;
+
+  case RET_FALSE:
+    nbdkit_error ("%s: %s method returned unexpected code (3/false)",
+                  script, method);
+    errno = EIO;
+    return NULL;
+
+  default: abort ();
+  }
+}
+
 int64_t
 sh_get_size (void *handle)
 {
   const char *method = "get_size";
   const char *script = get_script (method);
   struct sh_handle *h = handle;
-  const char *args[] = { script, method, h->h, NULL };
-  CLEANUP_FREE char *s = NULL;
-  size_t slen;
+  const char *args[] = { script, method, h->h.ptr, NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
   int64_t r;
 
-  switch (call_read (&s, &slen, args)) {
+  switch (call_read (&s, args)) {
   case OK:
-    if (slen > 0 && s[slen-1] == '\n')
-      s[slen-1] = '\0';
-    r = nbdkit_parse_size (s);
+    if (s.len > 0 && s.ptr[s.len-1] == '\n')
+      s.ptr[s.len-1] = '\0';
+    r = nbdkit_parse_size (s.ptr);
     if (r == -1)
       nbdkit_error ("%s: could not parse output from get_size method: %s",
-                    script, s);
+                    script, s.ptr);
     return r;
 
   case MISSING:
@@ -346,6 +521,71 @@ sh_get_size (void *handle)
   }
 }
 
+int
+sh_block_size (void *handle,
+               uint32_t *minimum, uint32_t *preferred, uint32_t *maximum)
+{
+  const char *method = "block_size";
+  const char *script = get_script (method);
+  struct sh_handle *h = handle;
+  const char *args[] = { script, method, h->h.ptr, NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
+  const char *delim = " \t\n";
+  char *sp, *p;
+  int64_t r;
+
+  switch (call_read (&s, args)) {
+  case OK:
+    if ((p = strtok_r (s.ptr, delim, &sp)) == NULL) {
+    parse_error:
+      nbdkit_error ("%s: %s method cannot be parsed", script, method);
+      return -1;
+    }
+    r = nbdkit_parse_size (p);
+    if (r == -1 || r > UINT32_MAX)
+      goto parse_error;
+    *minimum = r;
+
+    if ((p = strtok_r (NULL, delim, &sp)) == NULL)
+      goto parse_error;
+    r = nbdkit_parse_size (p);
+    if (r == -1 || r > UINT32_MAX)
+      goto parse_error;
+    *preferred = r;
+
+    if ((p = strtok_r (NULL, delim, &sp)) == NULL)
+      goto parse_error;
+    r = nbdkit_parse_size (p);
+    if (r == -1 || r > UINT32_MAX)
+      goto parse_error;
+    *maximum = r;
+
+#if 0
+    nbdkit_debug ("setting block_size: "
+                  "minimum=%" PRIu32 " "
+                  "preferred=%" PRIu32 " "
+                  "maximum=%" PRIu32,
+                  *minimum, *preferred, *maximum);
+#endif
+    return 0;
+
+  case MISSING:
+    *minimum = *preferred = *maximum = 0;
+    return 0;
+
+  case ERROR:
+    return -1;
+
+  case RET_FALSE:
+    nbdkit_error ("%s: %s method returned unexpected code (3/false)",
+                  script, method);
+    errno = EIO;
+    return -1;
+
+  default: abort ();
+  }
+}
+
 int
 sh_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
           uint32_t flags)
@@ -354,23 +594,22 @@ sh_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
   const char *script = get_script (method);
   struct sh_handle *h = handle;
   char cbuf[32], obuf[32];
-  const char *args[] = { script, method, h->h, cbuf, obuf, NULL };
-  CLEANUP_FREE char *data = NULL;
-  size_t len;
+  const char *args[] = { script, method, h->h.ptr, cbuf, obuf, NULL };
+  CLEANUP_FREE_STRING string data = empty_vector;
 
   snprintf (cbuf, sizeof cbuf, "%" PRIu32, count);
   snprintf (obuf, sizeof obuf, "%" PRIu64, offset);
 
-  switch (call_read (&data, &len, args)) {
+  switch (call_read (&data, args)) {
   case OK:
-    if (count != len) {
+    if (count != data.len) {
       nbdkit_error ("%s: incorrect amount of data read: "
                     "expecting %" PRIu32 " bytes but "
                     "received %zu bytes from the script",
-                    script, count, len);
+                    script, count, data.len);
       return -1;
     }
-    memcpy (buf, data, count);
+    memcpy (buf, data.ptr, count);
     return 0;
 
   case MISSING:
@@ -442,7 +681,7 @@ sh_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
   const char *script = get_script (method);
   struct sh_handle *h = handle;
   char cbuf[32], obuf[32], fbuf[32];
-  const char *args[] = { script, method, h->h, cbuf, obuf, fbuf, NULL };
+  const char *args[] = { script, method, h->h.ptr, cbuf, obuf, fbuf, NULL };
 
   snprintf (cbuf, sizeof cbuf, "%" PRIu32, count);
   snprintf (obuf, sizeof obuf, "%" PRIu64, offset);
@@ -475,7 +714,7 @@ boolean_method (const char *script, const char *method,
                 void *handle, int def)
 {
   struct sh_handle *h = handle;
-  const char *args[] = { script, method, h->h, NULL };
+  const char *args[] = { script, method, h->h.ptr, NULL };
 
   switch (call (args)) {
   case OK:                      /* true */
@@ -565,24 +804,23 @@ sh_can_fua (void *handle)
   const char *method = "can_fua";
   const char *script = get_script (method);
   struct sh_handle *h = handle;
-  const char *args[] = { script, method, h->h, NULL };
-  CLEANUP_FREE char *s = NULL;
-  size_t slen;
+  const char *args[] = { script, method, h->h.ptr, NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
   int r;
 
-  switch (call_read (&s, &slen, args)) {
+  switch (call_read (&s, args)) {
   case OK:
-    if (slen > 0 && s[slen-1] == '\n')
-      s[slen-1] = '\0';
-    if (ascii_strcasecmp (s, "none") == 0)
+    if (s.len > 0 && s.ptr[s.len-1] == '\n')
+      s.ptr[s.len-1] = '\0';
+    if (ascii_strcasecmp (s.ptr, "none") == 0)
       r = NBDKIT_FUA_NONE;
-    else if (ascii_strcasecmp (s, "emulate") == 0)
+    else if (ascii_strcasecmp (s.ptr, "emulate") == 0)
       r = NBDKIT_FUA_EMULATE;
-    else if (ascii_strcasecmp (s, "native") == 0)
+    else if (ascii_strcasecmp (s.ptr, "native") == 0)
       r = NBDKIT_FUA_NATIVE;
     else {
       nbdkit_error ("%s: could not parse output from %s method: %s",
-                    script, method, s);
+                    script, method, s.ptr);
       r = -1;
     }
     return r;
@@ -620,24 +858,23 @@ sh_can_cache (void *handle)
   const char *method = "can_cache";
   const char *script = get_script (method);
   struct sh_handle *h = handle;
-  const char *args[] = { script, method, h->h, NULL };
-  CLEANUP_FREE char *s = NULL;
-  size_t slen;
+  const char *args[] = { script, method, h->h.ptr, NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
   int r;
 
-  switch (call_read (&s, &slen, args)) {
+  switch (call_read (&s, args)) {
   case OK:
-    if (slen > 0 && s[slen-1] == '\n')
-      s[slen-1] = '\0';
-    if (ascii_strcasecmp (s, "none") == 0)
+    if (s.len > 0 && s.ptr[s.len-1] == '\n')
+      s.ptr[s.len-1] = '\0';
+    if (ascii_strcasecmp (s.ptr, "none") == 0)
       r = NBDKIT_CACHE_NONE;
-    else if (ascii_strcasecmp (s, "emulate") == 0)
+    else if (ascii_strcasecmp (s.ptr, "emulate") == 0)
       r = NBDKIT_CACHE_EMULATE;
-    else if (ascii_strcasecmp (s, "native") == 0)
+    else if (ascii_strcasecmp (s.ptr, "native") == 0)
       r = NBDKIT_CACHE_NATIVE;
     else {
       nbdkit_error ("%s: could not parse output from %s method: %s",
-                    script, method, s);
+                    script, method, s.ptr);
       r = -1;
     }
     return r;
@@ -685,7 +922,7 @@ sh_flush (void *handle, uint32_t flags)
   const char *method = "flush";
   const char *script = get_script (method);
   struct sh_handle *h = handle;
-  const char *args[] = { script, method, h->h, NULL };
+  const char *args[] = { script, method, h->h.ptr, NULL };
 
   switch (call (args)) {
   case OK:
@@ -715,7 +952,7 @@ sh_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   const char *script = get_script (method);
   struct sh_handle *h = handle;
   char cbuf[32], obuf[32], fbuf[32];
-  const char *args[] = { script, method, h->h, cbuf, obuf, fbuf, NULL };
+  const char *args[] = { script, method, h->h.ptr, cbuf, obuf, fbuf, NULL };
 
   snprintf (cbuf, sizeof cbuf, "%" PRIu32, count);
   snprintf (obuf, sizeof obuf, "%" PRIu64, offset);
@@ -749,7 +986,7 @@ sh_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   const char *script = get_script (method);
   struct sh_handle *h = handle;
   char cbuf[32], obuf[32], fbuf[32];
-  const char *args[] = { script, method, h->h, cbuf, obuf, fbuf, NULL };
+  const char *args[] = { script, method, h->h.ptr, cbuf, obuf, fbuf, NULL };
 
   snprintf (cbuf, sizeof cbuf, "%" PRIu32, count);
   snprintf (obuf, sizeof obuf, "%" PRIu64, offset);
@@ -854,18 +1091,17 @@ sh_extents (void *handle, uint32_t count, uint64_t offset, uint32_t flags,
   const char *script = get_script (method);
   struct sh_handle *h = handle;
   char cbuf[32], obuf[32], fbuf[32];
-  const char *args[] = { script, method, h->h, cbuf, obuf, fbuf, NULL };
-  CLEANUP_FREE char *s = NULL;
-  size_t slen;
+  const char *args[] = { script, method, h->h.ptr, cbuf, obuf, fbuf, NULL };
+  CLEANUP_FREE_STRING string s = empty_vector;
   int r;
 
   snprintf (cbuf, sizeof cbuf, "%" PRIu32, count);
   snprintf (obuf, sizeof obuf, "%" PRIu64, offset);
   flags_string (flags, fbuf, sizeof fbuf);
 
-  switch (call_read (&s, &slen, args)) {
+  switch (call_read (&s, args)) {
   case OK:
-    r = parse_extents (script, s, slen, extents);
+    r = parse_extents (script, s.ptr, s.len, extents);
     return r;
 
   case MISSING:
@@ -899,7 +1135,7 @@ sh_cache (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   const char *script = get_script (method);
   struct sh_handle *h = handle;
   char cbuf[32], obuf[32];
-  const char *args[] = { script, method, h->h, cbuf, obuf, NULL };
+  const char *args[] = { script, method, h->h.ptr, cbuf, obuf, NULL };
 
   snprintf (cbuf, sizeof cbuf, "%" PRIu32, count);
   snprintf (obuf, sizeof obuf, "%" PRIu64, offset);
