@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright (C) 2013-2021 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -51,8 +51,8 @@
  * -D nbdkit.backend.controlpath=0 to suppress control path debugging.
  * -D nbdkit.backend.datapath=0 to suppress data path debugging.
  */
-SERVER_DEBUG_FLAG(nbdkit_debug_backend_controlpath) = 1;
-SERVER_DEBUG_FLAG(nbdkit_debug_backend_datapath) = 1;
+NBDKIT_DLL_PUBLIC int nbdkit_debug_backend_controlpath = 1;
+NBDKIT_DLL_PUBLIC int nbdkit_debug_backend_datapath = 1;
 
 #define controlpath_debug(fs, ...)                                     \
   do {                                                                 \
@@ -163,14 +163,12 @@ backend_list_exports (struct backend *b, int readonly,
                       struct nbdkit_exports *exports)
 {
   GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
   size_t count;
 
   controlpath_debug ("%s: list_exports readonly=%d tls=%d",
                      b->name, readonly, conn->using_tls);
 
-  assert (h->handle == NULL);
-  assert ((h->state & HANDLE_OPEN) == 0);
+  assert (conn->top_context == NULL);
 
   if (b->list_exports (b, readonly, conn->using_tls, exports) == -1 ||
       exports_resolve_default (exports, b, readonly) == -1) {
@@ -187,15 +185,13 @@ const char *
 backend_default_export (struct backend *b, int readonly)
 {
   GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
   const char *s;
 
   controlpath_debug ("%s: default_export readonly=%d tls=%d",
                      b->name, readonly, conn->using_tls);
 
-  if (h->default_exportname == NULL) {
-    assert (h->handle == NULL);
-    assert ((h->state & HANDLE_OPEN) == 0);
+  if (conn->default_exportname[b->i] == NULL) {
+    assert (conn->top_context == NULL);
     s = b->default_export (b, readonly, conn->using_tls);
     /* Ignore over-length strings. XXX Also ignore non-UTF8? */
     if (s && strnlen (s, NBD_MAX_STRING + 1) > NBD_MAX_STRING) {
@@ -205,173 +201,201 @@ backend_default_export (struct backend *b, int readonly)
     }
     if (s) {
       /* Best effort caching */
-      h->default_exportname = strdup (s);
-      if (h->default_exportname == NULL)
+      conn->default_exportname[b->i] = strdup (s);
+      if (conn->default_exportname[b->i] == NULL)
         return s;
     }
   }
-  return h->default_exportname;
+  return conn->default_exportname[b->i];
 }
 
-int
-backend_open (struct backend *b, int readonly, const char *exportname)
+static struct nbdkit_next_ops next_ops = {
+  .prepare = backend_prepare,
+  .finalize = backend_finalize,
+  .export_description = backend_export_description,
+  .get_size = backend_get_size,
+  .block_size = backend_block_size,
+  .can_write = backend_can_write,
+  .can_flush = backend_can_flush,
+  .is_rotational = backend_is_rotational,
+  .can_trim = backend_can_trim,
+  .can_zero = backend_can_zero,
+  .can_fast_zero = backend_can_fast_zero,
+  .can_extents = backend_can_extents,
+  .can_fua = backend_can_fua,
+  .can_multi_conn = backend_can_multi_conn,
+  .can_cache = backend_can_cache,
+  .pread = backend_pread,
+  .pwrite = backend_pwrite,
+  .flush = backend_flush,
+  .trim = backend_trim,
+  .zero = backend_zero,
+  .extents = backend_extents,
+  .cache = backend_cache,
+};
+
+struct context *
+backend_open (struct backend *b, int readonly, const char *exportname,
+              int shared)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  struct connection *conn = threadlocal_get_conn ();
+  bool using_tls;
+  struct context *c;
+
+  if (shared)
+    using_tls = tls == 2;
+  else {
+    assert (conn);
+    using_tls = conn->using_tls;
+  }
+
+  c = malloc (sizeof *c);
+  if (c == NULL) {
+    nbdkit_error ("malloc: %m");
+    return NULL;
+  }
+  PUSH_CONTEXT_FOR_SCOPE (c);
 
   controlpath_debug ("%s: open readonly=%d exportname=\"%s\" tls=%d",
-                     b->name, readonly, exportname, conn->using_tls);
+                     b->name, readonly, exportname, using_tls);
 
-  assert (h->handle == NULL);
-  assert ((h->state & HANDLE_OPEN) == 0);
-  assert (h->can_write == -1);
-  if (readonly)
-    h->can_write = 0;
+  c->next = next_ops;
+  c->handle = NULL;
+  c->b = b;
+  c->c_next = NULL;
+  c->conn = shared ? NULL : conn;
+  c->state = 0;
+  c->exportsize = -1;
+  c->minimum_block_size = c->preferred_block_size = c->maximum_block_size = -1;
+  c->can_write = readonly ? 0 : -1;
+  c->can_flush = -1;
+  c->is_rotational = -1;
+  c->can_trim = -1;
+  c->can_zero = -1;
+  c->can_fast_zero = -1;
+  c->can_fua = -1;
+  c->can_multi_conn = -1;
+  c->can_extents = -1;
+  c->can_cache = -1;
 
   /* Determine the canonical name for default export */
-  if (!*exportname) {
+  if (!*exportname && c->conn) {
     exportname = backend_default_export (b, readonly);
     if (exportname == NULL) {
       nbdkit_error ("default export (\"\") not permitted");
-      return -1;
+      free (c);
+      return NULL;
     }
   }
 
   /* Most filters will call next_open first, resulting in
    * inner-to-outer ordering.
    */
-  h->handle = b->open (b, readonly, exportname, conn->using_tls);
-  controlpath_debug ("%s: open returned handle %p", b->name, h->handle);
+  c->handle = b->open (c, readonly, exportname, using_tls);
+  controlpath_debug ("%s: open returned handle %p", b->name, c->handle);
 
-  if (h->handle == NULL) {
-    if (b->i) /* Do not strand backend if this layer failed */
-      backend_close (b->next);
-    return -1;
+  if (c->handle == NULL) {
+    if (b->i && c->c_next != NULL)
+      backend_close (c->c_next);
+    free (c);
+    return NULL;
   }
 
-  h->state |= HANDLE_OPEN;
-  return 0;
+  c->state |= HANDLE_OPEN;
+  return c;
 }
 
 int
-backend_prepare (struct backend *b)
+backend_prepare (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
 
-  assert (h->handle);
-  assert ((h->state & (HANDLE_OPEN | HANDLE_CONNECTED)) == HANDLE_OPEN);
+  assert (c->handle);
+  assert ((c->state & (HANDLE_OPEN | HANDLE_CONNECTED)) == HANDLE_OPEN);
 
   /* Call these in order starting from the filter closest to the
    * plugin, similar to typical .open order.  But remember that
    * a filter may skip opening its backend.
    */
-  if (b->i && get_handle (conn, b->i-1)->handle != NULL &&
-      backend_prepare (b->next) == -1)
+  if (b->i && c->c_next != NULL && backend_prepare (c->c_next) == -1)
     return -1;
 
-  controlpath_debug ("%s: prepare readonly=%d", b->name, h->can_write == 0);
+  controlpath_debug ("%s: prepare readonly=%d", b->name, c->can_write == 0);
 
-  if (b->prepare (b, h->handle, h->can_write == 0) == -1)
+  if (b->prepare (c, c->can_write == 0) == -1)
     return -1;
-  h->state |= HANDLE_CONNECTED;
+  c->state |= HANDLE_CONNECTED;
   return 0;
 }
 
 int
-backend_finalize (struct backend *b)
+backend_finalize (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
 
   /* Call these in reverse order to .prepare above, starting from the
    * filter furthest away from the plugin, and matching .close order.
    */
 
   /* Once finalize fails, we can do nothing further on this connection */
-  if (h->state & HANDLE_FAILED)
+  if (c->state & HANDLE_FAILED)
     return -1;
 
-  if (h->state & HANDLE_CONNECTED) {
-    assert (h->state & HANDLE_OPEN && h->handle);
+  if (c->state & HANDLE_CONNECTED) {
+    assert (c->state & HANDLE_OPEN && c->handle);
     controlpath_debug ("%s: finalize", b->name);
-    if (b->finalize (b, h->handle) == -1) {
-      h->state |= HANDLE_FAILED;
+    if (b->finalize (c) == -1) {
+      c->state |= HANDLE_FAILED;
       return -1;
     }
   }
 
-  if (b->i)
-    return backend_finalize (b->next);
+  if (b->i && c->c_next != NULL)
+    return backend_finalize (c->c_next);
   return 0;
 }
 
 void
-backend_close (struct backend *b)
+backend_close (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
+  struct context *c_next = c->c_next;
 
   /* outer-to-inner order, opposite .open */
-
-  if (h->handle) {
-    assert (h->state & HANDLE_OPEN);
-    controlpath_debug ("%s: close", b->name);
-    b->close (b, h->handle);
-  }
-  else
-    assert (! (h->state & HANDLE_OPEN));
-  reset_handle (h);
-  if (b->i)
-    backend_close (b->next);
+  assert (c->handle);
+  assert (c->state & HANDLE_OPEN);
+  controlpath_debug ("%s: close", b->name);
+  b->close (c);
+  free (c);
+  if (c_next != NULL)
+    backend_close (c_next);
 }
 
 bool
-backend_valid_range (struct backend *b, uint64_t offset, uint32_t count)
+backend_valid_range (struct context *c, uint64_t offset, uint32_t count)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
-
-  assert (h->exportsize <= INT64_MAX); /* Guaranteed by negotiation phase */
-  return count > 0 && offset <= h->exportsize &&
-    offset + count <= h->exportsize;
+  assert (c->exportsize <= INT64_MAX); /* Guaranteed by negotiation phase */
+  return count > 0 && offset <= c->exportsize &&
+    offset + count <= c->exportsize;
 }
 
 /* Wrappers for all callbacks in a filter's struct nbdkit_next_ops. */
 
-int
-backend_reopen (struct backend *b, int readonly, const char *exportname)
-{
-  controlpath_debug ("%s: reopen readonly=%d exportname=\"%s\"",
-                     b->name, readonly, exportname);
-
-  if (backend_finalize (b) == -1)
-    return -1;
-  backend_close (b);
-  if (backend_open (b, readonly, exportname) == -1) {
-    backend_close (b);
-    return -1;
-  }
-  if (backend_prepare (b) == -1) {
-    backend_finalize (b);
-    backend_close (b);
-    return -1;
-  }
-  return 0;
-}
-
 const char *
-backend_export_description (struct backend *b)
+backend_export_description (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   const char *s;
 
   controlpath_debug ("%s: export_description", b->name);
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
   /* Caching is not useful for this value. */
-  s = b->export_description (b, h->handle);
+  s = b->export_description (c);
 
   /* Ignore over-length strings. XXX Also ignore non-UTF8? */
   if (s && strnlen (s, NBD_MAX_STRING + 1) > NBD_MAX_STRING) {
@@ -383,301 +407,363 @@ backend_export_description (struct backend *b)
 }
 
 int64_t
-backend_get_size (struct backend *b)
+backend_get_size (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->exportsize == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->exportsize == -1) {
     controlpath_debug ("%s: get_size", b->name);
-    h->exportsize = b->get_size (b, h->handle);
+    c->exportsize = b->get_size (c);
   }
-  return h->exportsize;
+  return c->exportsize;
 }
 
 int
-backend_can_write (struct backend *b)
+backend_block_size (struct context *c,
+                    uint32_t *minimum, uint32_t *preferred, uint32_t *maximum)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
-
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_write == -1) {
-    controlpath_debug ("%s: can_write", b->name);
-    h->can_write = b->can_write (b, h->handle);
-  }
-  return h->can_write;
-}
-
-int
-backend_can_flush (struct backend *b)
-{
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
-
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_flush == -1) {
-    controlpath_debug ("%s: can_flush", b->name);
-    h->can_flush = b->can_flush (b, h->handle);
-  }
-  return h->can_flush;
-}
-
-int
-backend_is_rotational (struct backend *b)
-{
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
-
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->is_rotational == -1) {
-    controlpath_debug ("%s: is_rotational", b->name);
-    h->is_rotational = b->is_rotational (b, h->handle);
-  }
-  return h->is_rotational;
-}
-
-int
-backend_can_trim (struct backend *b)
-{
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_trim == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->minimum_block_size != -1) {
+    *minimum = c->minimum_block_size;
+    *preferred = c->preferred_block_size;
+    *maximum = c->maximum_block_size;
+    return 0;
+  }
+  else {
+    controlpath_debug ("%s: block_size", b->name);
+    r = b->block_size (c, minimum, preferred, maximum);
+    if (r == 0) {
+      c->minimum_block_size = *minimum;
+      c->preferred_block_size = *preferred;
+      c->maximum_block_size = *maximum;
+    }
+    return r;
+  }
+}
+
+int
+backend_can_write (struct context *c)
+{
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
+
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_write == -1) {
+    controlpath_debug ("%s: can_write", b->name);
+    c->can_write = b->can_write (c);
+  }
+  return c->can_write;
+}
+
+int
+backend_can_flush (struct context *c)
+{
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
+
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_flush == -1) {
+    controlpath_debug ("%s: can_flush", b->name);
+    c->can_flush = b->can_flush (c);
+  }
+  return c->can_flush;
+}
+
+int
+backend_is_rotational (struct context *c)
+{
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
+
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->is_rotational == -1) {
+    controlpath_debug ("%s: is_rotational", b->name);
+    c->is_rotational = b->is_rotational (c);
+  }
+  return c->is_rotational;
+}
+
+int
+backend_can_trim (struct context *c)
+{
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
+  int r;
+
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_trim == -1) {
     controlpath_debug ("%s: can_trim", b->name);
-    r = backend_can_write (b);
+    r = backend_can_write (c);
     if (r != 1) {
-      h->can_trim = 0;
+      c->can_trim = 0;
       return r;
     }
-    h->can_trim = b->can_trim (b, h->handle);
+    c->can_trim = b->can_trim (c);
   }
-  return h->can_trim;
+  return c->can_trim;
 }
 
 int
-backend_can_zero (struct backend *b)
+backend_can_zero (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_zero == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_zero == -1) {
     controlpath_debug ("%s: can_zero", b->name);
-    r = backend_can_write (b);
+    r = backend_can_write (c);
     if (r != 1) {
-      h->can_zero = NBDKIT_ZERO_NONE;
+      c->can_zero = NBDKIT_ZERO_NONE;
       return r; /* Relies on 0 == NBDKIT_ZERO_NONE */
     }
-    h->can_zero = b->can_zero (b, h->handle);
+    c->can_zero = b->can_zero (c);
   }
-  return h->can_zero;
+  return c->can_zero;
 }
 
 int
-backend_can_fast_zero (struct backend *b)
+backend_can_fast_zero (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_fast_zero == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_fast_zero == -1) {
     controlpath_debug ("%s: can_fast_zero", b->name);
-    r = backend_can_zero (b);
+    r = backend_can_zero (c);
     if (r < NBDKIT_ZERO_EMULATE) {
-      h->can_fast_zero = 0;
+      c->can_fast_zero = 0;
       return r; /* Relies on 0 == NBDKIT_ZERO_NONE */
     }
-    h->can_fast_zero = b->can_fast_zero (b, h->handle);
+    c->can_fast_zero = b->can_fast_zero (c);
   }
-  return h->can_fast_zero;
+  return c->can_fast_zero;
 }
 
 int
-backend_can_extents (struct backend *b)
+backend_can_extents (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_extents == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_extents == -1) {
     controlpath_debug ("%s: can_extents", b->name);
-    h->can_extents = b->can_extents (b, h->handle);
+    c->can_extents = b->can_extents (c);
   }
-  return h->can_extents;
+  return c->can_extents;
 }
 
 int
-backend_can_fua (struct backend *b)
+backend_can_fua (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_fua == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_fua == -1) {
     controlpath_debug ("%s: can_fua", b->name);
-    r = backend_can_write (b);
+    r = backend_can_write (c);
     if (r != 1) {
-      h->can_fua = NBDKIT_FUA_NONE;
+      c->can_fua = NBDKIT_FUA_NONE;
       return r; /* Relies on 0 == NBDKIT_FUA_NONE */
     }
-    h->can_fua = b->can_fua (b, h->handle);
+    c->can_fua = b->can_fua (c);
   }
-  return h->can_fua;
+  return c->can_fua;
 }
 
 int
-backend_can_multi_conn (struct backend *b)
+backend_can_multi_conn (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_multi_conn == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_multi_conn == -1) {
     controlpath_debug ("%s: can_multi_conn", b->name);
-    h->can_multi_conn = b->can_multi_conn (b, h->handle);
+    c->can_multi_conn = b->can_multi_conn (c);
   }
-  return h->can_multi_conn;
+  return c->can_multi_conn;
 }
 
 int
-backend_can_cache (struct backend *b)
+backend_can_cache (struct context *c)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  if (h->can_cache == -1) {
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  if (c->can_cache == -1) {
     controlpath_debug ("%s: can_cache", b->name);
-    h->can_cache = b->can_cache (b, h->handle);
+    c->can_cache = b->can_cache (c);
   }
-  return h->can_cache;
+  return c->can_cache;
 }
 
 int
-backend_pread (struct backend *b,
+backend_pread (struct context *c,
                void *buf, uint32_t count, uint64_t offset,
                uint32_t flags, int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (backend_valid_range (b, offset, count));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (backend_valid_range (c, offset, count));
   assert (flags == 0);
   datapath_debug ("%s: pread count=%" PRIu32 " offset=%" PRIu64,
                   b->name, count, offset);
 
-  r = b->pread (b, h->handle, buf, count, offset, flags, err);
+  r = b->pread (c, buf, count, offset, flags, err);
   if (r == -1)
     assert (*err);
   return r;
 }
 
 int
-backend_pwrite (struct backend *b,
+backend_pwrite (struct context *c,
                 const void *buf, uint32_t count, uint64_t offset,
                 uint32_t flags, int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   bool fua = !!(flags & NBDKIT_FLAG_FUA);
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (h->can_write == 1);
-  assert (backend_valid_range (b, offset, count));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (c->can_write == 1);
+  assert (backend_valid_range (c, offset, count));
   assert (!(flags & ~NBDKIT_FLAG_FUA));
   if (fua)
-    assert (h->can_fua > NBDKIT_FUA_NONE);
+    assert (c->can_fua > NBDKIT_FUA_NONE);
   datapath_debug ("%s: pwrite count=%" PRIu32 " offset=%" PRIu64 " fua=%d",
                   b->name, count, offset, fua);
 
-  r = b->pwrite (b, h->handle, buf, count, offset, flags, err);
+  r = b->pwrite (c, buf, count, offset, flags, err);
   if (r == -1)
     assert (*err);
   return r;
 }
 
 int
-backend_flush (struct backend *b,
+backend_flush (struct context *c,
                uint32_t flags, int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (h->can_flush == 1);
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (c->can_flush == 1);
   assert (flags == 0);
   datapath_debug ("%s: flush", b->name);
 
-  r = b->flush (b, h->handle, flags, err);
+  r = b->flush (c, flags, err);
   if (r == -1)
     assert (*err);
   return r;
 }
 
 int
-backend_trim (struct backend *b,
+backend_trim (struct context *c,
               uint32_t count, uint64_t offset, uint32_t flags,
               int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   bool fua = !!(flags & NBDKIT_FLAG_FUA);
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (h->can_write == 1);
-  assert (h->can_trim == 1);
-  assert (backend_valid_range (b, offset, count));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (c->can_write == 1);
+  assert (c->can_trim == 1);
+  assert (backend_valid_range (c, offset, count));
   assert (!(flags & ~NBDKIT_FLAG_FUA));
   if (fua)
-    assert (h->can_fua > NBDKIT_FUA_NONE);
+    assert (c->can_fua > NBDKIT_FUA_NONE);
   datapath_debug ("%s: trim count=%" PRIu32 " offset=%" PRIu64 " fua=%d",
                   b->name, count, offset, fua);
 
-  r = b->trim (b, h->handle, count, offset, flags, err);
+  r = b->trim (c, count, offset, flags, err);
   if (r == -1)
     assert (*err);
   return r;
 }
 
 int
-backend_zero (struct backend *b,
+backend_zero (struct context *c,
               uint32_t count, uint64_t offset, uint32_t flags,
               int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   bool fua = !!(flags & NBDKIT_FLAG_FUA);
   bool fast = !!(flags & NBDKIT_FLAG_FAST_ZERO);
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (h->can_write == 1);
-  assert (h->can_zero > NBDKIT_ZERO_NONE);
-  assert (backend_valid_range (b, offset, count));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (c->can_write == 1);
+  assert (c->can_zero > NBDKIT_ZERO_NONE);
+  assert (backend_valid_range (c, offset, count));
   assert (!(flags & ~(NBDKIT_FLAG_MAY_TRIM | NBDKIT_FLAG_FUA |
                       NBDKIT_FLAG_FAST_ZERO)));
   if (fua)
-    assert (h->can_fua > NBDKIT_FUA_NONE);
+    assert (c->can_fua > NBDKIT_FUA_NONE);
   if (fast)
-    assert (h->can_fast_zero == 1);
+    assert (c->can_fast_zero == 1);
   datapath_debug ("%s: zero count=%" PRIu32 " offset=%" PRIu64
                   " may_trim=%d fua=%d fast=%d",
                   b->name, count, offset,
                   !!(flags & NBDKIT_FLAG_MAY_TRIM), fua, fast);
 
-  r = b->zero (b, h->handle, count, offset, flags, err);
+  if (c->can_zero == NBDKIT_ZERO_NATIVE)
+    r = b->zero (c, count, offset, flags, err);
+  else { /* NBDKIT_ZERO_EMULATE */
+    int writeflags = 0;
+    bool need_flush = false;
+
+    if (fast) {
+      *err = ENOTSUP;
+      return -1;
+    }
+
+    if (fua) {
+      if (c->can_fua == NBDKIT_FUA_EMULATE)
+        need_flush = true;
+      else
+        writeflags = NBDKIT_FLAG_FUA;
+    }
+
+    while (count) {
+      /* Always contains zeroes, but we can't use const or else gcc 9
+       * will use .rodata instead of .bss and inflate the binary size.
+       */
+      static /* const */ char buffer[MAX_REQUEST_SIZE];
+      uint32_t limit = MIN (count, MAX_REQUEST_SIZE);
+
+      if (limit == count && need_flush)
+        writeflags = NBDKIT_FLAG_FUA;
+
+      if (backend_pwrite (c, buffer, limit, offset, writeflags, err) == -1)
+        return -1;
+      offset += limit;
+      count -= limit;
+    }
+    r = 0;
+  }
+
   if (r == -1) {
     assert (*err);
     if (!fast)
@@ -687,22 +773,22 @@ backend_zero (struct backend *b,
 }
 
 int
-backend_extents (struct backend *b,
+backend_extents (struct context *c,
                  uint32_t count, uint64_t offset, uint32_t flags,
                  struct nbdkit_extents *extents, int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (h->can_extents >= 0);
-  assert (backend_valid_range (b, offset, count));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (c->can_extents >= 0);
+  assert (backend_valid_range (c, offset, count));
   assert (!(flags & ~NBDKIT_FLAG_REQ_ONE));
   datapath_debug ("%s: extents count=%" PRIu32 " offset=%" PRIu64 " req_one=%d",
                   b->name, count, offset, !!(flags & NBDKIT_FLAG_REQ_ONE));
 
-  if (h->can_extents == 0) {
+  if (c->can_extents == 0) {
     /* By default it is safe assume that everything in the range is
      * allocated.
      */
@@ -711,41 +797,42 @@ backend_extents (struct backend *b,
       *err = errno;
     return r;
   }
-  r = b->extents (b, h->handle, count, offset, flags, extents, err);
+  r = b->extents (c, count, offset, flags, extents, err);
   if (r == -1)
     assert (*err);
   return r;
 }
 
 int
-backend_cache (struct backend *b,
+backend_cache (struct context *c,
                uint32_t count, uint64_t offset,
                uint32_t flags, int *err)
 {
-  GET_CONN;
-  struct handle *h = get_handle (conn, b->i);
+  PUSH_CONTEXT_FOR_SCOPE (c);
+  struct backend *b = c->b;
   int r;
 
-  assert (h->handle && (h->state & HANDLE_CONNECTED));
-  assert (h->can_cache > NBDKIT_CACHE_NONE);
-  assert (backend_valid_range (b, offset, count));
+  assert (c->handle && (c->state & HANDLE_CONNECTED));
+  assert (c->can_cache > NBDKIT_CACHE_NONE);
+  assert (backend_valid_range (c, offset, count));
   assert (flags == 0);
   datapath_debug ("%s: cache count=%" PRIu32 " offset=%" PRIu64,
                   b->name, count, offset);
 
-  if (h->can_cache == NBDKIT_CACHE_EMULATE) {
+  if (c->can_cache == NBDKIT_CACHE_EMULATE) {
     static char buf[MAX_REQUEST_SIZE]; /* data sink, never read */
     uint32_t limit;
 
     while (count) {
       limit = MIN (count, sizeof buf);
-      if (backend_pread (b, buf, limit, offset, flags, err) == -1)
+      if (backend_pread (c, buf, limit, offset, flags, err) == -1)
         return -1;
       count -= limit;
+      offset += limit;
     }
     return 0;
   }
-  r = b->cache (b, h->handle, count, offset, flags, err);
+  r = b->cache (c, count, offset, flags, err);
   if (r == -1)
     assert (*err);
   return r;
