@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright (C) 2013-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -59,14 +59,16 @@ static void free_connection (struct connection *conn);
 /* Don't call these raw socket functions directly.  Use conn->recv etc. */
 static int raw_recv ( void *buf, size_t len);
 static int raw_send_socket (const void *buf, size_t len, int flags);
+#ifndef WIN32
 static int raw_send_other (const void *buf, size_t len, int flags);
-static void raw_close (void);
+#endif
+static void raw_close (int how);
 
-int
+conn_status
 connection_get_status (void)
 {
   GET_CONN;
-  int r;
+  conn_status r;
 
   if (conn->nworkers &&
       pthread_mutex_lock (&conn->status_lock))
@@ -78,11 +80,9 @@ connection_get_status (void)
   return r;
 }
 
-/* Update the status if the new value is lower than the existing value.
- * For convenience, return the incoming value.
- */
-int
-connection_set_status (int value)
+/* Update the status if the new value is lower than the existing value. */
+void
+connection_set_status (conn_status value)
 {
   GET_CONN;
 
@@ -90,19 +90,20 @@ connection_set_status (int value)
       pthread_mutex_lock (&conn->status_lock))
     abort ();
   if (value < conn->status) {
-    if (conn->nworkers && conn->status > 0) {
+    if (conn->nworkers && conn->status > STATUS_CLIENT_DONE) {
       char c = 0;
 
       assert (conn->status_pipe[1] >= 0);
       if (write (conn->status_pipe[1], &c, 1) != 1 && errno != EAGAIN)
         debug ("failed to notify pipe-to-self: %m");
     }
+    if (conn->status >= STATUS_CLIENT_DONE && value < STATUS_CLIENT_DONE)
+      conn->close (SHUT_WR);
     conn->status = value;
   }
   if (conn->nworkers &&
       pthread_mutex_unlock (&conn->status_lock))
     abort ();
-  return value;
 }
 
 struct worker_data {
@@ -123,7 +124,7 @@ connection_worker (void *data)
   threadlocal_set_conn (conn);
   free (worker);
 
-  while (!quit && connection_get_status () > 0)
+  while (!quit && connection_get_status () > STATUS_CLIENT_DONE)
     protocol_recv_request_send_reply ();
   debug ("exiting worker thread %s", threadlocal_get_name ());
   free (name);
@@ -175,7 +176,7 @@ handle_single_connection (int sockin, int sockout)
   if (!nworkers) {
     /* No need for a separate thread. */
     debug ("handshake complete, processing requests serially");
-    while (!quit && connection_get_status () > 0)
+    while (!quit && connection_get_status () > STATUS_CLIENT_DONE)
       protocol_recv_request_send_reply ();
   }
   else {
@@ -194,13 +195,13 @@ handle_single_connection (int sockin, int sockout)
 
       if (unlikely (!worker)) {
         perror ("malloc");
-        connection_set_status (-1);
+        connection_set_status (STATUS_DEAD);
         goto wait;
       }
       if (unlikely (asprintf (&worker->name, "%s.%d", plugin_name, nworkers)
                     < 0)) {
         perror ("asprintf");
-        connection_set_status (-1);
+        connection_set_status (STATUS_DEAD);
         free (worker);
         goto wait;
       }
@@ -210,7 +211,7 @@ handle_single_connection (int sockin, int sockout)
       if (unlikely (err)) {
         errno = err;
         perror ("pthread_create");
-        connection_set_status (-1);
+        connection_set_status (STATUS_DEAD);
         free (worker);
         goto wait;
       }
@@ -224,7 +225,7 @@ handle_single_connection (int sockin, int sockout)
 
   /* Finalize (for filters), called just before close. */
   lock_request ();
-  r = backend_finalize (top);
+  r = backend_finalize (conn->top_context);
   unlock_request ();
   if (r == -1)
     goto done;
@@ -238,9 +239,10 @@ static struct connection *
 new_connection (int sockin, int sockout, int nworkers)
 {
   struct connection *conn;
+#ifndef WIN32
   int opt;
   socklen_t optlen = sizeof opt;
-  struct backend *b;
+#endif
 
   conn = calloc (1, sizeof *conn);
   if (conn == NULL) {
@@ -254,16 +256,14 @@ new_connection (int sockin, int sockout, int nworkers)
   pthread_mutex_init (&conn->write_lock, NULL);
   pthread_mutex_init (&conn->status_lock, NULL);
 
-  conn->handles = calloc (top->i + 1, sizeof *conn->handles);
-  if (conn->handles == NULL) {
+  conn->default_exportname = calloc (top->i + 1,
+                                     sizeof *conn->default_exportname);
+  if (conn->default_exportname == NULL) {
     perror ("malloc");
     goto error1;
   }
-  conn->nr_handles = top->i + 1;
-  for_each_backend (b)
-    reset_handle (get_handle (conn, b->i));
 
-  conn->status = 1;
+  conn->status = STATUS_ACTIVE;
   conn->nworkers = nworkers;
   if (nworkers) {
 #ifdef HAVE_PIPE2
@@ -324,12 +324,14 @@ new_connection (int sockin, int sockout, int nworkers)
 
   return conn;
 
+#if defined(HAVE_PIPE2) || defined(HAVE_PIPE)
  error2:
+#endif
   if (conn->status_pipe[0] >= 0)
     close (conn->status_pipe[0]);
   if (conn->status_pipe[1] >= 0)
     close (conn->status_pipe[1]);
-  free (conn->handles);
+  free (conn->default_exportname);
 
  error1:
   pthread_mutex_destroy (&conn->request_lock);
@@ -348,7 +350,7 @@ free_connection (struct connection *conn)
   if (!conn)
     return;
 
-  conn->close ();
+  conn->close (SHUT_RDWR);
 
   /* Don't call the plugin again if quit has been set because the main
    * thread will be in the process of unloading it.  The plugin.unload
@@ -356,7 +358,10 @@ free_connection (struct connection *conn)
    */
   if (!quit) {
     lock_request ();
-    backend_close (top);
+    if (conn->top_context) {
+      backend_close (conn->top_context);
+      conn->top_context = NULL;
+    }
     unlock_request ();
   }
 
@@ -373,10 +378,9 @@ free_connection (struct connection *conn)
   free (conn->exportname_from_set_meta_context);
   free_interns ();
 
-  /* This is needed in order to free a field in struct handle. */
   for_each_backend (b)
-    reset_handle (get_handle (conn, b->i));
-  free (conn->handles);
+    free (conn->default_exportname[b->i]);
+  free (conn->default_exportname);
 
   free (conn);
   threadlocal_set_conn (NULL);
@@ -395,6 +399,7 @@ raw_send_socket (const void *vbuf, size_t len, int flags)
   ssize_t r;
   int f = 0;
 
+  assert (sock >= 0);
 #ifdef MSG_MORE
   if (flags & SEND_MORE)
     f |= MSG_MORE;
@@ -413,6 +418,7 @@ raw_send_socket (const void *vbuf, size_t len, int flags)
   return 0;
 }
 
+#ifndef WIN32
 /* Write buffer to conn->sockout with write() and either succeed completely
  * (returns 0) or fail (returns -1). flags is ignored.
  */
@@ -424,6 +430,7 @@ raw_send_other (const void *vbuf, size_t len, int flags)
   const char *buf = vbuf;
   ssize_t r;
 
+  assert (sock >= 0);
   while (len > 0) {
     r = write (sock, buf, len);
     if (r == -1) {
@@ -437,6 +444,7 @@ raw_send_other (const void *vbuf, size_t len, int flags)
 
   return 0;
 }
+#endif /* !WIN32 */
 
 /* Read buffer from conn->sockin and either succeed completely
  * (returns > 0), read an EOF (returns 0), or fail (returns -1).
@@ -485,12 +493,21 @@ raw_recv (void *vbuf, size_t len)
  * close, so this function ignores errors.
  */
 static void
-raw_close (void)
+raw_close (int how)
 {
   GET_CONN;
 
-  if (conn->sockin >= 0)
-    closesocket (conn->sockin);
-  if (conn->sockout >= 0 && conn->sockin != conn->sockout)
-    closesocket (conn->sockout);
+  if (conn->sockout >= 0 && how == SHUT_WR) {
+    if (conn->sockin == conn->sockout)
+      shutdown (conn->sockout, how);
+    else
+      closesocket (conn->sockout);
+    conn->sockout = -1;
+  }
+  else {
+    if (conn->sockin >= 0)
+      closesocket (conn->sockin);
+    if (conn->sockout >= 0 && conn->sockin != conn->sockout)
+      closesocket (conn->sockout);
+  }
 }
