@@ -55,7 +55,13 @@
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
-#ifdef HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED
+#if defined HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED && \
+  defined HAVE_GNUTLS_SOCKET_H
+#define TRY_KTLS 1
+#else
+#define TRY_KTLS 0
+#endif
+#if TRY_KTLS
 #include <gnutls/socket.h>
 #endif
 
@@ -406,7 +412,7 @@ crypto_send (const void *vbuf, size_t len, int flags)
  * close, so this function ignores errors.
  */
 static void
-crypto_close (void)
+crypto_close (int how)
 {
   GET_CONN;
   gnutls_session_t session = conn->crypto_session;
@@ -414,17 +420,21 @@ crypto_close (void)
 
   assert (session != NULL);
 
-  gnutls_transport_get_int2 (session, &sockin, &sockout);
+  if (how == SHUT_WR)
+    gnutls_bye (session, GNUTLS_SHUT_WR);
+  else {
+    gnutls_transport_get_int2 (session, &sockin, &sockout);
 
-  gnutls_bye (session, GNUTLS_SHUT_RDWR);
+    gnutls_bye (session, GNUTLS_SHUT_RDWR);
 
-  if (sockin >= 0)
-    closesocket (sockin);
-  if (sockout >= 0 && sockin != sockout)
-    closesocket (sockout);
+    if (sockin >= 0)
+      closesocket (sockin);
+    if (sockout >= 0 && sockin != sockout)
+      closesocket (sockout);
 
-  gnutls_deinit (session);
-  conn->crypto_session = NULL;
+    gnutls_deinit (session);
+    conn->crypto_session = NULL;
+  }
 }
 
 #ifdef WIN32
@@ -514,8 +524,15 @@ debug_x590_cert (gnutls_session_t session)
     gnutls_x509_crt_t cert;
     gnutls_datum_t cinfo;
 
-    gnutls_x509_crt_init (&cert);
-    gnutls_x509_crt_import (cert, &cert_list[i], GNUTLS_X509_FMT_DER);
+    /* This is for debugging; best-effort is okay */
+    ret = gnutls_x509_crt_init (&cert);
+    if (ret != 0)
+      continue;
+    ret = gnutls_x509_crt_import (cert, &cert_list[i], GNUTLS_X509_FMT_DER);
+    if (ret != 0) {
+      gnutls_x509_crt_deinit (cert);
+      continue;
+    }
 
     ret = gnutls_x509_crt_print (cert, GNUTLS_CRT_PRINT_ONELINE, &cinfo);
     if (ret == 0) {
@@ -535,7 +552,7 @@ debug_session (gnutls_session_t session)
   bool dhe = false, ecdh = false;
   int grp;
   const char *desc, *username, *hint;
-#ifdef HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED
+#if TRY_KTLS
   gnutls_transport_ktls_enable_flags_t ktls_enabled;
 #endif
 
@@ -545,7 +562,7 @@ debug_session (gnutls_session_t session)
   desc = gnutls_session_get_desc (session);
   if (desc) nbdkit_debug ("TLS session: %s", desc);
 
-#ifdef HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED
+#if TRY_KTLS
   ktls_enabled = gnutls_transport_is_ktls_enabled (session);
   switch (ktls_enabled) {
   case GNUTLS_KTLS_RECV:
