@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright (C) 2013-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,6 +43,7 @@
 #include "byte-swapping.h"
 #include "nbd-protocol.h"
 #include "protostrings.h"
+#include "strndup.h"
 
 /* Initial bound of client options we allow before giving up.
  * However, a client that issues NBD_OPT_LIST is permitted to follow
@@ -62,6 +63,8 @@ send_newstyle_option_reply (uint32_t option, uint32_t reply)
   fixed_new_option_reply.reply = htobe32 (reply);
   fixed_new_option_reply.replylen = htobe32 (0);
 
+  debug ("replying to %s with %s", name_of_nbd_opt (option),
+         name_of_nbd_rep (reply));
   if (conn->send (&fixed_new_option_reply,
                   sizeof fixed_new_option_reply, 0) == -1) {
     /* The protocol document says that the client is allowed to simply
@@ -194,6 +197,38 @@ send_newstyle_option_reply_info_str (uint32_t option, uint32_t reply,
   return 0;
 }
 
+/* Send reply containing NBD_INFO_BLOCK_SIZE. */
+static int
+send_newstyle_option_reply_info_block_size (uint32_t option, uint32_t reply,
+                                            uint16_t info,
+                                            uint32_t minimum,
+                                            uint32_t preferred,
+                                            uint32_t maximum)
+{
+  GET_CONN;
+  struct nbd_fixed_new_option_reply fixed_new_option_reply;
+  struct nbd_fixed_new_option_reply_info_block_size block_size;
+
+  fixed_new_option_reply.magic = htobe64 (NBD_REP_MAGIC);
+  fixed_new_option_reply.option = htobe32 (option);
+  fixed_new_option_reply.reply = htobe32 (reply);
+  fixed_new_option_reply.replylen = htobe32 (14);
+  block_size.info = htobe16 (info);
+  block_size.minimum = htobe32 (minimum);
+  block_size.preferred = htobe32 (preferred);
+  block_size.maximum = htobe32 (maximum);
+
+  if (conn->send (&fixed_new_option_reply,
+                  sizeof fixed_new_option_reply, SEND_MORE) == -1 ||
+      conn->send (&block_size,
+                  sizeof block_size, 0) == -1) {
+    nbdkit_error ("write: %s: %m", name_of_nbd_opt (option));
+    return -1;
+  }
+
+  return 0;
+}
+
 static int
 send_newstyle_option_reply_meta_context (uint32_t option, uint32_t reply,
                                          uint32_t context_id,
@@ -226,7 +261,7 @@ send_newstyle_option_reply_meta_context (uint32_t option, uint32_t reply,
 /* Sub-function during negotiate_handshake_newstyle, to uniformly handle
  * a client hanging up on a message boundary.
  */
-static int __attribute__ ((format (printf, 3, 4)))
+static int ATTRIBUTE_FORMAT_PRINTF(3, 4)
 conn_recv_full (void *buf, size_t len, const char *fmt, ...)
 {
   GET_CONN;
@@ -389,8 +424,16 @@ negotiate_handshake_newstyle_options (void)
      */
     if (tls == 2 && !conn->using_tls &&
         !(option == NBD_OPT_ABORT || option == NBD_OPT_STARTTLS)) {
+      if (option == NBD_OPT_EXPORT_NAME) {
+        debug ("newstyle negotiation: can't reply NBD_REP_ERR_TLS_REQD to %s",
+               name_of_nbd_opt (option));
+        return -1;
+      }
       if (send_newstyle_option_reply (option, NBD_REP_ERR_TLS_REQD))
         return -1;
+      if (conn_recv_full (data, optlen,
+                          "waiting for starttls: %m") == -1)
+        return -1;
       continue;
     }
 
@@ -430,6 +473,8 @@ negotiate_handshake_newstyle_options (void)
 
     case NBD_OPT_LIST:
       if (optlen != 0) {
+        debug ("ignoring request, client sent unexpected payload: %s",
+               name_of_nbd_opt (option));
         if (send_newstyle_option_reply (option, NBD_REP_ERR_INVALID)
             == -1)
           return -1;
@@ -458,6 +503,8 @@ negotiate_handshake_newstyle_options (void)
 
     case NBD_OPT_STARTTLS:
       if (optlen != 0) {
+        debug ("ignoring request, client sent unexpected payload: %s",
+               name_of_nbd_opt (option));
         if (send_newstyle_option_reply (option, NBD_REP_ERR_INVALID)
             == -1)
           return -1;
@@ -495,11 +542,14 @@ negotiate_handshake_newstyle_options (void)
           return -1;
         conn->using_tls = true;
         debug ("using TLS on this connection");
-        /* Wipe out any cached default export name. */
+        /* Wipe out any cached state. */
+        conn->structured_replies = false;
+        free (conn->exportname_from_set_meta_context);
+        conn->exportname_from_set_meta_context = NULL;
+        conn->meta_context_base_allocation = false;
         for_each_backend (b) {
-          struct handle *h = get_handle (conn, b->i);
-          free (h->default_exportname);
-          h->default_exportname = NULL;
+          free (conn->default_exportname[b->i]);
+          conn->default_exportname[b->i] = NULL;
         }
       }
       break;
@@ -568,24 +618,28 @@ negotiate_handshake_newstyle_options (void)
          */
         if (finish_newstyle_options (&exportsize,
                                      &data[4], exportnamelen) == -1) {
-          if (backend_finalize (top) == -1)
-            return -1;
-          backend_close (top);
+          if (conn->top_context) {
+            if (backend_finalize (conn->top_context) == -1)
+              return -1;
+            backend_close (conn->top_context);
+            conn->top_context = NULL;
+          }
           if (send_newstyle_option_reply (option, NBD_REP_ERR_UNKNOWN) == -1)
             return -1;
           continue;
         }
 
+        assert (conn->top_context);
         if (send_newstyle_option_reply_info_export (option,
                                                     NBD_REP_INFO,
                                                     NBD_INFO_EXPORT,
                                                     exportsize) == -1)
           return -1;
 
-        /* For now we send NBD_INFO_NAME and NBD_INFO_DESCRIPTION if
-         * requested, and ignore all other info requests (including
-         * NBD_INFO_EXPORT if it was requested, because we replied
-         * already above).
+        /* For now we send NBD_INFO_NAME, NBD_INFO_DESCRIPTION and
+         * NBD_INFO_BLOCK_SIZE if requested, and ignore all other info
+         * requests (including NBD_INFO_EXPORT if it was requested,
+         * because we replied already above).
          */
         for (i = 0; i < nrinfos; ++i) {
           memcpy (&info, &data[4 + exportnamelen + 2 + i*2], 2);
@@ -615,7 +669,7 @@ negotiate_handshake_newstyle_options (void)
             break;
           case NBD_INFO_DESCRIPTION:
             {
-              const char *desc = backend_export_description (top);
+              const char *desc = backend_export_description (conn->top_context);
 
               if (!desc) {
                 debug ("newstyle negotiation: %s: "
@@ -630,6 +684,29 @@ negotiate_handshake_newstyle_options (void)
                 return -1;
             }
             break;
+          case NBD_INFO_BLOCK_SIZE:
+            {
+              uint32_t minimum, preferred, maximum;
+
+              if (backend_block_size (conn->top_context,
+                                      &minimum, &preferred, &maximum) == -1)
+                return -1;
+              if (minimum == 0) {
+                debug ("newstyle negotiation: %s: "
+                       "NBD_INFO_BLOCK_SIZE: client requested but "
+                       "no plugin or filter provided block size information, "
+                       "ignoring client request",
+                       optname);
+                break;
+              }
+              if (send_newstyle_option_reply_info_block_size
+                  (option,
+                   NBD_REP_INFO,
+                   NBD_INFO_BLOCK_SIZE,
+                   minimum, preferred, maximum) == -1)
+                return -1;
+            }
+            break;
           default:
             debug ("newstyle negotiation: %s: "
                    "ignoring NBD_INFO_* request %u (%s)",
@@ -646,15 +723,18 @@ negotiate_handshake_newstyle_options (void)
         return -1;
 
       if (option == NBD_OPT_INFO) {
-        if (backend_finalize (top) == -1)
+        if (backend_finalize (conn->top_context) == -1)
           return -1;
-        backend_close (top);
+        backend_close (conn->top_context);
+        conn->top_context = NULL;
       }
 
       break;
 
     case NBD_OPT_STRUCTURED_REPLY:
       if (optlen != 0) {
+        debug ("ignoring request, client sent unexpected payload: %s",
+               name_of_nbd_opt (option));
         if (send_newstyle_option_reply (option, NBD_REP_ERR_INVALID)
             == -1)
           return -1;
@@ -699,7 +779,7 @@ negotiate_handshake_newstyle_options (void)
         /* Note that we support base:allocation whether or not the plugin
          * supports can_extents.
          */
-        if (!conn->structured_replies) {
+        if (option == NBD_OPT_SET_META_CONTEXT && !conn->structured_replies) {
           if (send_newstyle_option_reply (option, NBD_REP_ERR_INVALID)
               == -1)
             return -1;
