@@ -64,6 +64,8 @@
 
 #ifdef HAVE_LINUX_VM_SOCKETS_H
 #include <linux/vm_sockets.h>
+#elif HAVE_SYS_VSOCK_H
+#include <sys/vsock.h>
 #endif
 
 #ifdef HAVE_LIBSELINUX
@@ -173,18 +175,19 @@ bind_tcpip_socket (sockets *socks)
   struct addrinfo hints;
   struct addrinfo *a;
   int err, opt;
-  bool addr_in_use = false;
+  int saved_errno = 0;
 
   if (port == NULL)
     port = "10809";
 
   memset (&hints, 0, sizeof hints);
-  hints.ai_flags = AI_PASSIVE | AI_ADDRCONFIG;
+  hints.ai_flags = AI_PASSIVE;
+  hints.ai_family = tcpip_sock_af;
   hints.ai_socktype = SOCK_STREAM;
 
   err = getaddrinfo (ipaddr, port, &hints, &ai);
   if (err != 0) {
-    fprintf (stderr, "%s: getaddrinfo: %s: %s: %s",
+    fprintf (stderr, "%s: getaddrinfo: %s: %s: %s\n",
              program_name,
              ipaddr ? ipaddr : "<any>",
              port,
@@ -206,8 +209,20 @@ bind_tcpip_socket (sockets *socks)
     sock = set_cloexec (socket (a->ai_family, a->ai_socktype, a->ai_protocol));
 #endif
     if (sock == -1) {
-      perror ("bind_tcpip_socket: socket");
-      exit (EXIT_FAILURE);
+      if (errno == EAFNOSUPPORT) {
+        /* If ipv6.disable=1 was specified to the Linux kernel then
+         * getaddrinfo may still return AF_INET6 sockets but socket(2)
+         * will return this error.  I think it's safe to basically
+         * ignore this error.
+         */
+        saved_errno = errno;
+        debug ("bind_tcpip_socket: socket: %m (ignored)");
+        continue;
+      }
+      else {
+        perror ("bind_tcpip_socket: socket");
+        exit (EXIT_FAILURE);
+      }
     }
 
     opt = 1;
@@ -223,7 +238,8 @@ bind_tcpip_socket (sockets *socks)
 
     if (bind (sock, a->ai_addr, a->ai_addrlen) == -1) {
       if (errno == EADDRINUSE) {
-        addr_in_use = true;
+        saved_errno = errno;
+        debug ("bind_tcpip_socket: bind: %m (ignored)");
         closesocket (sock);
         continue;
       }
@@ -246,20 +262,23 @@ bind_tcpip_socket (sockets *socks)
 
   freeaddrinfo (ai);
 
-  if (socks->size == 0 && addr_in_use) {
-    fprintf (stderr, "%s: unable to bind to any sockets: %s\n",
-             program_name, strerror (EADDRINUSE));
+  if (socks->len == 0) {
+    fprintf (stderr, "%s: unable to bind to any TCP/IP sockets\n",
+             program_name);
+    if (saved_errno)
+      fprintf (stderr, "%s: socket error: %s\n",
+               program_name, strerror (saved_errno));
     exit (EXIT_FAILURE);
   }
 
   debug ("bound to IP address %s:%s (%zu socket(s))",
-         ipaddr ? ipaddr : "<any>", port, socks->size);
+         ipaddr ? ipaddr : "<any>", port, socks->len);
 }
 
 void
 bind_vsock (sockets *socks)
 {
-#ifdef AF_VSOCK
+#if defined(AF_VSOCK) && defined(VMADDR_CID_ANY)
   uint32_t vsock_port;
   int sock;
   struct sockaddr_vm addr;
@@ -274,10 +293,14 @@ bind_vsock (sockets *socks)
       exit (EXIT_FAILURE);
   }
 
-  /* Any platform with AF_VSOCK also supports SOCK_CLOEXEC so there is
-   * no fallback path.
-   */
+#ifdef SOCK_CLOEXEC
   sock = socket (AF_VSOCK, SOCK_STREAM|SOCK_CLOEXEC, 0);
+#else
+  /* Fortunately, this code is only run at startup, so there is no
+   * risk of the fd leaking to a plugin's fork()
+   */
+  sock = set_cloexec (socket (AF_VSOCK, SOCK_STREAM, 0));
+#endif
   if (sock == -1) {
     perror ("bind_vsock: socket");
     exit (EXIT_FAILURE);
@@ -399,7 +422,7 @@ accept_connection (int listen_sock)
   if (thread_data->sock == -1) {
     if (errno == EINTR || errno == EAGAIN)
       goto again;
-    perror ("accept");
+    nbdkit_error ("accept: %m");
     free (thread_data);
     return;
   }
@@ -418,7 +441,8 @@ accept_connection (int listen_sock)
   err = pthread_create (&thread, &attrs, start_thread, thread_data);
   pthread_attr_destroy (&attrs);
   if (unlikely (err != 0)) {
-    fprintf (stderr, "%s: pthread_create: %s\n", program_name, strerror (err));
+    errno = err;
+    nbdkit_error ("pthread_create: %m");
     closesocket (thread_data->sock);
     free (thread_data);
     return;
@@ -443,7 +467,7 @@ accept_connection (int listen_sock)
 static void
 check_sockets_and_quit_fd (const sockets *socks)
 {
-  const size_t nr_socks = socks->size;
+  const size_t nr_socks = socks->len;
   size_t i;
   int r;
 
@@ -489,11 +513,18 @@ check_sockets_and_quit_fd (const sockets *socks)
 static void
 check_sockets_and_quit_fd (const sockets *socks)
 {
-  const size_t nr_socks = socks->size;
+  const size_t nr_socks = socks->len;
   size_t i;
-  HANDLE h, handles[nr_socks+1];
+  HANDLE h;
+  CLEANUP_FREE HANDLE *handles = NULL;
   DWORD r;
 
+  handles = malloc ((nr_socks+1) * sizeof (HANDLE));
+  if (handles == NULL) {
+    perror ("malloc");
+    exit (EXIT_FAILURE);
+  }
+
   for (i = 0; i < nr_socks; ++i) {
     h = WSACreateEvent ();
     WSAEventSelect (_get_osfhandle (socks->ptr[i]), h,
@@ -552,7 +583,7 @@ accept_incoming_connections (const sockets *socks)
   }
   pthread_mutex_unlock (&count_mutex);
 
-  for (i = 0; i < socks->size; ++i)
+  for (i = 0; i < socks->len; ++i)
     closesocket (socks->ptr[i]);
   free (socks->ptr);
 }
