@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright (C) 2018-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,6 +43,8 @@
 
 #include "internal.h"
 
+#include "array-size.h"
+
 static bool error_flagged;
 
 /* Stubs for linking against minimal source files, and for proving
@@ -75,7 +77,13 @@ threadlocal_get_conn (void)
   abort ();
 }
 
-int
+struct context *
+threadlocal_get_context (void)
+{
+  abort ();
+}
+
+conn_status
 connection_get_status (void)
 {
   abort ();
@@ -87,9 +95,14 @@ backend_default_export (struct backend *b, int readonly)
   abort ();
 }
 
+int tls;
+
+/* Unit tests. */
+
 static bool
 test_nbdkit_parse_size (void)
 {
+  size_t i;
   bool pass = true;
   struct pair {
     const char *str;
@@ -156,7 +169,7 @@ test_nbdkit_parse_size (void)
     { "1E", 1024LL * 1024 * 1024 * 1024 * 1024 * 1024 },
   };
 
-  for (size_t i = 0; i < sizeof pairs / sizeof pairs[0]; i++) {
+  for (i = 0; i < ARRAY_SIZE (pairs); i++) {
     int64_t r;
 
     error_flagged = false;
