@@ -39,8 +39,8 @@ requires_filter cache
 requires test -d /proc/self/fd
 requires qemu-io --version
 # Need the stat, cut, tr commands from coreutils.
-requires stat --version
-requires cut --version
+requires $STAT --version
+requires $CUT --version
 requires tr --version
 
 sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
@@ -56,7 +56,7 @@ TMPDIR=$d/cache
 export TMPDIR
 
 # Create an empty base image.
-truncate -s 1G $d/cache-max-size.img
+$TRUNCATE -s 1G $d/cache-max-size.img
 
 # Run nbdkit with the caching filter and a low size limit to ensure
 # that the reclaim code is exercised.
@@ -77,11 +77,11 @@ qemu-io -f raw "nbd+unix://?socket=$sock" \
 # deleted by the filter, and so is only accessible via /proc/$pid/fd.
 # Get the /proc link to the cache file, and the size of it in bytes.
 fddir="/proc/$( cat $d/cache-max-size.pid )/fd"
-ls -l $fddir ||:
-fd="$fddir/$( ls -l $fddir | grep $TMPDIR/ | head -1 |
-              tr -s ' ' | cut -d ' ' -f 9 )"
-stat -L $fd
-size=$(( $(stat -L -c '%b * %B' $fd) ))
+LANG=C ls -ln $fddir ||:
+fd="$fddir/$( LANG=C ls -ln $fddir | grep $TMPDIR/ | head -1 |
+              tr -s ' ' | $CUT -d ' ' -f 9 )"
+$STAT -L $fd
+size=$(( $($STAT -L -c '%b * %B' $fd) ))
 
 if [ "$size" -gt $(( 11 * 1024 * 1024 )) ]; then
     echo "$0: cache size is larger than 10M (actual size: $size bytes)"
