@@ -51,7 +51,7 @@ fi
 requires_plugin cc
 requires $CXX --version
 requires_nbdsh_uri
-requires nbdinfo --version
+requires_nbdinfo
 
 out=test-cc-cpp.out
 cleanup_fn rm -f $out
