@@ -38,6 +38,7 @@ source ./functions.sh
 set -e
 set -x
 
+requires_run
 requires jq --version
 requires qemu-img --version
 requires qemu-img map --help
@@ -84,34 +85,6 @@ cat > $expected <<'EOF'
 EOF
 do_test "1" 1M
 
-# Allocated zeroes at the start of the disk.  This should create a
-# zero extent (data=true zero=true).
-cat > $expected <<'EOF'
-{"start":0,"length":32768,"data":true,"zero":true}
-{"start":32768,"length":1015808,"data":false,"zero":true}
-EOF
-do_test "0" 1M
-
-# Completely zero disk.
-cat > $expected <<'EOF'
-{"start":0,"length":32768,"data":true,"zero":true}
-EOF
-do_test "0 0 0" 32K
-
-# Allocated data and zero extents in a few places in the middle.
-cat > $expected <<'EOF'
-{"start":0,"length":32768,"data":false,"zero":true}
-{"start":32768,"length":32768,"data":true,"zero":false}
-{"start":65536,"length":32768,"data":true,"zero":true}
-{"start":98304,"length":32768,"data":false,"zero":true}
-{"start":131072,"length":32768,"data":true,"zero":false}
-{"start":163840,"length":98304,"data":false,"zero":true}
-{"start":262144,"length":32768,"data":true,"zero":true}
-{"start":294912,"length":32768,"data":true,"zero":false}
-{"start":327680,"length":720896,"data":false,"zero":true}
-EOF
-do_test "@32768 1 @65536 0 @131072 1 @262144 0 @294912 1" 1M
-
 # This should test coalescing in the extents code.
 cat > $expected <<'EOF'
 {"start":0,"length":32768,"data":false,"zero":true}
