@@ -42,9 +42,14 @@ requires jq --version
 requires qemu-img --version
 requires qemu-img map --help
 
+# Because of macOS SIP misfeature the DYLD_* environment variable
+# added by libnbd/run is filtered out and the test won't work.  Skip
+# it entirely on Macs.
+requires_not test "$(uname)" = "Darwin"
+
 out="test-nbd-extents.out"
 expected="test-nbd-extents.expected"
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 pid1="test-nbd-extents.pid1"
 pid2="test-nbd-extents.pid2"
 pid3="test-nbd-extents.pid3"
