@@ -37,9 +37,9 @@
 source ./functions.sh
 set -e
 
-requires nbdsh --version
+requires_nbdsh_uri
 
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="pattern-largest.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
