@@ -34,7 +34,9 @@ source ./functions.sh
 set -e
 set -x
 
-requires nbdsh --base-allocation -c 'exit(not h.supports_uri())'
+requires_plugin sh
+requires_nbdsh_uri
+requires nbdsh --base-allocation
 
 files="retry-extents-count retry-extents-open-count"
 rm -f $files
