@@ -36,7 +36,9 @@ source ./functions.sh
 set -e
 set -x
 
-requires nbdsh --base-allocation -c 'exit(not h.supports_uri())'
+requires_plugin eval
+requires_nbdsh_uri
+requires nbdsh --base-allocation
 
 files="swab-extents.out swab-extents.exp8 swab-extents.exp16
        swab-extents.exp32 swab-extents.exp64"
