@@ -71,31 +71,38 @@
 #include <limits.h>
 #include <time.h>
 
+#include "const-string-vector.h"
 #include "options.h"
+#include "windows-compat.h"
 #include "utils.h"
 
-/* Construct an array of parameters passed through to real nbdkit. */
-static const char **cmd;
-static size_t len;
-
-/* Plugins written in scripting languages (only Perl right now) need
- * to be rewritten on the command line in a different way from plugins
- * written in C.  So we have to list those here.
+/* Plugins written in scripting languages need to be rewritten on the
+ * command line in a different way from plugins written in C.  So we
+ * have to list those here, and return the language plugin needed to
+ * run them.
  */
 static bool
-is_perl_plugin (const char *name)
+is_script_plugin (const char *name, const char **language)
 {
-  return strcmp (name, "example4") == 0;
+  if (strcmp (name, "example4") == 0) {
+    *language = "perl";
+    return true;
+  }
+  if (strcmp (name, "S3") == 0) {
+    *language = "python";
+    return true;
+  }
+  return false;
 }
 
+/* Construct an array of parameters passed through to real nbdkit. */
+static const_string_vector cmd;
+
 static void
 passthru (const char *s)
 {
-  cmd = realloc (cmd, (len+1) * sizeof (const char *));
-  if (cmd == NULL)
+  if (const_string_vector_append (&cmd, s) == -1)
     abort ();
-  cmd[len] = s;
-  ++len;
 }
 
 static void __attribute__((format (printf, 1, 2)))
@@ -122,25 +129,73 @@ print_command (void)
 {
   size_t i;
 
-  if (len > 0)
-    shell_quote (cmd[0], stderr);
-  for (i = 1; i < len && cmd[i] != NULL; ++i) {
+  if (cmd.len > 0)
+    shell_quote (cmd.ptr[0], stderr);
+  for (i = 1; i < cmd.len && cmd.ptr[i] != NULL; ++i) {
     fputc (' ', stderr);
-    shell_quote (cmd[i], stderr);
+    shell_quote (cmd.ptr[i], stderr);
   }
   fputc ('\n', stderr);
 }
 
+#ifdef WIN32
+/* Windows behaviour of _spawnvp is completely insane:
+ * https://stackoverflow.com/questions/4146980/how-to-avoid-space-splitting-and-quote-removal-with-spawnvp
+ * See also glib sources where it's described as "really weird".
+ */
+static const char *
+quote_string_for_spawn (const char *str)
+{
+  size_t i, len, pb;
+  char *p, *ret = (char *) str;
+
+  if (*str == 0 || strchr (str, ' ') || strchr (str, '\t')) {
+    len = strlen (str);
+
+    p = ret = malloc (2 + len*2 + 1);
+    if (ret == NULL) {
+      perror ("malloc");
+      exit (EXIT_FAILURE);
+    }
+
+    *p++ = '"';
+    pb = 0;
+    for (i = 0; i < len; ++i) {
+      switch (str[i]) {
+      case '"':
+        *p++ = '\\';
+        for (; pb > 0; --pb)
+          *p++ = '\\';
+        *p++ = '"';
+        break;
+      case '\\':
+        pb++;
+        *p++ = str[i];
+        break;
+      default:
+        pb = 0;
+        *p++ = str[i];
+      }
+    }
+    for (; pb > 0; --pb)
+      *p++ = '\\';
+    *p++ = '"';
+    *p++ = '\0';
+  }
+
+  /* We never free these strings. */
+  return ret;
+}
+#endif /* WIN32 */
+
 int
 main (int argc, char *argv[])
 {
   bool verbose = false;
   char *s;
-  time_t t;
-  unsigned tu;
-  char ts[32];
   int r;
 
+#ifndef WIN32
   /* If NBDKIT_VALGRIND=1 is set in the environment, then we run the
    * program under valgrind.  This is used by the tests.  Similarly if
    * NBDKIT_GDB=1 is set, we run the program under GDB, useful during
@@ -159,6 +214,14 @@ main (int argc, char *argv[])
     passthru ("--num-callers=100");
     /* This is a temporary workaround until RHBZ#1662656 is fixed: */
     passthru ("--read-inline-info=no");
+
+    /* Don't invoke malloc debugging when we are valgrinding because
+     * it duplicates work done by valgrind and might even hide issues.
+     *
+     * Originally this was a workaround for:
+     * https://sourceware.org/bugzilla/show_bug.cgi?id=28256
+     */
+    unsetenv ("GLIBC_TUNABLES");
   }
   else {
     s = getenv ("NBDKIT_GDB");
@@ -167,9 +230,17 @@ main (int argc, char *argv[])
       passthru ("--args");
     }
   }
+#endif
 
   /* Needed for plugins written in OCaml. */
-  s = getenv ("LD_LIBRARY_PATH");
+#if defined(WIN32)
+#define LD_LIBRARY_PATH "PATH"
+#elif defined(__APPLE__)
+#define LD_LIBRARY_PATH "DYLD_LIBRARY_PATH"
+#else
+#define LD_LIBRARY_PATH "LD_LIBRARY_PATH"
+#endif
+  s = getenv (LD_LIBRARY_PATH);
   if (s)
     r = asprintf (&s, "%s/plugins/ocaml/.libs:%s", builddir, s);
   else
@@ -178,7 +249,7 @@ main (int argc, char *argv[])
     perror ("asprintf");
     exit (EXIT_FAILURE);
   }
-  setenv ("LD_LIBRARY_PATH", s, 1);
+  setenv (LD_LIBRARY_PATH, s, 1);
   free (s);
   s = getenv ("LIBRARY_PATH");
   if (s)
@@ -193,7 +264,7 @@ main (int argc, char *argv[])
   free (s);
 
   /* Absolute path of the real nbdkit command. */
-  passthru_format ("%s/server/nbdkit", builddir);
+  passthru_format ("%s/server/nbdkit" EXEEXT, builddir);
 
   /* Option parsing.  We don't really parse options here.  We are only
    * interested in which options have arguments and which need
@@ -225,7 +296,8 @@ main (int argc, char *argv[])
     /* Filters can be rewritten if they are a short name. */
     else if (c == FILTER_OPTION) {
       if (is_short_name (optarg))
-        passthru_format ("--filter=%s/filters/%s/.libs/nbdkit-%s-filter.so",
+        passthru_format ("--filter="
+                         "%s/filters/%s/.libs/nbdkit-%s-filter." SOEXT,
                          builddir, optarg, optarg);
       else
         passthru_format ("--filter=%s", optarg);
@@ -256,15 +328,20 @@ main (int argc, char *argv[])
      * short name then rewrite it.
      */
     if (is_short_name (argv[optind])) {
-      /* Special plugins written in Perl. */
-      if (is_perl_plugin (argv[optind])) {
-        passthru_format ("%s/plugins/perl/.libs/nbdkit-perl-plugin.so",
-                         builddir);
+      const char *language;
+
+      /* Plugins written in scripting languages. */
+      if (is_script_plugin (argv[optind], &language)) {
+        passthru_format ("%s/plugins/%s/.libs/nbdkit-%s-plugin." SOEXT,
+                         builddir, language, language);
         passthru_format ("%s/plugins/%s/nbdkit-%s-plugin",
                          builddir, argv[optind], argv[optind]);
       }
+      /* Otherwise normal plugins written in C or other languages that
+       * compile to .so files.
+       */
       else {
-        passthru_format ("%s/plugins/%s/.libs/nbdkit-%s-plugin.so",
+        passthru_format ("%s/plugins/%s/.libs/nbdkit-%s-plugin." SOEXT,
                          builddir, argv[optind], argv[optind]);
       }
       ++optind;
@@ -281,21 +358,20 @@ main (int argc, char *argv[])
   if (verbose)
     print_command ();
 
-  /* This is a cheap way to find some use-after-free and uninitialized
-   * read problems when using glibc, and doesn't affect normal
-   * operation or other libc.  We don't overwrite existing values so
-   * this can be disabled or overridden at runtime.
-   */
-  setenv ("MALLOC_CHECK_", "1", 0);
-  time (&t);
-  tu = t;
-  tu %= 255;
-  tu++;
-  snprintf (ts, sizeof ts, "%u", tu);
-  setenv ("MALLOC_PERTURB_", ts, 0);
-
   /* Run the final command. */
-  execvp (cmd[0], (char **) cmd);
-  perror (cmd[0]);
+#ifndef WIN32
+  execvp (cmd.ptr[0], (char **) cmd.ptr);
+  perror (cmd.ptr[0]);
   exit (EXIT_FAILURE);
+#else /* WIN32 */
+  size_t i;
+  for (i = 1; cmd.ptr[i] != NULL; ++i)
+    cmd.ptr[i] = quote_string_for_spawn (cmd.ptr[i]);
+  r = _spawnvp (_P_WAIT, cmd.ptr[0], cmd.ptr);
+  if (r == -1) {
+    perror (cmd.ptr[0]);
+    exit (EXIT_FAILURE);
+  }
+  exit (r == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
+#endif /* WIN32 */
 }
